// cfpaccessor.h
// this file contains accessor classes

// Start file
#ifndef INC_cfpaccessorh
#define INC_cfpaccessorh

#include "cfpinc.h"
#include "cfpforwards.h"
#include "matrixinc.h"

//Not implemented in gcc yet (2.8.1)
//using namespace std;
//namespace Configpp{







// Enumerator for the ProxyTrigger
typedef enum ProxyTriggerType
  {
    inputtype,
    triggertype,
    outputtype
  };






//--------------------------------------------------------------------------------
//---- CfpAccessor ---------------------------------------------------------------
//--------------------------------------------------------------------------------
//: The base class for all Accesor objects
// an accessor object implements a getter, a setter, and a validator, which are used
// to manipulate values of a meber variable or function.
class CfpAccessor
{
  friend class CfpMember;
protected:
  string name;
public:
  CfpAccessor(string inname) {name=inname;};					// constructor
  string getname() {return name;};						// accessor
};





//--------------------------------------------------------------------------------
//---- CfpVAccessor ---------------------------------------------------------------
//--------------------------------------------------------------------------------
//: The base class for arg and variable Accesor objects
// an accessor object implements a getter, a setter, and a validator, which are used
// to manipulate values of a variable.

class CfpVAccessor : public CfpAccessor
{
protected:
  CfpVal *valp;
  bool valislocal;
  int listenercount;
  int speakercount;
  bool secretchanges;
protected:
  CfpMemberVA *memberp;
  CfpObject *objectp;
  list<CfpVAccstub *> listeners;
  list<CfpVAccstub *> speakers;
public:
  CfpVAccessor(string inname,CfpObject *objp=NULL,CfpVal *invalp=NULL);		// constructor
  ~CfpVAccessor();								// destructor
public:
  string getname() {return name;};				// accessor
  string getlabel();						// accessor
  string getdesc();						// accessor
  string getclassname();					// accessor
  string getunitstr();						// accessor
  string getobjname();						// accessor
  string getobjtrail();						// accessor
  CfpObject *getobjp() {return objectp;};                       // accessor
  virtual bool isdefined();					// accessor
  CfpVAccstub *addlistener(CfpVAccessor *vlistenerp,int fromindex=-1, int toindex=-1);		// add a listener which gets set on changes to us
  void addspeaker(CfpVAccstub *astub);		// keep track of who we listen to
  void correctspeaker(CfpVAccstub *oldstub,CfpVAccstub *newstub);
  void notify(CfpVal *outvalp);	              			// notify listeners (usually called by changes())
  void checkspeakers();	              			// get latest values from speakers
  virtual void changes(CfpVal *outvalp) {notify(outvalp);};	// the value has changed
  virtual void changes() {notify(valp);};			// the value has changed;
  virtual void changes(int index) {changes();};
public:
  virtual void set(CfpVal *val) {*valp=*val;changes();};
  virtual void set(CfpVal val) {set (&val);};
  virtual void set(string val) {valp->set_string(val);changes();};
  virtual CfpVal get(){return *valp;};
//  virtual CfpVal* getp(){return valp;};
  virtual void update() {set(valp);};                           // set it to itself (just a quick way to get it to invoke changes);
public:
  virtual void set(CfpVal val,int index) {set(&val,index);};    // vector element set;
  virtual void set(CfpVal *valp,int index);                      // vector element set;
  virtual CfpVal get(int index);                                // vector element get;
  virtual int parseindex(string vectoradstring);                // return the index into the vector
public:
  virtual string getstr();					// return a string version of the accessible value
  virtual const char *getcharp();				// return a const char* to string version of value;
public:
  virtual bool getable() {return false;};
  virtual bool setable() {return false;};
//  virtual bool willnotify() {return true;};
  virtual bool willnotify() {return false;};
  virtual bool canchangesecretly() {return secretchanges;};
  void latestupdate();
  void retrieveandset(int index=-1);
};



//--------------------------------------------------------------------------------
//---- CfpVAccstub ---------------------------------------------------------------
//--------------------------------------------------------------------------------
//: stub pointing to accessors (with indexes)
class CfpVAccstub
{
private:
  CfpVAccessor *acp,*sourceacp;
  int fromindex,toindex;
public:
  CfpVAccstub(CfpVAccessor *insourceacp,CfpVAccessor *inacp,int infromindex,int intoindex) {sourceacp=insourceacp;acp=inacp;fromindex=infromindex;toindex=intoindex;};
public:
  string getname() {return acp->getname();};
  CfpVAccessor *getsource() {return sourceacp;};
  int getfromindex() {return fromindex;};
  int gettoindex() {return toindex;};
  void set(CfpVal *invalp) {acp->set(invalp,toindex);};
  bool retrieveandset();
  void latestupdate();
public:
  CfpVAccstub *listenersplice(CfpVAccessor *inacp,int intoindex,CfpVAccessor *outacp,int outtoindex);  // splice in a filters
};




class CfpTriggerP
{
private:
  CfpVAccessor *accessorp;
  ProxyTriggerType ptype;
  int index;
public:
  CfpTriggerP(CfpVAccessor *inacp,ProxyTriggerType inptype, int inindex=-1) {accessorp=inacp;ptype=inptype;index=inindex;};
  CfpVAccessor *getaccessorp(){return accessorp;};
  ProxyTriggerType getptype() {return ptype;};
  int getindex(){return index;};
  bool isinput() {if (ptype==inputtype) return true; else return false;};
  bool istrigger() {if (ptype==triggertype) return true; else return false;};
  bool isoutput() {if (ptype==outputtype) return true; else return false;};
};

//--------------------------------------------------------------------------------
//---- CfpAccTriggerProxy --------------------------------------------------------
//--------------------------------------------------------------------------------
//: keeps a list of accessor and triggers and makes sure they are updated before triggering
class CfpVAccTriggerProxy
{
private:
  CfpVAccessor *proxytrigger;
  list<CfpTriggerP *> accessors;
public:
  CfpVAccTriggerProxy(string name);
  ~CfpVAccTriggerProxy();
public:
  CfpVAccessor *getproxytriggerp(){return proxytrigger;};
public:
  void addaccessor(CfpVAccessor *accp,ProxyTriggerType inptype,int indindex=-1);
  void activate(CfpVal val);
};


























//--------------------------------------------------------------------------------
//---- CfpValAccessor ------------------------------------------------------------
//--------------------------------------------------------------------------------
//: The CfpVal accessor, used notably for automatic arguments
// This uses a local copy of a CfpVal to hold all values (ie no external storage is used)

class CfpValAccessor : public CfpVAccessor
{
public:
  CfpValAccessor(string inname,CfpObject *objp,CfpVal *invalp=NULL)
    :CfpVAccessor(inname,objp,invalp){;};
public:
  virtual bool getable() {return true;};
  virtual bool setable() {return true;};
};







//--------------------------------------------------------------------------------
//---- CfpFixedValAccessor --------------------------------------------------------------
//--------------------------------------------------------------------------------
//:The <ValType> accessor object
// this accessor just stores an initial value, and when it is triggered (set)
// it notifies all its listeners. (useful for things like menu item activations)

template<class ValType> class CfpFixedValAccessor: public CfpVAccessor
{
public:
  CfpFixedValAccessor(string inname,CfpObject *objp,ValType invalue)
    :CfpVAccessor(inname,objp) {*valp=(CfpVal)invalue;};
public:
  void set(CfpVal *val) {changes();};
public:
  virtual bool getable() {return true;};
  virtual bool setable() {return true;};
};






//--------------------------------------------------------------------------------
//---- CfpVTAccessor -------------------------------------------------------------
//--------------------------------------------------------------------------------
//: The <ValType> accessor object
// For creating accessors to variables through pointers
// In this case, instead of passing on any value that we are told to set to,
//  we pass on our fixed argument.

template<class ValType> class CfpVTAccessor: public CfpVAccessor
{
private:
  ValType *avalp;
public:
  CfpVTAccessor(string inname,CfpObject *objp,ValType *inavalp)
    :CfpVAccessor(inname,objp) {avalp=inavalp;};
public:
  void set(CfpVal *val) {*avalp=(ValType)*val;changes();};
  void update() {CfpVal v2=*avalp;*valp=*avalp;if (*valp!=v2) changes();};
  CfpVal get() {*valp=*avalp;return *valp;};
public:
  virtual bool getable() {return true;};
  virtual bool setable() {return true;};
};





//--------------------------------------------------------------------------------
//---- CfpVTAccessorarg --------------------------------------------------------------
//--------------------------------------------------------------------------------
//: The <ValType> accessor object
// For creating accessors to variables through pointers
// In this case, instead of passing on any value that we are told to set to,
//  we pass on our fixed argument.

template<class ValType> class CfpVTAccessorarg: public CfpVAccessor
{
private:
  ValType *avalp;
  ValType argval;
public:
  CfpVTAccessorarg(string inname,CfpObject *objp,ValType *inavalp,ValType inargval)
    :CfpVAccessor(inname,objp) {avalp=inavalp;argval=inargval;*valp=CfpVal(inargval);};
public:
  void set(CfpVal *val) {*avalp=argval;changes();};
public:
  virtual bool getable() {return true;};
  virtual bool setable() {return true;};
};





















































//--------------------------------------------------------------------------------
//---- CfpVectorAccessor ---------------------------------------------------------
//--------------------------------------------------------------------------------
//: The CfpVector accessor, base class used for vector accessrs

class CfpVectorAccessor : public CfpVAccessor
{
protected:
  int veclength;
  int selectedindex;
  bool isdefinedflag;
public:
  CfpVectorAccessor(string inname,CfpObject *objp,int inlength)
  : CfpVAccessor(inname,objp)
    {
      // constructor for the vector accessor;
      veclength=inlength;
      selectedindex=-1;
      isdefinedflag=false;
    }
  
  CfpVectorAccessor(string inname,CfpObject *objp,int inlength,string initstring)
  : CfpVAccessor(inname,objp)
    {
      // constructor for the vector accessor;
      isdefinedflag=false;
      veclength=inlength;
      selectedindex=-1;
      set(initstring);
    }

  ~CfpVectorAccessor()
    {
      // destructor for the vector accessor;
    }

public:
  virtual CfpVal get(int index)=0;                                  // vector element set;
  virtual void set(CfpVal *valp,int index)=0;                       // vector element set;
public:
  virtual void set(string valstr,int index)
    {
      CfpVal tempsv(valstr);
      set(&tempsv,index);
    }

public:
  virtual void set(string invecval)
  {
    // set full vector using a string;
    string tokeval;
    int depth=0;
    string vecval=invecval;
    int curelement=0;
    bool lastwastoken=false;
    // we expect the string to be in the form "[10 20 30]"  or "[10,20,30]";
    // should we allow the absence of open and close brackets? here, yes, although that says nothing about the language allowing it.;

    for (;;)
      {
	// grab a token;
	tokeval=JRstriptoken(vecval);
	if (tokeval=="")
	  {
	    // no more tokens;
	    break;
	  }
	else if (tokeval=="[")
	  {
	    ++depth;
	    if (depth>1)
	      {
		// currently we allow only one dimensional vectors so this is an error;
		throw CfpError("syntax error in assignment of vector '"+getname()+"' to '" +invecval+"'.","",syntactical,false,false);
	      }
	    lastwastoken=false;
	  }
	else if (tokeval=="]")
	  {
	    --depth;
	    if (depth!=0)
	      {
		// currently we allow only one dimensional vectors so this is an error;
		throw CfpError("syntax error in assignment of vector '"+getname()+"' to '" +invecval+"'.","",syntactical,false,false);
	      }
	    lastwastoken=false;
	  }
	else if (depth==0)
	  {
	    // there shouldnt be any other tokens then the above at depth 0 so this is an error;
	    throw CfpError("syntax error in assignment of vector '"+getname()+"' to '" +invecval+"' (past array bounds).","",syntactical,false,false);
	  }
	else if (tokeval==",")
	  {
	    // if the last value was not a token, then we treat this as saying skip the current element (no assignment);
	    if (!lastwastoken)
	      {
		if (++curelement>=veclength)
		  {
		    // we went past array boundry;
		    throw CfpError("syntax error in assignment of vector '"+getname()+"' to '" +invecval+"' (past array bounds).","",syntactical,false,false);
		  }
	      }
	    else
	      {
		// its just a comma after a token, ignore it.;
	      }
	    lastwastoken=false;
	  }
	else
	  {
	    // okay, it must be a token, so set it and advance the element index;
	    if (curelement>=veclength)
	      {
		// we went past array boundry;
		throw CfpError("syntax error in assignment of vector '"+getname()+"' to '" +invecval+"' (past array bounds).","",syntactical,false,false);
	      }
	    // now set it using element string set;
	    set(tokeval,curelement);
	    // advance the counter;
	    ++curelement;
	    lastwastoken=true;
	  }
      }
    if (depth!=0)
      throw CfpError("syntax error in assignment of vector '"+getname()+"' to '" +invecval+"' (unmatched []).","",syntactical,false,false);
    if (curelement!=veclength)
      throw CfpError("syntax error in assignment of vector '"+getname()+"' to '" +invecval+"' (vector size to small).","",syntactical,false,false);
    // now notify the accessor listeners of changes;
    changes(-1);
  }
  
  virtual string getstr()					// return a string version of the vector;
  {
    // return a string version of the vector;
    // string should be in the form "[10,20,30]";
    int count;
    string vecstring="[  ";
    CfpVal tempsv;

    for (count=0;count<veclength;++count)
      {
	tempsv=get(count);
	vecstring.append(tempsv.get_string()+" ");
      }
    vecstring+=" ]";
    return vecstring;
  }


public:
  void checkrange(int index)
  {
    if (index<0 || index>=veclength)
      {
	throw CfpError("attempt to access element " + JRtostr(index) +" of vector '"+getname()+"', when vector is of length "+ JRtostr(veclength)+".","",classerr,false,false);
      }
  }

  void selectindex(int inindex)
  {
    // choose the index which will be assumed if none is specified in subsequent sets/gets;
    checkrange(inindex);
    selectedindex=inindex;
  }

  void set(CfpVal *valp)
  {
    // a suspicious set, since this is a vector and no address was specified.;
    if (selectedindex!=-1)
      {
	// this accessor is set to default to a specific index;
	// assume that they are refering to the selectedindex;
	set (valp,selectedindex);
      }
    else
      {
	// assume they want to set the entire vector using a string;
	string tempstr;

	// interesting compiler bug;
	//  if we try to combine the two statemets below (ie "set(valp->get_string())" then if an error is thrown,
	//  the system core dumps.  looks like the exception stack is getting messed up somewhere.
	tempstr=valp->get_string();
	set (tempstr);
      }
  }

 CfpVal get()           // vector get of selectedindex;
  {
    // a suspicious set, since this is a vector and no address was specified.;
    if (selectedindex!=-1)
      {
	// this accessor is set to default to a specific index;
	// assume that they are refering to the selectedindex;
	return get(selectedindex);
      }
    else
      {
	// assume they want to get the entire vector as a string;
	return CfpVal(getstr());
      }
  }
  
  virtual int parseindex(string vecval)
    {
      // return the index into the vector specified by string (currently we only use single dimension vectors so no big deal);
      int depth=0;
      int dimad,dimmult;
      int index=0;
      string tokeval;
      string invecval=vecval;
      bool lastwastoken;
      int curelement=0;
 
      for (;;)
	{
	  // grab a token;
	  tokeval=JRstriptoken(vecval);

	  if (tokeval=="")
	    {
	      // no more tokens
		   break;
	    }
	  else if (tokeval=="[")
	    {
	      ++depth;
	      if (depth>1)
		{
		  // currently we allow only one dimensional vectors so this is an error;
		  throw CfpError("syntax error in address of vector '"+getname()+"' specified as '" +invecval+"'.","",syntactical,false,false);
		}
	      lastwastoken=false;
	    }
	  else if (tokeval=="]")
	    {
	      --depth;
	      if (depth!=0)
		{
		  // currently we allow only one dimensional vectors so this is an error;
		  throw CfpError("syntax error in address of vector '"+getname()+"' specified as '" +invecval+"'.","",syntactical,false,false);
		}
	      lastwastoken=false;
	    }
	  else if (depth==0)
	    {
	      // there shouldnt be any other tokens then the above at depth 0 so this is an error;
	      throw CfpError("syntax error in address of vector '"+getname()+"' specified as '" +invecval+"' (dimensions should be in [] brackets).","",syntactical,false,false);
	    }
	  else if (tokeval==",")
	    {
	      // if the last value was not a token, then we treat this as saying skip the current element (no assignment);
	      if (!lastwastoken)
		{
		  if (++curelement>=1)
		    {
		      // we went past array boundry;
		      throw CfpError("syntax error in address of vector '"+getname()+"' specified as '" +invecval+"' (past array bounds).","",syntactical,false,false);
		    }
		}
	      else
		{
		  // its just a comma after a token, ignore it.;
		}
	      lastwastoken=false;
	    }
	  else
	    {
	      // okay, it must be a token, so set it and advance the element index;
	      if (curelement>=1)
		{
		  // we went past array boundry;
		  throw CfpError("syntax error in assignment of vector '"+getname()+"' to '" +invecval+"' (too many dimensions specified).","",syntactical,false,false);
		}
	      
	      // okay weve got an address for current dimension;
	      dimad=(int)(CfpVal(tokeval));
	      dimmult=1; // change this to function of curelement for multidimensional arrays;
	      index+=dimmult*dimad;
	      
	      // advance the counter;
	      ++curelement;
	      lastwastoken=true;
	    }
	}
      
      if (depth!=0)
	{
	  // syntax error (unmatched []);
	  throw CfpError("syntax error in assignment of vector '"+getname()+"' to '" +invecval+"' (unmatched []).","",syntactical,false,false);
	}

      //      cout << "leaving parseindex with "+JRtostr(index)+".\n";
      return index;
    }

  virtual void changes(int index) {vecnotify(index);};

  virtual void vecnotify(int index)
    {
      // notify the appropriate listeners that the value for entru index has changed;
      // someone has changed our value, so we notify any listeners;
      list<CfpVAccstub *>::iterator li=listeners.begin();
      list<CfpVAccstub *>::iterator lend;
      CfpVal indexval;
      CfpVal outval;
      int toindex,fromindex;
      // store the changed index since it will be used most freq.;
      if (index==-1)
	indexval=get(0);
      else
	indexval=get(index);
      lend=listeners.end();
      while (li!=lend)
	{
	  // for each listener, determine which index it is listening to;
	  fromindex=(*li)->getfromindex();
	  if (fromindex==index || fromindex==-1)
	    {
	      // yes, it wants to be notified (either about this one or all);
	      (*li)->set(&indexval);
	    }
	  else if (index==-1)
	    {
	      // all have changed so send the listener the one it wants;
	      outval=get(fromindex);
	      (*li)->set(&outval);
	    }
	  li++;
	}
      isdefinedflag=true;
    }

  virtual bool isdefined() {return isdefinedflag;};			// accessor

};





























//--------------------------------------------------------------------------------
//---- CfpVecAccessor ------------------------------------------------------------
//--------------------------------------------------------------------------------
//: The CfpVec accessor, used for accessing vectors of template types

template<class ValType> class CfpVecAccessor : public CfpVectorAccessor
{
private:
  ValType *vecp;
  bool vecislocal;
public:
  CfpVecAccessor(string inname,CfpObject *objp,int inlength,ValType *invecp=NULL)
  : CfpVectorAccessor(inname,objp,inlength)
    {
      // constructor for the vector accessor;
      if (invecp==NULL)
	{
	  // if invecp==NULL then we will allocate space (and deallocate);
	  vecislocal=true;
	  vecp=new ValType [veclength];
	}
      else
	{
	  vecislocal=false;
	  vecp=invecp;
	}
    }
  
  CfpVecAccessor(string inname,CfpObject *objp,int inlength,string initstring)
  : CfpVectorAccessor(inname,objp,inlength)
    {
      // constructor for the vector accessor;
      // we will allocate space (and deallocate);
      vecislocal=true;
      vecp=new ValType [inlength];

      // now initialize it using the string (for some reason gcc doesnt see the parent scope for this func. call unless i specify it exlicitly);
      if (initstring!="")
	CfpVectorAccessor::set(initstring);
    }

  ~CfpVecAccessor()
    {
      // destructor for the vector accessor;
      if (vecislocal && vecp)
	{
	  // we allocated the vector space so we free it;
	  delete [] vecp;
	}
      vecp=NULL;
      vecislocal=false;
    }

public:

  virtual void set(CfpVal *valp,int index)                       // vector element set;
  {
    // vector element set;;
    checkrange(index);
    vecp[index]=(ValType)(*valp);
    // now notify the accessor listeners of changes;
    changes(index);
  }

  virtual CfpVal get(int index)                                // vector element get;
  {
    // vector element get;;
    checkrange(index);
    return CfpVal(vecp[index]);
  }

public:
  virtual void vecset(ValType *arrayaddr,int len=-1)                 // full vector set
    {
      // full vector copy from an array;
      // NOTE: current version only allows copying to/from an array of same type, but we should expand via templates;
      //       to let C++ do type conversion for us.;
      if (len==-1)
	{
	  // assume the array being copied is the right length;
	  len=veclength;
	}
      else
	{
	  if (len!=veclength)
	    {
	      throw CfpError("attempt to set vector '"+getname()+"' which is length "+JRtostr(veclength)+", using an array of size " + JRtostr(len)+".","",classerr,false,false);
	    }
	}
      // okay set the elements of the vector one at a time;
      for (int count=0;count<veclength;++count)
	vecp[count]=arrayaddr[count];
      // now notify the accessor listeners of changes;
      changes(-1);
    }

  virtual void vecget(ValType *arrayaddr,int len=-1)                 // full vector get;
  {
    // full vector copy to an array.;
    // NOTE: current version only allows copying to/from an array of same type, but we should expand via templates;
    //       to let C++ do type conversion for us.;
    if (len==-1)
      {
	// assume the array being copied is the right length;
	len=veclength;
      }
    else
      {
	if (len!=veclength)
	  {
	    throw CfpError("attempt to copy vector '"+getname()+"' which is length "+JRtostr(veclength)+", into an array of size " + JRtostr(len)+".","",classerr,false,false);
	  }
      }
    // okay copy the elements of the vector one at a time;
    for (int count=0;count<veclength;++count)
      arrayaddr[count]=vecp[count];
  }

  virtual void set(double x,double y,double z)                    // shortcut for size 3 set
    {
      // shortcut for size 3 set;
      if (veclength!=3)
	{
	  throw CfpError("attempt to do a shortcut 3 element set on vector '"+getname()+"', which is of length "+ JRtostr(veclength)+".","",classerr,false,false);
	}
      // set the 3 elements;
      vecp[0]=(ValType)x;
      vecp[1]=(ValType)y;
      vecp[2]=(ValType)z;
      // now notify the accessor listeners of changes;
      changes(-1);
    }
};





















//--------------------------------------------------------------------------------
//---- CfpVecindexAccessor -------------------------------------------------------
//--------------------------------------------------------------------------------
//: The CfpVecindex accessor, used for accessing vectors using accessor-functions-with-indices

template<class Base,class ValType> class CfpVFAMvecindex : public CfpVectorAccessor
{
private:
  ValType *vecp;
  Base *bclassp;
  void (Base::*setmemberfunc)(int,ValType);
  ValType (Base::*getmemberfunc)(int);
public:
  CfpVFAMvecindex(string inname,CfpObject *objp,int inlength,Base *inclassp,void (Base::*insetmemberfunc)(int,ValType),ValType (Base::*ingetmemberfunc)(int))
  : CfpVectorAccessor(inname,objp,inlength)
    {
      // constructor for the vector accessor;
      bclassp=inclassp;
      setmemberfunc=insetmemberfunc;
      getmemberfunc=ingetmemberfunc;
    }

public:
  virtual void set(CfpVal *valp,int index)                       // vector element set;
  {
    // vector element set;;
    checkrange(index);
    if (setmemberfunc==NULL)
      {
	throw CfpError("attempt to set element " + JRtostr(index)+" of vector '"+getname()+"', when vector is readonly.","",classerr,false,false);
      }
    // okay, its within range, so set the element specified to the value specified;
    (bclassp->*setmemberfunc)(index,(ValType)(*valp));
    // now notify the accessor listeners of changes;
    changes(index);
  }

  virtual CfpVal get(int index)                                // vector element get;
  {
    // vector element get;;
    checkrange(index);
    if (getmemberfunc==NULL)
      {
	throw CfpError("attempt to read element " + JRtostr(index)+" of vector '"+getname()+"', when vector is writeonly.","",classerr,false,false);
      }
    return CfpVal((bclassp->*getmemberfunc)(index));
  }

public:
  virtual bool getable() {if (getmemberfunc) return true; else return false;};
  virtual bool setable() {if (setmemberfunc) return true; else return false;};
};
























































//--------------------------------------------------------------------------------
//---- CfpWVecAccessor ------------------------------------------------------------
//--------------------------------------------------------------------------------
//: The CfpWVec accessor, used for accessing weltonVector elements

class CfpWVecAccessor : public CfpVectorAccessor
{
private:
  DVector *vecp;
public:
  CfpWVecAccessor(string inname,CfpObject *objp,DVector *invecp,int inveclength,int inindex=0)
    :CfpVectorAccessor(inname,objp,inveclength) {vecp=invecp;};
public:
  virtual bool getable() {return true;};
  virtual bool setable() {return true;};
public:
  virtual bool willnotify() {return false;};    // the source vector may be changed by an external source which may not notify us
public:
  virtual CfpVal get(int index)                                // vector element get;
  {
    // vector element get;;
    checkrange(index);
    return CfpVal((*vecp)(index));
  }

  virtual void set(string stringval, int index)                   // set element using string
    {
      // set element using string;
      checkrange(index);
      CfpVal tempsv(stringval);
      (*vecp)(index)=tempsv;
      changes(index);
    }

  virtual void set(double x,double y,double z)                    // shortcut for size 3 set
    {
      // shortcut for size 3 set;
      if (veclength!=3)
	{
	  throw CfpError("attempt to do a shortcut 3 element set on vector '"+getname()+"', which is of length "+ JRtostr(veclength)+".","",classerr,false,false);
	}
      // set the 3 elements;
      (*vecp)(0)=x;
      (*vecp)(1)=y;
      (*vecp)(2)=z;
      // now notify the accessor listeners of changes;
      changes(-1);
    }
};














//--------------------------------------------------------------------------------
//---- NON-MEMBER-FUNCTION ACCESSORS ---------------------------------------------
//--------------------------------------------------------------------------------



//--------------------------------------------------------------------------------
//---- CfpVFAnotify --------------------------------------------------------------
//--------------------------------------------------------------------------------
//:NON member-function invoking accessor.
// Function linked has form "void ()"
// These are accessors that are useful for invoking a member fuction on receiving
// a callback (as opposed to directly setting a variable on receiving a callback).
// It is called a notify because the invoked function doesn't get passed any info
//  about the value of the accessor, it just gets called.

class CfpVFAnotify : public CfpVAccessor
{
private:
  void (*funcp)();
public:
  CfpVFAnotify(string inname,CfpObject *objp,void (*infuncp)())
    :CfpVAccessor(inname,objp) {funcp=infuncp;};
public:
  void set(CfpVal *val) {(*funcp)();changes();};
  CfpVal get() {throw CfpError("attempt to get the value of a VFAnotify.","",syntactical,false,false);};
public:
  virtual bool getable() {return false;};
  virtual bool setable() {return true;};
};



//--------------------------------------------------------------------------------
//---- CfpVFAcondnotify
//--------------------------------------------------------------------------------
//:NON member-function invoking accessor.
// Function linked has form "<retType> ()"
// These are accessors that are useful for invoking a member fuction on receiving
// a callback (as opposed to directly setting a variable on receiving a callback).
// It is called a notify because the invoked function doesn't get passed any info
//  about the value of the accessor, it just gets called.
// The cond modifier signified that the *listeners* of the called function are only called
//  IF the return value evaluates to true.

template <class RetT> class CfpVFAcondnotify : public CfpVAccessor
{
private:
  RetT (*funcp)();
public:
  CfpVFAcondnotify(string inname,CfpObject *objp,RetT (*infuncp)())
    :CfpVAccessor(inname,objp) {funcp=infuncp;};
public:
  void set(CfpVal *val) {if ((bool)((*funcp)())) changes();};
  CfpVal get() {throw CfpError("attempt to get the value of a VFAcondnotify.","",syntactical,false,false);};
public:
  virtual bool getable() {return false;};
  virtual bool setable() {return true;};
};



//--------------------------------------------------------------------------------
//---- CfpVFAsetval
//--------------------------------------------------------------------------------
//:Template for NON member-function invoking accessor.
// Function linked has form "void function(ValT)"

template<class ValTs,class ValTg=int> class CfpVFAsetval : public CfpVAccessor
{
private:
  void (*memberfuncs)(ValTs);
  ValTg (*memberfuncg)();
public:
  CfpVFAsetval(string inname,CfpObject *objp,void (*infs)(ValTs),ValTg (*infg)()=NULL)
    :CfpVAccessor(inname,objp) {memberfuncs=infs;memberfuncg=infg;};
public:
  void set(CfpVal *val) {if (memberfuncs) (classp->*memberfuncs)((ValT)*val);changes(val);};
  CfpVal get() {if (memberfuncg) *valp=(classp->*memberfuncg)(); return *valp;};
public:
  virtual bool getable() {if (memberfuncg) return true; else false;};
  virtual bool setable() {if (memberfuncs) return true; else false;};
};



//--------------------------------------------------------------------------------
//---- CfpVFAsetvalret -----------------------------------------------------------
//--------------------------------------------------------------------------------
//:Template for NON member-function invoking accessor.
// Function linked has form "<RetT> function(ValT)"
// [note that this is handled exactly as setval but we throw away the return value.]

template<class ValTs,class RetTs,class ValTg=int> class CfpVFAsetvalret : public CfpVAccessor
{
private:
  RetTs (*memberfuncs)(ValTs);
  ValTg (*memberfuncg)();
public:
  CfpVFAsetvalret(string inname,CfpObject *objp,RetTs (*infs)(ValTs),ValTg (*infg)()=NULL)
    :CfpVAccessor(inname,objp) {memberfuncs=infs;memberfuncg=infg;};
public:
  void set(CfpVal *val) {(*memberfuncs)((ValTs)*val);changes(val);};
  CfpVal get() {if (memberfuncg) *valp=(*memberfuncg)(); return *valp;};
public:
  virtual bool getable() {if (memberfuncg) return true; else false;};
  virtual bool setable() {if (memberfuncs) return true; else false;};
};



//--------------------------------------------------------------------------------
//---- CfpVFAsetarg --------------------------------------------------------------
//--------------------------------------------------------------------------------
//:Template for NON member-function invoking accessor.
// Function linked has form "void function(ValT)"
// In this case, the function invoked is NOT passed the set() value, but rather is
//  passed the buit-in arg stored with the accessor.
// This has use for example is you were adding a callback from a menu item.  In this
//  case, the menu item callback Value might be a meaningless number signifying its
//  position in the menu list.  Rather than passing the callback value to the invoked
//  function, we could create a CfpVFAMsetarg with a built-in label signifying the value
//  to be passed to the function.

template<class ValT> class CfpVFAsetarg : public CfpVAccessor
{
private:
  void (*memberfunc)(ValT);
  ValT data;
public:
  CfpVFAsetarg(string inname,CfpObject *objp,void (*inmemberfunc)(ValT),ValT indata)
    :CfpVAccessor(inname,objp) {memberfunc=inmemberfunc;data=indata;*valp=CfpVal(indata);};
public:
  void set(CfpVal *val) {if (memberfunc) (*memberfunc)(data);changes(val);};
public:
  virtual bool getable() {return true;};
  virtual bool setable() {if (memberfunc) return true; else false;};
};








//--------------------------------------------------------------------------------
//---- MEMBER-FUNCTION ACCESSORS -------------------------------------------------
//--------------------------------------------------------------------------------


//--------------------------------------------------------------------------------
//---- CfpVFAMnotify -------------------------------------------------------------
//--------------------------------------------------------------------------------
//:Template for Member-function invoking accessor.
// Function linked has form "void <Base>::()"
// These are accessors that are useful for invoking a member fuction on receiving
// a callback (as opposed to directly setting a variable on receiving a callback)

template<class Base> class CfpVFAMnotify : public CfpVAccessor
{
private:
  Base *bclassp;
  void (Base::*memberfunc)();
public:
  CfpVFAMnotify(string inname,CfpObject *objp,Base *inclassp,void (Base::*inmemberfunc)())
    :CfpVAccessor(inname,objp) {bclassp=inclassp;memberfunc=inmemberfunc;};
public:
  void set(CfpVal *val) {(bclassp->*memberfunc)();changes();};
  CfpVal get() {throw CfpError("attempt to get the value of a VFAMnotify.","",syntactical,false,false);};
public:
  virtual bool getable() {return false;};
  virtual bool setable() {return true;};
};



//--------------------------------------------------------------------------------
//---- CfpVFAMcondnotify
//--------------------------------------------------------------------------------
//:Template for Member-function invoking accessor.
// Function linked has form "void <Base>::()"
// These are accessors that are useful for invoking a member fuction on receiving
// a callback (as opposed to directly setting a variable on receiving a callback)
// The cond modifier signified that the *listeners* of the called function are only called
//  IF the return value evaluates to true.

template<class Base,class RetT> class CfpVFAMcondnotify : public CfpVAccessor
{
private:
  Base *bclassp;
  RetT (Base::*memberfunc)();
public:
  CfpVFAMcondnotify(string inname,CfpObject *objp,Base *inclassp,RetT (Base::*inmemberfunc)())
    :CfpVAccessor(inname,objp) {bclassp=inclassp;memberfunc=inmemberfunc;};
public:
  void set(CfpVal *val) {if ((bool)(bclassp->*memberfunc)()) changes();};
  CfpVal get() {throw CfpError("attempt to get the value of a VFAMcondnotify.","",syntactical,false,false);};
public:
  virtual bool getable() {return false;};
  virtual bool setable() {return true;};
};







//--------------------------------------------------------------------------------
//---- CfpVFAMgetval
//--------------------------------------------------------------------------------
//:Template for Member-function invoking accessor.
// Function linked has form "void <Base>::function(ValT)"

template<class Base,class ValTg> class CfpVFAMgetval : public CfpVAccessor
{
private:
  Base *bclassp;
  ValTg (Base::*memberfuncg)();
public:
  CfpVFAMgetval(string inname,CfpObject *objp,Base *inclassp,ValTg (Base::*infg)())
    :CfpVAccessor(inname,objp) {bclassp=inclassp;memberfuncg=infg;};
public:
  CfpVal get() {*valp=(bclassp->*memberfuncg)(); return *valp;};
public:
  virtual bool getable() {return true;};
  virtual bool setable() {return true;};
};


//--------------------------------------------------------------------------------
//---- CfpVFAMsetval
//--------------------------------------------------------------------------------
//:Template for Member-function invoking accessor.
// Function linked has form "void <Base>::function(ValT)"

template<class Base,class ValTs,class ValTg=int> class CfpVFAMsetval : public CfpVAccessor
{
private:
  Base *bclassp;
  void (Base::*memberfuncs)(ValTs);
  ValTg (Base::*memberfuncg)();
public:
  CfpVFAMsetval(string inname,CfpObject *objp,Base *inclassp,void (Base::*infs)(ValTs),ValTg (Base::*infg)()=NULL)
    :CfpVAccessor(inname,objp) {bclassp=inclassp;memberfuncs=infs;memberfuncg=infg;};
public:
  void set(CfpVal *val) {(bclassp->*memberfuncs)((ValTs)*val);changes(val);};
  CfpVal get() {if (memberfuncg) *valp=(bclassp->*memberfuncg)(); return *valp;};
public:
  virtual bool getable() {if (memberfuncg) return true; else false;};
  virtual bool setable() {return true;};
};




//--------------------------------------------------------------------------------
//---- CfpVFAMcfpsetval
//--------------------------------------------------------------------------------
//:Template for Member-function invoking accessor.
// Function linked has form "void <Base>::function(ValT)"

template<class Base> class CfpVFAMcfpsetval : public CfpVAccessor
{
private:
  Base *bclassp;
  void (Base::*memberfuncs)(CfpVal*);
  CfpVal (Base::*memberfuncg)();
public:
  CfpVFAMcfpsetval(string inname,CfpObject *objp,Base *inclassp,void (Base::*infs)(CfpVal*),CfpVal (Base::*infg)()=NULL)
    :CfpVAccessor(inname,objp) {bclassp=inclassp;memberfuncs=infs;memberfuncg=infg;};
public:
  void set(CfpVal *valp) {(bclassp->*memberfuncs)(valp);changes(valp);};
  CfpVal get() {if (memberfuncg) *valp=(bclassp->*memberfuncg)(); return *valp;};
public:
  virtual bool getable() {if (memberfuncg) return true; else false;};
  virtual bool setable() {return true;};
};





//--------------------------------------------------------------------------------
//---- CfpVFAMsetvalret -----------------------------------------------------------
//--------------------------------------------------------------------------------
//:Template for Member-function invoking accessor.
// Function linked has form "<RetT> <Base>function(ValT)"
// [note that this is handled exactly as setval but we throw away the return value.]

template<class Base,class ValTs,class RetTs,class ValTg=int> class CfpVFAMsetvalret : public CfpVAccessor
{
private:
  Base *bclassp;
  RetTs (Base::*memberfuncs)(ValTs);
  ValTg (Base::*memberfuncg)();
public:
  CfpVFAMsetvalret(string inname,CfpObject *objp,Base *inclassp,RetTs (Base::*infs)(ValTs),ValTg (Base::*infg)()=NULL)
    :CfpVAccessor(inname,objp) {bclassp=inclassp;memberfuncs=infs;memberfuncg=infg;};
public:
  void set(CfpVal *val) {(bclassp->*memberfuncs)((ValTs)*val);changes(val);};
  CfpVal get() {if (memberfuncg) *valp=(bclassp->*memberfuncg)(); return *valp;};
public:
  virtual bool getable() {if (memberfuncg) return true; else false;};
  virtual bool setable() {return true;};
};






//--------------------------------------------------------------------------------
//---- CfpVFAMsetarg -------------------------------------------------------------
//--------------------------------------------------------------------------------
//:Template for Member-function invoking accessor.
// Function linked has form "void <Base>::function(ValT)"
// In this case, the function invoked is NOT passed the set() value, but rather is
//  passed the buit-in arg stored with the accessor.

template<class Base,class ValT> class CfpVFAMsetarg : public CfpVAccessor
{
private:
  Base *bclassp;
  void (Base::*memberfunc)(ValT);
  ValT data;
public:
  CfpVFAMsetarg(string inname,CfpObject *objp,Base *inclassp,void (Base::*inmemberfunc)(ValT),ValT indata)
    :CfpVAccessor(inname,objp) {bclassp=inclassp;memberfunc=inmemberfunc;data=indata;*valp=CfpVal(indata);};
public:
  void set(CfpVal *val) {if (memberfuc) (bclassp->*memberfunc)(data);changes(val);};
};







//--------------------------------------------------------------------------------
//---- Cfpdirectval
//--------------------------------------------------------------------------------
//:Template for direct variable access
// Function linked has form "ValTg *varname"

template<class ValTg> class Cfpdirectval : public CfpVAccessor
{
private:
  ValTg *vaddr;
public:
  Cfpdirectval(string inname,CfpObject *objp,ValTg *invaddr)
    :CfpVAccessor(inname,objp) {vaddr=invaddr;};
public:
//  CfpVal get() {*valp=(CfpVal)0; return *valp;};
//  void set(CfpVal *val) {*vaddr=(ValTg)0;};
  CfpVal get() {*valp=(CfpVal)*vaddr; return *valp;};
  void set(CfpVal *val) {*vaddr=(ValTg)*val;};
public:
  virtual bool getable() {return true;};
  virtual bool setable() {return true;};
};


template<class Base,class ValTg> class CfpMdirectval : public CfpVAccessor
{
private:
  Base *bclassp;
  ValTg Base::*vaddr;
public:
  CfpMdirectval(string inname,CfpObject *objp,Base *inclassp,ValTg Base::*invaddr)
    :CfpVAccessor(inname,objp) {bclassp=inclassp;vaddr=invaddr;};
public:
  CfpVal get() {*valp=(CfpVal)0; return *valp;};
  void set(CfpVal *val) {*vaddr=(ValTg)0;};
//  CfpVal get() {*valp=(CfpVal)(bclassp->*vaddr); return *valp;};
//  void set(CfpVal *val) {(bclassp->*vaddr)=(ValTg)*val;};
public:
  virtual bool getable() {return true;};
  virtual bool setable() {return true;};
};































//--------------------------------------------------------------------------------
//---- CfpObject FUNCTION ACCESSORS ----------------------------------------------
//--------------------------------------------------------------------------------









//--------------------------------------------------------------------------------
//---- CfpFuncAccessor -----------------------------------------------------------
//--------------------------------------------------------------------------------
//: The base class for other function accessors

class CfpFuncAccessor : public CfpAccessor
{
protected:
  CfpMemberFunc *memberp;
public:
  string getargstring();								// accessor
public:
  CfpFuncAccessor(string inname,string argstring,CfpClass *inclassp);
public:
  virtual CfpVal invoke(map<string,CfpVal*> *argmap);					// invoke the function
};




//--------------------------------------------------------------------------------
//---- CfpObjFuncAcc -------------------------------------------------------------
//--------------------------------------------------------------------------------
//: The base class for other function accessors

class CfpObjFuncAcc : public CfpFuncAccessor
{
private:
  CfpObject *objp;
  string keyword;
public:
  CfpObjFuncAcc(string inname,CfpObject *inobjp,string argstring,CfpClass *inclassp,string inkey="");
public:
  CfpVal invoke(map<string,CfpVal*> *argmap);					// invoke the function
};



















//}
// Namespace close-brace
#endif
