// axis.C

#include "cfpaxis.h"

#include <iostream>
#include <cmath>


//-------------------------------------------------------------------------------
//-- Axis
//-------------------------------------------------------------------------------

Axis::Axis(string inname, string inlabel,string inenums)
{
  name=inname;
  label=inlabel;
  enums=inenums;
  discretes=0;

  // default
  adjustmode=noadjust;
  adjustmode=expand_mode;
  adjustmode=expandsymetricplus_mode;

  minshow=maxshow=0.0;
  firstval=true;

  // test
  discretes=15;
}

bool Axis::adjustout(float val)
{
  // adjust range and ret true if changed
  float dif;
  float difd;

  //  cout << "adjusting axis " << name << " ("<<minshow<<","<<maxshow<<") to " << val <<".\n";

  if (firstval)
    {
      minshow=maxshow=val;
      firstval=false;
    }
  else if (adjustmode==noadjust || (val>=minshow && val<=maxshow))
    return false;

  switch (adjustmode)
    {
    case noadjust:
      break;
    case hardshift_mode:
      dif=maxshow-minshow;
      while (val<minshow)
	{
	  minshow-=dif;
	  maxshow-=dif;
	}
      while (val>maxshow)
	{
	  minshow+=dif;
	  maxshow+=dif;
	}
      break;
    case softshift_mode:
      dif=maxshow-minshow;
      if (val<minshow)
	{
	  minshow=val;
	  maxshow=minshow+dif;
	}
      if (val>maxshow)
	{
	  maxshow=val;
	  minshow=maxshow-dif;
	}
      break;
    case expand_mode:
    case fit_mode:
      if (val<minshow)
	minshow=val;
      if (val>maxshow)
	maxshow=val;
      break;
    case expandsymetric_mode:
      // expand min and max same amount always
      if (val<minshow)
	{
	  dif=minshow-val;
	  minshow-=dif;
	  maxshow+=dif;
	}
      if (val>maxshow)
	{
	  dif=val-maxshow;
	  maxshow+=dif;
	  minshow-=dif;
	}
      break;
    case expandsymetricplus_mode:
      // expand min and max same amount always
      difd=(maxshow-minshow)/2;
      if (val<minshow)
	{
	  dif=(minshow-val)+difd;
	  minshow-=dif;
	  maxshow+=dif;
	}
      if (val>maxshow)
	{
	  dif=(val-maxshow)+difd;
	  maxshow+=dif;
	  minshow-=dif;
	}
      break;
    case expandplus_mode:
    case fitplus_mode:
      difd=(maxshow-minshow)/2;
      if (val<minshow)
	minshow=val-difd;
      if (val>maxshow)
	maxshow=val+difd;
      break;
    }

  // adjust anything and mark as needing redraw
  change();

  return true;
}


int Axis::discreteind(float val)
{
  // change val into a discrete value which represents the "bin" it would fall into
  float dif=maxshow-minshow;
  float per;
  int ind;

  if (val<minshow || val>maxshow || dif==0)
    return -1;
  
  // calculate discrete index
  per=discretes/dif;
  if (per>1)
    per=1;
  ind=(int) floor(per * (val-minshow));
  // correct if it falls exactly on maxshow
  if (ind==discretes)
    --ind;

  return ind;
}



float Axis::reverseind(int discind)
{
  // return the inverse datapoint location associated with the discrete index
  float dif=maxshow-minshow;
  float per;
  float val;

  if (dif==0)
    return 0;
  
  // calculate discrete index
  per=dif/discretes;

  val=((float)discind*per+minshow)+per/2.0;
  return val;
}




void Axis::setdiscretes(int indiscretes)
{
  discretes=indiscretes;
  change();
}






//-------------------------------------------------------------------------------
//-- VisibleAxis
//-------------------------------------------------------------------------------

VisibleAxis::VisibleAxis(string inname,string inlabel,string inenums)
  :Axis(inname,inlabel,inenums)
{
  // initialize
  minorticks=majorticks=0.0;
  spacing=0.2;
  scale=1.0;
  intercept=0.0;
  autoticks=true;
  minwall=intwall=maxwall=no_wall;
  bitfonts=false;
  scaletype=linear_scale;
  connect=false;
  minimize_intercept=true;
  needsredraw=true;
  stretchsymbols=false;

  // tests
  connect=true;
  //  minorticks=.5;
  //  majorticks=1.0;
  spacing=0.3;
  if (name=="x")
    {
      //      maxwall=wire_wall;
      //      minwall=wire_wall;
      show=true;
    }
  if (name=="y")
    {
      //      maxwall=wire_wall;
      //      minwall=wire_wall;
      show=true;
    }
  if (name=="z")
    {
      //      maxwall=wire_wall;
      minwall=solid_wall;
      show=true;
    }
}


void VisibleAxis::autotick()
{
  float dif;

  dif=maxshow-minshow;
  if (discretes>1)
    {
      // tick every row/column
      minorticks=dif/(float)(discretes-1.0);
      // number every other one
      majorticks=2;
    }
  else
    {
      majorticks=minorticks=0;
    }

}


void VisibleAxis::setticks(float mjticks, float miticks)
{
  float dif;

  majorticks=mjticks;
  minorticks=miticks;
  if (discretes==0)
    {
      dif=maxshow-minshow;
      discretes=(int)(dif/majorticks);
    }
}


void VisibleAxis::getaxispoints(float &sloc,float &iloc,float &eloc)
{
  sloc=minshow;
  eloc=maxshow;
  iloc=intercept;
}





void VisibleAxis::change()
{
  // axis has changed

  if (minimize_intercept)
    {
      // adjust intercept to minimum
      intercept=minshow;
    }

  if (autoticks)
    autotick();
  
  // calculate scale normalization
  scale=1.0/(maxshow-minshow);

  needsredraw=true;
}




float VisibleAxis::symbolsize()
{
  // return the size of a symbol on the axis, as a percentage of the axis length
  //  ie if axis is discrete, then the size would be something near 1/discretes

  return 1.0/(discretes*(1.0+spacing));
}


float VisibleAxis::symbolwidth()
{
  // return the size of a symbol on the axis, as the width in the axis frame

  return ((maxshow-minshow)/(discretes*(1.0+spacing)));
}


