// configpp.C
// This file contains member functions for the Configpp class, those dealing with
//  commandline options.

#include "cfpinc.h"

// Not implemented in gcc yet (2.8.1)
// using namespace std;
// namespace Configpp{





//--------------------------------------------------------------------------------
//---- Configpp::show_about() -----------------------------------------------------
//--------------------------------------------------------------------------------

void Configpp::show_about()
{
  // just display some self serving info
  cout << "Configpp version " << SAF_VERSIONSTRING << "\n";
  cout << "\n";
}



//--------------------------------------------------------------------------------
//---- Configpp::show_commandline_options() ---------------------------------------
//--------------------------------------------------------------------------------

void Configpp::show_commandline_options()
{
  cout << "\n  Configpp commandline options:\n";
  cout << "    -d  --debug        Debug mode on                  \n";
  cout << "    -w  --nowarnings   Warning mode off               \n";
  cout << "    -s  --syntax       Syntaxcheck only mode          \n";
  cout << "    -v  --version      Version info                   \n";
  cout << "    -h  --help         Display help                   \n";
  cout << "        --nodebug      Debug mode off        (default)\n";
  cout << "        --warnings     Warning mode on       (default)\n";
  cout << "\n";
}




//--------------------------------------------------------------------------------
//---- Configpp::commandline() ----------------------------------------------------
//--------------------------------------------------------------------------------

void Configpp::commandline(int argc, char **argv)
{
  // Parse any command line flags.
  // Might we want to save a copy of these and pass them to each library we load?
  // Should we remove those we use? (ie zero them out?  we do so.)
  int count;

  // all commandline flags begin with a hyphen.  we check all string args
  //  any that begin with a hyphen are treated as flag strings.
  for (count=0;count<argc;++count)
    checkforflags(argv[count]);
}




//--------------------------------------------------------------------------------
//---- Configpp::checkforflags() --------------------------------------------------
//--------------------------------------------------------------------------------
//: helper function for parsing command line options and rcfiles

void Configpp::checkforflags(char *word)
{
  if (word[0]=='-')
    {
      // got one
      if (JRclcmp(word,"-d","--debug"))
	Debugmode=true;
      if (JRclcmp(word,"","--nodebug"))
	Debugmode=false;
      if (JRclcmp(word,"-s","--syntax"))
	Parsermode=syntaxcheckMode;
      if (JRclcmp(word,"-w","--nowarnings"))
	Warningmode=false;
      if (JRclcmp(word,"","--warnings"))
	Warningmode=true;
      if (JRclcmp(word,"","--nohints"))
	Hintmode=false;
      if (JRclcmp(word,"","--hints"))
	Hintmode=true;
      if (JRclcmp(word,"-v","--version"))
	{
	  show_about();
	  exit(EXIT_SUCCESS);
	}
      if (JRclcmp(word,"-h","--help") || JRclcmp(word,"","--info") || JRclcmp(word,"","--about"))
	{
	  show_about();
	  show_commandline_options();
	  exit(EXIT_SUCCESS);
	}
    }
}

//--------------------------------------------------------------------------------
//---- Configpp::loadrcenv() ------------------------------------------------------
//--------------------------------------------------------------------------------

void Configpp::loadrcenv()
{
  // Load any configpp info from the environment string (ie directories)
}




//--------------------------------------------------------------------------------
//---- Configpp::loadrcfiles() ----------------------------------------------------
//--------------------------------------------------------------------------------

void Configpp::loadrcfiles()
{
  // Locate any .configpprc file and parse it (it could be in ~/.configpprc or /usr/local/lib/configpp/configpprc)
  //  our strategy is to treat .configpprc file(s) as commandline arguments
  //  and allow them to override each other.
  // ie the order of parsing should be:
  //   defaults -> environment_args -> ~/.configpprc -> ./.configpprc -> commandline_args
  // so that commandline_args will override any other flags.
}





// }
// Namespace close-brace
