//  Jesse Reichler, January 1996
//  Fred Delcomyn
//
//  chain.h
//  This file contains the class definition for the Chain class,
//   a component class of the Robot class.
//  Robots are made up of a single base, to which is connected multiple
//   open chains, as per decoupled tree structure of Freeman&Orin.
//  Open chains are made up of links described in Denavit/Hartenberg format
//  The Robot class is defined in a separate file,
//   as is the Link class.
//  Also the class Rot is also used and stored in rot.C (.h)
//  
//

#ifndef Chain_defined
#define Chain_defined

#include "misc.def"
#include "matrixinc.h"
#include "rot.h"
#include "physobject.h"
#include "link.h"



class Chain: public Thing
{
  friend class Rot;
  friend class Robot;
private:
  class Robot *robotp;
  int links;
  class Link *link[MAX_LINKS_PERCHAIN];
private:
  perVectorR3 hip_distance;
  percision hip_alpha_twist;
  percision hip_theta_angle;
private:
  class Rot rot;
public:
  Chain(const char *chainname,World *inworldp,Robot *inrobotp);
  void sethip(perVectorR3 inhipdistance,percision inat,percision inta);
  void sethip(float hipdx,float hipdy,float hipdz,percision inat,percision inta);
  void done_construct();
  ~Chain();
public:
  void addlink(class Link *linkp);
public:					
  void resetforces();
  void iterate(double deltat);
  void setbaselinks(double deltat);
  void forward_kinematics(double deltat);
  void getforces(double deltat);
  void foot_forces(double deltat);
  void backward_dynamics(double deltat);
  void fillDNmat(Link *linkp);
  void hip_equations(double deltat);
  void forward_dynamics(double deltat);
public:					
  void update(double deltat);
  void draw(class GrobVideoCamera *cam);
private:
  int linklookup(char *linkname);
private:
  void updatefordraw(double deltat);
public:
  class Link *lookuplink(char *linkname);
  void congrutest(Link *linkp,int indexto,int indexfrom);
public:
  virtual void movie_load(FILE *moviefp);
  virtual void movie_save(FILE *moviefp);
};





#endif
	
