//  Jesse Reichler, December 1996
//  Fred Delcomyn
//
//  NEW contact.h
//  This file contains the class definition for the Contact class,
//   used to manage contacts & forces derived from contacts.
//  Each instance of the class Contact is a possible contact between 2 things.
//  A Contact instance is created on world initialization, and persists
//   as long as the two things are still viable.
//  The Contact class is responsible for detecting impacts and reasoning
//   about forces generated.
//  Code is based on Freeman Thesis (1989) and Jan's impact.c code.
//
//  This new Contact class supports ONLY point-to-tile contacts
//  
//

class Surface;
class PointTrack;
class Contact;
class PointContact;
class Attachment;

#ifndef Contact_defined
#define Contact_defined

#include "misc.def"
#include "matrixinc.h"
#include "thing.h"
#include "physobject.h"
#include "world.h"




class Surface
{
  friend class PointContact;
protected:
public:
  percision Kx,Ky,Kz;
  percision Yx,Yy,Yz;
  percision musx,musy,musz;
  percision mudx,mudy,mudz;
public:
  Surface() {;};
  Surface(percision inKx,percision inKy,percision inKz,percision inYx,percision inYy,percision inYz,percision inmusx,percision inmusy,percision inmusz,percision inmudx,percision inmudy,percision inmudz);

  Surface(percision inKx,percision inKz,percision inYx,percision inYz,percision inmusx,percision inmusz,percision inmudx,percision inmudz) {Surface(inKx,inKx,inKz,inYx,inYx,inYz,inmusx,inmusx,inmusz,inmudx,inmudx,inmudz);};

  Surface(percision inKx,percision inKz,percision inYx,percision inYz,percision inmus,percision inmud) {Surface(inKx,inKx,inKz,inYx,inYx,inYz,inmus,inmus,inmus,inmud,inmud,inmud);};

public:
  void setK(percision inKx,percision inKy,percision inKz) {Kx=inKx;Ky=inKy;Kz=inKz;};
  void setY(percision inYx,percision inYy,percision inYz) {Yx=inYx;Yy=inYy;Yz=inYz;};
  void setmus(percision inmusx,percision inmusy,percision inmusz) {musx=inmusx;musy=inmusy;musz=inmusz;};
  void setmud(percision inmudx,percision inmudy,percision inmudz) {mudx=inmudx;mudy=inmudy;mudz=inmudz;};
};




class PointTrack
{
  // This class keeps track of a point, it's prev pos. and it's velocity;
  friend class Contact;
  friend class PointContact;
private:
  perVectorR3 curloc;
  perVectorR3 prevloc;
  perVectorR3 velocity;
  perVectorR3 comdist;
public:
  PointTrack(perVectorR3 incomdist) {comdist=incomdist;};
  void updatepoint(perVectorR3 newloc,percision deltat);
};




class Contact
{
  friend class PointContact;
  friend class Attachment;
protected:
  perVectorR3 contactloc;
  Physicalobject *obj1, *obj2;
private:
  int smartspring;
  percision multfact;
public:
  Contact();
  Contact(Physicalobject *ob1,Physicalobject *ob2, int insmartspring=0);
  virtual void update(double deltat) {;};
  virtual perVectorR3 computeforces(perVectorR3 deltaloc,perVectorR3 veloc) {;};
  virtual int touching(perVectorR3 &contactloc, double deltat) {;};
  void setsmart(int insmartspring) {smartspring=insmartspring;};
};



class PointContact: public Contact
{
private:
  PointTrack *pt;
private:
  perVectorR3 hookloc;
  int slipping;        // is it slipping?;
  int contacted;       // is it in contact?
public:
  PointContact(Physicalobject *ob1,Physicalobject *ob2,PointTrack *inpt, int insmartspring);
  virtual void update(double deltat);
  virtual int touching(perVectorR3 &contactloc, double deltat);
  virtual perVectorR3 computeforces(perVectorR3 deltaloc,perVectorR3 veloc);
private:
  void sendextforces(perVectorR3 forces);
};



class Attachment:public Contact, public Surface
{
private:
  char name[NAMESTRLEN];
  class World *worldp;
private:
  int attached;
  int xslipping,yslipping,zslipping;
public:
  Attachment::Attachment(char *inname, World *inworldp, Physicalobject *inobj1, Physicalobject *inobj2, int insmartspring);
  virtual void update(double deltat);
  virtual perVectorR3 computeforces(perVectorR3 deltaloc,perVectorR3 veloc);
private:
  void sendextforces(perVectorR3 forces);
  virtual void updateconstants(perVectorR3 loc);
};







// Used to pave the ground
class GroundTile: public Physicalobject, public Surface
{
public:
  GroundTile(const char *objname, class World *worldp, int wowned)
    : Physicalobject(objname,worldp,wowned) {;};

  GroundTile::GroundTile(const char *objname, class World *worldp, int wowned, percision KZ, percision Ktan, percision YZ, percision Ytan, percision mus, percision mud);
public:
  virtual int thingtype() {return THING_TRITILE;};
public:
  int linecontact(perVectorR3 linep1, perVectorR3 linep2, perVectorR3 &contactloc);
};

	
// Used to pave the ground
class GroundMesh: public Physicalobject, public Surface
{
public:
  GroundMesh(const char *objname, class World *worldp, int wowned)
    : Physicalobject(objname,worldp,wowned) {;};

  GroundMesh::GroundMesh(const char *objname, class World *worldp, int wowned, percision KZ, percision Ktan, percision YZ, percision Ytan, percision mus, percision mud);
public:
  virtual int thingtype() {return THING_GROUNDMESH;};
public:
  int linecontact(perVectorR3 linep1, perVectorR3 linep2, perVectorR3 &contactloc);
};

	


#endif





