//  Jesse Reichler, January 1996
//  Fred Delcomyn
//
//  physobject.h
//  This file contains the class definition for the Physobject class,
//   which are used to define physical, geometrical components,
//   like cones or cubes.
//  
//

class Shape;
class Physicalobject;


#ifndef Physobject_defined
#define Physobject_defined

#include "misc.h"
#include "misc.def"
#include "thing.h"
#include "grob.h"
#include "grobcamera.h"

#include "grobinline.h"
#include "grobcamerainline.h"



class Physicalobject;



// ----------------------------------------------------------------------------------------------
// -- Base Shape class
// ----------------------------------------------------------------------------------------------

class Shape
{
  friend class Physicalobject;
  friend class Link;
public:
  Grob *grobp;
protected:
  percision wall_thickness;
  int contactpoints;
  int smartsprings;
  class PointTrack *point[MAX_CONTACTPTS_PERSHAPE];
  perVectorR3 endlocation;
public:
  Shape();
  ~Shape();
public:
  void draw(GrobVideoCamera *cam) {grobp->draw(cam,cam->getdetail());};
  void setlocation(perVector loc) {grobp->setlocation(loc);};
  void setorientation(perMatrix orient) {grobp->setorientation(orient);};
  perVector getlocation() {return grobp->getlocation();};
  perVectorR3 getendlocation() {return endlocation;};
  void setendlocation(perVectorR3 endloc) {endlocation=endloc;};
  perMatrix getorientation() {return grobp->getorientation();};
public:
  void addcontacts(Physicalobject *obj1,Physicalobject *obj2);
  void addpoint(perVectorR3 comdist);
public:
  // temporary test
  virtual float getlocx(){return grobp->getlocation(0);};
  virtual float getlocy(){return grobp->getlocation(1);};
  virtual float getlocz(){return grobp->getlocation(2);};
  void setlocation(float x,float y,float z) {grobp->setlocation(0,x);grobp->setlocation(1,y);grobp->setlocation(2,z);};
  void setlocx(float x){grobp->setlocation(0,x);};
  void setlocy(float y){grobp->setlocation(1,y);};
  void setlocz(float z){grobp->setlocation(2,z);};
  void setloc(int index,float val){grobp->setlocation(index,val);};
  float getloc(int index){return grobp->getlocation(index);};
public:
  virtual int shapetype() {return SHAPE_NOSHAPE;};
  virtual void makegrob() {;};
  virtual void makecontactpoints() {;};
  virtual void updatecontactlocs(Physicalobject *objp,percision deltat) {;};
//  virtual perMatrix normalframe(perVectorR3 contactloc) {return grobp->getorientation();};
  virtual void normalframe(perVectorR3 contactloc,perMatrix &orientation) {orientation=grobp->getorientation();};
};







// ----------------------------------------------------------------------------------------------
// -- Derived Shape classes
// ----------------------------------------------------------------------------------------------


class Cube: public Shape
{
  percision xlength, ylength, zlength;
public:
  Cube (percision len1,percision len2,percision len3);
public:
  virtual int shapetype() {return SHAPE_CUBE;};
  virtual void makegrob();
};


class RoundCube: public Shape
{
  percision xlength, ylength, zlength;
  percision shift;
public:
  RoundCube (percision len1,percision len2,percision len3,percision inshift);
public:
  virtual int shapetype() {return SHAPE_CUBE;};
  virtual void makegrob();
};


class Sphere: public Shape
{
protected:
  percision radius;
public:
  Sphere(percision inradius);
public:
  virtual int shapetype() {return SHAPE_SPHERE;};
  virtual void makegrob();
  virtual void makecontactpoints();
  virtual void updatecontactlocs(Physicalobject *objp,percision deltat);
};


class Legcone: public Shape
{
public:
  percision proximal_radius, distal_radius;
  percision length;
  int sleeve;
  percision offst;
  int pointed;
public:
  Legcone (percision pr, percision dr, percision l, percision wthickness=0,int insleeve=0, percision inoffst=0, int inpointed=0);
public:
  virtual int shapetype() {return SHAPE_LEGCONE;};
  virtual void makegrob();
  virtual void makecontactpoints();
  virtual void updatecontactlocs(Physicalobject *objp,percision deltat);
public:
  percision getlength() {return length;};
  percision radiusat(percision pos);
  percision volumeat(percision pr, percision dr, percision len);
public:
  float getlocx(){return grobp->getlocation(0);};
  float getlocy(){return grobp->getlocation(1);};
  float getlocz(){return grobp->getlocation(2);};
};


class Triangle: public Shape
{
private:
  percision thickness;
  perVectorR3 P1;  	//vertex point in world coordinate;
  perVectorR3 P2;  	//vertex point, P2 != P1;
  perVectorR3 P3;  	//vertex point, P3 != P1 and P3 != P2;
  perVectorR3 N;   	//to calculate once with plane_equation();
  percision k;          //to calculate once with plane_equation();
public:
  Triangle(perVectorR3 inP1,perVectorR3 inP2,perVectorR3 inP3,float inthickness);
public:
  virtual int shapetype() {return SHAPE_TRIANGLE;};
  virtual void makegrob();
public:
  int linecontact(perVectorR3 linep1, perVectorR3 linep2, perVectorR3 &Intersect);
//  virtual perMatrix normalframe(perVectorR3 contactloc);
  virtual void normalframe(perVectorR3 contactloc,perMatrix &orientation);
private:
  void plane_equation();
};









class Mesh: public Shape
{
protected:
  int gridsx,gridsy;
  float *heights;
  float sx,ex,sy,ey,sz,ez;
  float xdist,ydist;
  float lowest,highest;
public:
  Mesh(float isx,float iex,float isy,float iey,float isz,float iez);
  ~Mesh();
public:
  void loadheights(float *inheights,int entries=-1);
  virtual int shapetype() {return SHAPE_MESH;};
  virtual void makedefaultgrob();
  virtual void makegrob();
public:
  int linecontact(perVectorR3 linep1, perVectorR3 linep2, perVectorR3 &Intersect);
  virtual void normalframe(perVectorR3 contactloc,perMatrix &orientation);
  int getgridsx(){return gridsx;};
  int getgridsy(){return gridsy;};
  void setdims(int igridsx,int igridsy);
protected:
  void scale();
  int index(int x,int y) {return y*gridsx+x;};
  void gridcoords(float x, float y, int &xi, int &yi, float &xof, float &yof);
public:
  float heightat(float x,float y);
};




	



// ----------------------------------------------------------------------------------------------
// -- Base Physicalobject class
// ----------------------------------------------------------------------------------------------

class Physicalobject: public Thing
{
  friend class Chain;
  friend class Shape;
protected:
  class Shape *shape;
  percision kilograms;
  class perVectorR3 Com;
  class perMatrix inertia;
  perVectorR3 userextF;
  perVectorR3 userextN;
  perVectorR3 extF;
  perVectorR3 extN;
  class perVectorR3 velocity;
public:
  Physicalobject(const char *objname, class World *worldp, int wowned);
  ~Physicalobject();
public:
  virtual void iterate(percision deltat) {;};
  virtual void update(percision deltat) {;};
  virtual void draw(class GrobVideoCamera *cam);
  void drawforce(GrobVideoCamera *cam);
public:
  void setlocation(perVector loc, percision deltat=0.0);
  void setlocation(float x,float y,float z);
  void setlocx(float x){shape->setlocx(x);};
  void setlocy(float y){shape->setlocy(y);};
  void setlocz(float z){shape->setlocz(z);};
public:
  void setorientation(perMatrix orient) {shape->setorientation(orient);};
  void setshape(class Shape *setshape);
  void setmass(percision inkilograms) {kilograms=inkilograms;};
  void setCom(perVector incom) {Com=incom;};
  void setinertia(perVector ininertia) {inertia=ininertia;};
  void setCom(float com0,float com1,float com2) {Com(0)=com0;Com(1)=com1;Com(2)=com2;};
  void setinertia(float i0,float i1,float i2) {inertia(0,0)=i0;inertia(1,1)=i1;inertia(2,2)=i2;};
  void setinertiax(float i0,float i1,float i2) {inertia(0,0)=i0;inertia(1,0)=i1;inertia(2,0)=i2;};
  void setinertiay(float i0,float i1,float i2) {inertia(0,1)=i0;inertia(1,1)=i1;inertia(2,1)=i2;};
  void setinertiaz(float i0,float i1,float i2) {inertia(0,2)=i0;inertia(1,2)=i1;inertia(2,2)=i2;};
  void setuserextFx(percision val) {userextF(0)=val;};
  void setuserextFy(percision val) {userextF(1)=val;};
  void setuserextFz(percision val) {userextF(2)=val;};
  percision mass() {return kilograms;};
  perVectorR3 locationval() {return shape->getlocation();};
  perMatrix orientationval() {return shape->getorientation();};
  perVectorR3 velocityval() {return velocity;};
  void setloc(int index,float val){shape->setloc(index,val);};
  float getloc(int index){return shape->getloc(index);};
public:
  virtual int shapetype() {return shape->shapetype();};
  virtual int thingtype() {return THING_PHYSOBJECT;};
  virtual int isphys() {return YES;};
  virtual void normalframe(perVectorR3 contactloc,perMatrix &orientation) {shape->normalframe(contactloc,orientation);};
  virtual void clearotherforces() {;};
public:
  // temporary test
  float getlocx(){return shape->getlocx();};
  float getlocy(){return shape->getlocy();};
  float getlocz(){return shape->getlocz();};
public:
  void clearextforce();
  void addextforce(perVectorR3 F, perVectorR3 N);
  void addcontacts(Physicalobject *obj2) {shape->addcontacts(this,obj2);};
  virtual void updatecontactlocs(percision deltat) {shape->updatecontactlocs(this,deltat);};
public:
  virtual void movie_load(FILE *moviefp);
  virtual void movie_save(FILE *moviefp);
};
	


	
#endif
