//  Jesse Reichler, January 1996
//  Fred Delcomyn
//
//  robot.C
//  This file contains the class methods for the Robot class.
//   see header file (.h) for description.
//  
//

#include "robot.h"
#include "world.h"

#include <string.h>




Robot::Robot(const char *robotname,World *worldp)
{
  // initialize the robot and add it to the world
  
  // init variables
  chains=0;
  base=(Base *)NULL;

  // add it
  setthing(robotname,worldp,YES);
  worldp->addthing(this);
}





void Robot::done_construct()
{
  // the robot has been completed (all chains, base added).
}





Robot::~Robot()
{
  int count;
  // cleanup before deleting a robot

  // remove the base;
  if (base!=NULL)
    delete base;
  
  // now remove all chains
  for (count=0;count<chains;++count)
    delete chain[count];
  chains=0;
}







void Robot::clearotherforces()
{
  // Reset the externally applied forces to a chain,
  //  this must be done before each phase of getting external torque inputs and user controls
  int count;
  
  for (count=0;count<chains;++count)
    chain[count]->resetforces();
}


void Robot::resetbaseav()
{
  // this can get called on an error, it says to reset all acceleration and velocities
  if (base!=NULL)
    {
      (base->Dv).fill(0.0);
      (base->Da).fill(0.0);
    }
}



void Robot::iterate(double deltat)
{
  int count;
  // Calculate updated accelerations of robot parts
  // Modeled after Freeman & Orin [1991] DTS method
  
  // Phase -1: Let the brain do its work
  
  // Phase 0:  Get the torques provoked by muscles
  
  // Phase 1:  Get the forces and do initial chain dynamics
  // All chains can be done in parallel.
  for (count=0;count<chains;++count)
    {
      // iterate the chain, which includes calc. forces (including ground forces) and propagating force to base
      //      cout << "iterating chain " << count << ".\n";
      chain[count]->iterate(deltat);
    }
  
  // Phase 2:  Having delived hip info from *all* chains to base, calculate base acceleration
  //  cout << "... iterating base.\n";
  iteratebase(deltat);
  //  cout << "... done iterating base.\n";
  
  // Phase 3:  Do Forward Leg Dynamics
  // Go from base to chains (all chains can be done in parallel).
  //  cout << "forawrd dynamics.\n";
  for (count=0;count<chains;++count)
    chain[count]->forward_dynamics(deltat);
  //  cout << "done forawrd dynamics.\n";
  
  // Phase 4: Update any robot sensors.
  //          Why do we do this at the end?
  //          Theory is as follows, after each discrete time step,
  //           the sensors are updated.  This updated info is avail
  //           to the robot in the interim period btwn timesteps.
  //          In this time interim the robot makes motor decisions
  //           which are applied at begining of each loop (phase1).
  //          The only critical point is that sensor info should not
  //           be avail. in mid-motion.  Sensor info from previous
  //           time step must be used to calculate muscle activity
  //           for current timestep.
  //          If you think about it, this means it does not matter
  //           whether sensors are updated at very begining or very
  //           end.
}




void Robot::update(double deltat)
{
  // allow the robot to move for delta time, change it's position
  int count;

  // 1. Update the base
  base -> update(deltat);
  
  // 2. Update the chains
  for (count=0;count<chains;++count)
    chain[count]->update(deltat);
}



void Robot::updatefordraw()
{
  // force updating of cameras (this proc. is used when world is paused but internals change)
  int count;
  
  // Updatefordraw the chains
  for (count=0;count<chains;++count)
    chain[count]->updatefordraw(0.0);
}



void Robot::draw(GrobVideoCamera *cam)
{
  // draw the robot
  int count;
  perVectorR3 p1,p2;
  
  // first draw the base
  base -> draw(cam);
  
  // now the chains
  for (count=0;count<chains;++count)
    chain[count]->draw(cam);
}



void Robot:: setbase(Base *basep)
{
  // add the base to the robot and initialize
  base=basep;
}




void Robot:: addchain(Chain *chainp)
{
  // add the chain to the robot base and initialize
  if (chains>=MAX_CHAINS_PERROBOT)
    {
      printf("$$$ERROR: Too many chains (%d) already attached to robot '%s'.\n",chains,name);
      return;
    }
  
  chain[chains]=chainp;
  ++chains;
}




Chain* Robot::lookupchain(char *chainname)
{
  // Find a chain attached to the robot with the name specified
  //  and return a pointer to it
  for (int count=0;count<chains;++count)
    {
      if (strcmp(chain[count]->name,chainname)==0)
	return(chain[count]);
    }
  return (Chain *)NULL;
}



void Robot::movie_load(FILE *moviefp)
{
  // load all data for a robot
  int count;

  // load base
  base -> movie_load(moviefp);

  // load chains
  for (count=0;count<chains;++count)
    chain[count]->movie_load(moviefp);
}


void Robot::movie_save(FILE *moviefp)
{
  // save all data for a robot
  int count;
  
  // save base
  base -> movie_save(moviefp);

  // save chains
  for (count=0;count<chains;++count)
    chain[count]->movie_save(moviefp);
}







// NOTE
// iteratebase() routine is in file dynamics.C

