// swbiobot.C

#include "cfpinc.h"
#include "swbiobot.h"
#include "datastream.h"

extern bool showforce;
extern double forcezoom;


//--------------------------------------------------------------------------------
//---- SWbiobot_obj::SWbiobot_obj() ----------------------------------------------------------
//--------------------------------------------------------------------------------

SWbiobot_obj::SWbiobot_obj(Configpp *cfpfp,CfpClass *inclassp,string oname)
  :CfpObject(cfpfp,inclassp,oname)
{
  // this is the constructor that gets called every time the user creates a new
  // instantiated object of the class.
  // This function and makenew, the next, are partners.
  //  CfpFactory::dummyv=3;
}


//--------------------------------------------------------------------------------
//---- SWbiobot_obj::makenew() ---------------------------------------------------------
//--------------------------------------------------------------------------------

CfpObject *SWbiobot_obj::makenew(CfpClass *inclassp,string objname)
{
  // create a new instance of ourselves, the wrapper.
  // Important: instantiating the wrapped class will come later (in function 'construct').

  return (new SWbiobot_obj(configppp,inclassp,objname));
}








//--------------------------------------------------------------------------------
//---- SWbiobot_obj() ------------------------------------------------------------
//--------------------------------------------------------------------------------

void SWbiobot_obj::registerwrappers(CfpFactory *sfactp)
{
  // this is the constructor that gets called ONCE when the class is registered with the factory.
  CfpClass *newclassp;

  // Register Experiment class
  newclassp=sfactp->registerclass("Biobot","Experiment",this);
  newclassp->addarg("runtime","100",false,"");

  // Register World class
  newclassp=sfactp->registerclass("Biobot","World",this);
  newclassp->addarg("gravity","9.81",false,"gravity of the world, as a z positive downward force");
  newclassp->addarg("running","1",false,"is the simulation running?","bool","bool");
  newclassp->addarg("stepping","0",false,"run the simulation for one timestep","bool","bool");
  newclassp->addarg("deltat",".001",false,"timestep interval","seconds","double");
  newclassp->addarg("clockcycle","10",false,"how often world is iterated","cycles","int");
  newclassp->addarg("pretrigger","",false,"the trigger signal set off before update of world","bool","bool");
  newclassp->addarg("clocktrigger","",false,"the trigger signal set off after update of world","bool","bool");
  newclassp->addvar("time","",false,"simulation world time","seconds","double");
  newclassp->addvar("showforce","",false,"show force lines","bool");
  newclassp->addarg("forcezoom","1",false,"zoom magnification for force lines");

  // Register Robot class
  newclassp=sfactp->registerclass("Biobot","Robot",this);

  // Register Base class
  newclassp=sfactp->registerclass("Biobot","Base",this);
  newclassp->addarg("type","fixed",false,"type of base (fixed or mobile)");
  addargs_physical(newclassp);

  // Register Chain class
  newclassp=sfactp->registerclass("Biobot","Chain",this);
  newclassp->addvarg("hiplocation",3,"[0 0 0]",false,"");
  newclassp->addarg("hipdhtwist","1.570796327",false,"");
  newclassp->addarg("hipdhangle","0.0",false,"");

  // Register Link class
  newclassp=sfactp->registerclass("Biobot","Link",this);
  newclassp->addarg("dhlength","0",false,"");
  newclassp->addarg("dhtwist","0",false,"");
  newclassp->addarg("dhoffset","0",false,"");
  newclassp->addarg("dhangle","0",false,"");
  newclassp->addarg("hingedx","","");
  newclassp->addarg("hingedy","0","");
  newclassp->addarg("hingedz","0","");
  newclassp->addarg("hingecx","","");
  newclassp->addarg("hingecy","0","");
  newclassp->addarg("hingecz","0","");
  newclassp->addarg("angle","","");
  newclassp->addarg("goalangle","","");
  newclassp->addarg("pid_p","0","");
  newclassp->addarg("pid_i","0","");
  newclassp->addarg("pid_d","0","");
  newclassp->addarg("pid_limit","999998","");
  newclassp->addvar("error","0","");
  newclassp->addvar("torque","","");
  newclassp->addvar("torquepulse","","");
  addargs_physical(newclassp);

  // Register Shape.Legcone class
  newclassp=sfactp->registerclass("Biobot","Shape.Legcone",this);
  newclassp->addarg("length","5",false,"");
  newclassp->addarg("proximal_radius","0.5",false,"");
  newclassp->addarg("distal_radius","0.5",false,"");
  newclassp->addarg("wall_thickness","0.2",false,"");
  newclassp->addarg("sleeve","0",false,"");
  newclassp->addarg("effector","0",false,"");
  newclassp->addarg("dhoffset","",true,"");
  addargs_shape(newclassp);

  // Register Shape.Cube class
  newclassp=sfactp->registerclass("Biobot","Shape.Cube",this);
  newclassp->addvarg("dimensions",3,"",false,"");
  addargs_shape(newclassp);

  // Register Shape.Sphere class
  newclassp=sfactp->registerclass("Biobot","Shape.Sphere",this);
  newclassp->addarg("radius","3",false,"");
  addargs_shape(newclassp);

  // Register Shape.TriTile class
  newclassp=sfactp->registerclass("Biobot","Shape.Triangle",this);
  newclassp->addarg("first_point","",false,"");
  newclassp->addarg("second_point","",false,"");
  newclassp->addarg("third_point","",false,"");
  newclassp->addarg("thickness","",false,"");
  addargs_shape(newclassp);

  // Register Shape.Square class
  newclassp=sfactp->registerclass("Biobot","Shape.Square",this);
  newclassp->addvarg("dimensions",2,"",false,"");
  addargs_shape(newclassp);
  
  // Register Shape.Elipse class
  newclassp=sfactp->registerclass("Biobot","Shape.Elipse",this);
  newclassp->addvarg("dimensions",3,"",false,"");
  newclassp->addarg("shift","0.0",false,"");
  addargs_shape(newclassp);
  
  // Register Shape.Mesh class
  newclassp=sfactp->registerclass("Biobot","Shape.Mesh",this);
  newclassp->addarg("gridsx","-1",false,"");
  newclassp->addarg("gridsy","-1",false,"");
  newclassp->addarg("sx","",false,"");
  newclassp->addarg("ex","",false,"");
  newclassp->addarg("sy","",false,"");
  newclassp->addarg("ey","",false,"");
  newclassp->addarg("sz","",false,"");
  newclassp->addarg("ez","",false,"");
  addargs_shape(newclassp);
  
  // Register Object.groundtile class for ground contact model
  newclassp=sfactp->registerclass("Biobot","Object.Groundtile",this);
  newclassp->addarg("perpendicular_spring","",false,"");
  newclassp->addarg("lateral_spring","",false,"");
  newclassp->addarg("perpendicular_damper","",false,"");
  newclassp->addarg("lateral_damper","",false,"");
  newclassp->addarg("static_friction_cone","",false,"");
  newclassp->addarg("dynamic_friction_cone","",false,"");
  addargs_physical(newclassp);

  // Register Object.surfacemesh class for ground contact model
  newclassp=sfactp->registerclass("Biobot","Object.Surfacemesh",this);
  newclassp->addarg("perpendicular_spring","",false,"");
  newclassp->addarg("lateral_spring","",false,"");
  newclassp->addarg("perpendicular_damper","",false,"");
  newclassp->addarg("lateral_damper","",false,"");
  newclassp->addarg("static_friction_cone","",false,"");
  newclassp->addarg("dynamic_friction_cone","",false,"");
  newclassp->addisa("Physicalobject");
  //  addargs_physical(newclassp);

  // Prepare for the simulation
  initialize_simulation();
}






//--------------------------------------------------------------------------------
//---- SWbiobot_obj::preinterpret() ----------------------------------------------------
//--------------------------------------------------------------------------------

void SWbiobot_obj::preinterpret(CfpClass *inclassp)
{
  // this function is called ONCE for each class, before interpretation begins
}



//--------------------------------------------------------------------------------
//---- SWbiobot_obj::postinterpret() ---------------------------------------------------
//--------------------------------------------------------------------------------

void SWbiobot_obj::postinterpret(CfpClass *inclassp)
{
  // this function is called ONCE for each class, after interpretation has completed
}




//--------------------------------------------------------------------------------
//---- SWbiobot_obj::destruct() -------------------------------------------------
//--------------------------------------------------------------------------------

void SWbiobot_obj::destruct()
{
  // de-instantiate the wrapped object.
  // we wrap ourselves so there is no need to delete any 'wrapped class', just set the instantiated flag to false
  instantiated=false;
}








//--------------------------------------------------------------------------------
//---- SWbiobot_obj::addargs_physical() ------------------------------------------
//--------------------------------------------------------------------------------

void SWbiobot_obj::addargs_physical(CfpClass *newclassp)
{

  // add default arguments for physical objects
  newclassp->addarg("mass",".3","");

  // non vector temp parameters
  newclassp->addvarg("centerofmass",3,"",false,"");
  newclassp->addvarg("location",3,"[0 0 0]",false,"");
  newclassp->addvarg("inertiadiagonal",3,"",false,"");
  newclassp->addvarg("inertiax",3,"",false,"");
  newclassp->addvarg("inertiay",3,"",false,"");
  newclassp->addvarg("inertiaz",3,"",false,"");

  // note that the class "isa" physical object
  newclassp->addisa("Physicalobject");
}


void SWbiobot_obj::addargs_shape(CfpClass *newclassp)
{
  // add default arguments for physical objects
  newclassp->addarg("color","","");
}











//--------------------------------------------------------------------------------
//---- SWbiobot_obj::construct() ------------------------------------------------
//--------------------------------------------------------------------------------

void SWbiobot_obj::construct()
{
  // instantiate the wrapped object.  this is where we actually do late (lazy) instantiation.
  // if we are a wrapper for another class, we would allocate that target wrapped class here.
  // and save it to a local variable.  otherwise this is its own wrapper.
  // the args will have been filled by now.

  // create the wrapped object

  if (baseclass()=="Experiment")
    construct_experiment();
  else if (baseclass()=="World")
    construct_world();
  else if (baseclass()=="Robot")
    construct_robot();
  else if (baseclass()=="Base")
    construct_base();
  else if (baseclass()=="Chain")
    construct_chain();
  else if (baseclass()=="Link")
    construct_link();
  else if (baseclass()=="Shape")
    construct_shape();
  else if (baseclass()=="Object")
    construct_object();

  // add accessors
  addaccessors();

  // set the instantiated flag
  instantiated=true;
}











//--------------------------------------------------------------------------------
//---- SWbiobot_obj::done() ------------------------------------------------------------
//--------------------------------------------------------------------------------

CfpVal SWbiobot_obj::done()
{
  // the object is done (ie no more children) return any return value

  if (baseclass()=="Experiment")
    ;
  else if (baseclass()=="World")
    {
      // end construction of the world
      worldp->done_construct();
    }
  else if (baseclass()=="Robot")
    {
      // end construction of the robot
      robotp->done_construct();
    }
  else if (baseclass()=="Base")
    {
      // after the base is done, add it to the robot
      setphys();
      robotp->setbase(basep);
    }
  else if (baseclass()=="Chain")
    {
      // after the chain is done, tell it it's done, and add it to the robot
      // set the hip parameters
      chainp->sethip(argval("hiplocation",0),argval("hiplocation",1),argval("hiplocation",2),argval("hipdhtwist"),argval("hipdhangle"));
      chainp->done_construct();
      robotp->addchain(chainp);
    }
  else if (baseclass()=="Link")
    {
      // after the link is done, add it to its parent chain

      if (!argdefined("centerofmass"))
	{
	  // guess at com if it's not specified (half way along)
	  setargval("centerofmass","["+JRtostr(linkp->getlength()/2.0)+" 0 0]");
	  configppp->debug("Assuming com of link is in center (it is undefined by user) using link coordinates [length/2 0 0]="+(string)argval("centerofmass")+".");
	}

      setphys();

      linkp->done_construct();
      chainp->addlink(linkp);

      // now we might guess at hinge_d and hinge_c (legcone shape has been added)
      if (argdefined("hingedx"))
	{
	  // user has specified hinged values explicitly
	  linkp->sethinge_d(argval("hingedx"),argval("hingedy"),argval("hingedz"));
	}
      else
	{
	  // guess at hinged, which is "relative" dist to center of mass, so usually it's just com
	  linkp->sethinge_d(argval("centerofmass",0),argval("centerofmass",1),argval("centerofmass",2));
	}
      if (argdefined("hingecx"))
	{
	  // user has specified hinged values explicitly
	  linkp->sethinge_c(argval("hingecx"),argval("hingecy"),argval("hingecz"));
	}
      else
	{
	  // guess at hingec, which is "relative" dist to endpoint
	  linkp->sethinge_c(linkp->getlength(),0,0);
	}
    }
  else if (baseclass()=="Shape")
    {
      // after a shape is done, add it to its parent physobject
      physobjp->setshape(shapep);
      if (subclass()=="Mesh")
	{
	  // now we have to recover the height data array from the users specified datastream and
	  //  configure the mesh
	  loadmeshdata();
	}
    }
  else if (baseclass()=="Object")
    {
      // the object is done, add it to the world
      worldp->addthing(physobjp);

      if (subclass()=="Surfacemesh")
	{
	}
      else
	{
	  // set some physical properties of the object (ground mesh is excluded)
	  setphys();
	}
    }

  return retval;
}









//--------------------------------------------------------------------------------
//---- SWbiobot_obj::construct_experiment() --------------------------------------
//--------------------------------------------------------------------------------

void SWbiobot_obj::construct_experiment()
{
  // build the biobot object
}




//--------------------------------------------------------------------------------
//---- SWbiobot_obj::construct_world()
//--------------------------------------------------------------------------------

void SWbiobot_obj::construct_world()
{
  // start building a world
  localp=worldp=new World(name.c_str());
  worldp->setdeltat(argval("deltat"));

      // now create a timer for the world
      // THIS should be made configurable by the user
      CfpTimer *timerp=new CfpTimer("worldcycle",configppp);
      timerp->cycleclock((long)argval("clockcycle"));
      configppp->addtimer(timerp);

      // now add a listener onto this timer which will iterate the world
      CfpValAccessor *timerac;
      timerac=timerp->getaccessorp();
      timerp->addlistener(new CfpVFAMnotify<SWbiobot_obj>("_timerac",this,this,&SWbiobot_obj::worlditerate));

      // now we do something tricky.
      // we want to allow other classes to be triggered PRIOR to the world update, but also
      // POST world update.  there are many ways we could do this, but we do this by
      // providing access to two accessors which can be used as triggers.
      pretrigger=new CfpValAccessor("pretrigger",this);
      clocktrigger=new CfpValAccessor("clocktrigger",this);
      addvarcopy("pretrigger",pretrigger);
      addvarcopy("clocktrigger",clocktrigger);
}



//--------------------------------------------------------------------------------
//---- SWbiobot_obj::construct_robot()
//--------------------------------------------------------------------------------

void SWbiobot_obj::construct_robot()
{
  // start building a robot
  CfpVAccessor *vac;
  SWbiobot_obj *parworld;


  // get the pointer to the world
  parworld=(SWbiobot_obj *)(configppp->parentinstantiate(this,"World",true));
  worldp=(World *)((parworld)->getlocalp());
  localp=robotp=new Robot(name.c_str(),worldp);
}



//--------------------------------------------------------------------------------
//---- SWbiobot_obj::construct_base()
//--------------------------------------------------------------------------------

void SWbiobot_obj::construct_base()
{
  // start building a base
  // get the pointer to the enclosing world and robot
  worldp=(World *)((configppp->parentinstantiate(this,"World",true))->getlocalp());
  robotp=(Robot *)((configppp->parentinstantiate(this,"Robot",true))->getlocalp());

  // is it fixed or mobile
  if (argeq("type","fixed"))
    basep=new Base(name.c_str(),worldp,BASE_FIXED);
  else if (argeq("type","mobile"))
    basep=new Base(name.c_str(),worldp,BASE_MOBILE);
  else
    {
      // unknown base type
      throw CfpError("Unknown base type ('" + (string)(argval("type"))+"') for robot","",syntactical,false,false);
    }

  localp=basep;
  }



//--------------------------------------------------------------------------------
//---- SWbiobot_obj::construct_chain()
//--------------------------------------------------------------------------------

void SWbiobot_obj::construct_chain()
{
  // start building a chain
  // get the pointer to the enclosing world and robot
  worldp=(World *)((configppp->parentinstantiate(this,"World",true))->getlocalp());
  robotp=(Robot *)((configppp->parentinstantiate(this,"Robot",true))->getlocalp());
  localp=chainp=new Chain(name.c_str(),worldp,robotp);
}



//--------------------------------------------------------------------------------
//---- SWbiobot_obj::construct_link()
//--------------------------------------------------------------------------------

void SWbiobot_obj::construct_link()
{
  // start building a link
  // get the pointer to the enclosing world and chain
  worldp=(World *)((configppp->parentinstantiate(this,"World",true))->getlocalp());
  chainp=(Chain *)((configppp->parentinstantiate(this,"Chain",true))->getlocalp());
  localp=linkp=new Link(name.c_str(),worldp);
  linkp->setdh(argval("dhlength"),argval("dhtwist"),argval("dhoffset"),argval("dhangle"));
}


//--------------------------------------------------------------------------------
//---- SWbiobot_obj::setphys()
//--------------------------------------------------------------------------------

void SWbiobot_obj::setphys()
{
  // set the physical object parameters for this physical object
  Physicalobject *objp=(Physicalobject *) localp;

  // set inertia
  if (argdefined("inertiadiagonal"))
    {
      // they have specified just the diagonal inertia elements, rest assumed 0
      objp->setinertia(argval("inertiadiagonal",0),argval("inertiadiagonal",1),argval("inertiadiagonal",2));
    }
  else if (argdefined("inertiax") || argdefined("inertiay") || argdefined("inertiaz"))
    {
      // they may be specifying the complete inertia matrix
      objp->setinertiax(argval("inertiax",0),argval("inertiax",1),argval("inertiax",2));
      objp->setinertiay(argval("inertiay",0),argval("inertiay",1),argval("inertiay",2));
      objp->setinertiaz(argval("inertiaz",0),argval("inertiaz",1),argval("inertiaz",2));
    }
  else
    {
      // assume a default interia
      objp->setinertia(argval("mass"),argval("mass"),argval("mass"));
      configppp->debug("No inertia matrix (inertiadiagonal or (inertiax,inertiay,inertiaz) so assuming inertial matrix of object is [mass 0 0][0 mass 0][0 0 mass].");
    }
  
  // set com
  if (argdefined("centerofmass"))
    objp->setCom(argval("centerofmass",0),argval("centerofmass",1),argval("centerofmass",2));
  else
    {
      objp->setCom(0,0,0);
      configppp->debug("Assuming center of mass of object is in center (ie centerofmass=[0 0 0]).");
    }

  objp->setmass(argval("mass"));
  if (argdefined("location"))
    objp->setlocation(argval("location",0),argval("location",1),argval("location",2));
}



//--------------------------------------------------------------------------------
//---- SWbiobot_obj::construct_shape()
//--------------------------------------------------------------------------------

void SWbiobot_obj::construct_shape()
{
  // start building a shape
  worldp=(World *)((configppp->parentinstantiate(this,"World",true))->getlocalp());
  physobjp=(Physicalobject *)((configppp->parentinstantiate(this,"Physicalobject",true))->getlocalp());

  // get the pointer to the enclosing physical object

  if (subclass()=="Cube")
    {
      double width,height,depth;
      width=argval("dimensions",0);
      height=argval("dimensions",1);
      depth=argval("dimensions",2);
      localp=shapep=new Cube(width,height,depth);
    }
  else if (subclass()=="Sphere")
    {
      double radius;
      radius=argval("radius");
      localp=shapep=new Sphere(radius);
    }
  else if (subclass()=="Legcone")
    {
      double proximalradius,distalradius,length,thickness,sleeve,offset,pointed;
      proximalradius=argval("proximal_radius");
      distalradius=argval("distal_radius");
      length=argval("length");
      thickness=argval("wall_thickness");
      sleeve=argval("sleeve");
      offset=argval("dhoffset");
      pointed=argval("effector");
      localp=shapep=new Legcone(proximalradius,distalradius,length,thickness,sleeve,offset,pointed);
    }
  else if (subclass()=="Square")
    {
      float length,width;
      perVectorR3 p1,p2,p3;
      float thickness;

      // make sure the parent object is instantiated
      configppp->parentinstantiate(this,"Physicalobject",true);

      // create a triangle parallel to z plane
      length=argval("dimensions",0);
      width=argval("dimensions",1);
      p1(0)=argval("sx");
      p1(1)=argval("sy");
      p1(2)=argval("sz");

      p2(0)=p1(0)+width;
      p2(1)=p1(1);
      p2(2)=p1(2);
      p3(0)=p1(0)+width/2;
      p3(1)=p1(1)+length;
      p3(2)=p1(2);

      thickness=argval("thickness");

      // create the shape
      localp=shapep=new Triangle(p1,p2,p3,thickness);
    }
  else if (subclass()=="Elipse")
    {
      double width,height,depth,shift;
      width=argval("dimensions",0);
      height=argval("dimensions",1);
      depth=argval("dimensions",2);
      shift=argval("shift");
      localp=shapep=new RoundCube(width,height,depth,shift);
    }
  else if (subclass()=="Mesh")
    {
      // create the surface mesh
      // make sure the parent object is instantiated
      Mesh *meshp;
      configppp->parentinstantiate(this,"Physicalobject",true);

      // create the shape
      int gridsx,gridsy;
      double sx,sy,ex,ey,sz,ez;
      double xdist,ydist;
      gridsx=argval("gridsx");
      gridsy=argval("gridsy");
      sx=argval("sx");
      ex=argval("ex");
      sy=argval("sy");
      ey=argval("ey");
      sz=argval("sz");
      ez=argval("ez");

      shapep=meshp=new Mesh(sx,ex,sy,ey,sz,ez);
      meshp->setdims(gridsx,gridsy);
      // we will load shape data on completion of the object
    }

  //  if (subclass()!="Mesh")
  //    {
  //      // set location
  //      shapep->setlocation(argval("location",0),argval("location",1),argval("location",2));
  //    }
}


//--------------------------------------------------------------------------------
//---- SWbiobot_obj::construct_object()
//--------------------------------------------------------------------------------

void SWbiobot_obj::construct_object()
{
  // build the biobot object

  // get the pointer to the enclosing world
  worldp=(World *)((configppp->parentinstantiate(this,"World",true))->getlocalp());

  if (subclass()=="Groundtile")
    {
      float KZ, Ktan, YZ, Ytan, mus, mud;
      KZ=argval("perpendicular_spring");
      Ktan=argval("lateral_spring");
      YZ=argval("perpendicular_damper");
      Ytan=argval("lateral_damper");
      mus=argval("static_friction_cone");
      mud=argval("dynamic_friction_cone");
      localp=physobjp=new GroundTile(name.c_str(),worldp,1,KZ,Ktan,YZ,Ytan,mus,mud);
    }
  else if (subclass()=="Surfacemesh")
    {
      float KZ, Ktan, YZ, Ytan, mus, mud;
      KZ=argval("perpendicular_spring");
      Ktan=argval("lateral_spring");
      YZ=argval("perpendicular_damper");
      Ytan=argval("lateral_damper");
      mus=argval("static_friction_cone");
      mud=argval("dynamic_friction_cone");
      localp=physobjp=new GroundMesh(name.c_str(),worldp,1,KZ,Ktan,YZ,Ytan,mus,mud);
    }
}




























//--------------------------------------------------------------------------------
//---- SWbiobot_obj::initialize_simulation() -------------------------------------
//--------------------------------------------------------------------------------

void SWbiobot_obj::initialize_simulation()
{
  // Initial preparations for simulation
  // called when library is registered
}








void SWbiobot_obj::addaccessors()
{
  CfpVAccessor *varp,*notifierp;

  if (baseclass()=="World")
    {
      World *p=(World *)localp;
      varp=addvar(new CfpVTAccessor<bool>("running",this,&(p->running)));
      varp=addvar(new CfpVFAMnotify<World>("stepping",this,p,&(p->gostepmode)));
      varp=addvar(new CfpVFAMsetval<World,double,double>("time",this,p,&(p->settime),&(p->gettime)));
      varp=addvar(new CfpVTAccessor<bool>("showforce",this,&showforce));
      varp=addvar(new CfpVTAccessor<double>("forcezoom",this,&forcezoom));
    }
  else if (baseclass()=="Shape")
    {
      Physicalobject *p=physobjp;
    }
  else if (baseclass()=="Base" || (baseclass()=="Object" && subclass()!="Surfacemesh"))
    {
      Physicalobject *p=(Physicalobject *)localp;
      varp=addvar(new CfpVFAMvecindex<Physicalobject,float>("location",this,3,p,&(p->setloc),&(p->getloc)));
    }

  if (baseclass()=="Link")
    {
      Link *lp=(Link *)localp;
      varp=addvar(new CfpVFAMsetval<Link,float,float>("angle",this,lp,&(lp->setangle),&(lp->getangle)));
      varp=addvar(new CfpVFAMsetval<Link,float,float>("goalangle",this,lp,&(lp->setgoalangle),&(lp->getgoalangle)));
      varp=addvar(new CfpVFAMsetval<Link,float,float>("pid_p",this,lp,&(lp->setpid_p),&(lp->getpid_p)));
      varp=addvar(new CfpVFAMsetval<Link,float,float>("pid_i",this,lp,&(lp->setpid_i),&(lp->getpid_i)));
      varp=addvar(new CfpVFAMsetval<Link,float,float>("pid_d",this,lp,&(lp->setpid_d),&(lp->getpid_d)));
      varp=addvar(new CfpVFAMsetval<Link,float,float>("pid_limit",this,lp,&(lp->setpid_limit),&(lp->getpid_limit)));
      varp=addvar(new CfpVFAMgetval<Link,float>("error",this,lp,&(lp->geterror)));
      varp=addvar(new CfpVFAMsetval<Link,float,float>("torque",this,lp,&(lp->setusertorque),&(lp->getusertorque)));
      varp=addvar(new CfpVFAMsetval<Link,float,float>("torquepulse",this,lp,&(lp->addtorque),&(lp->getexttorque)));
    }
}





void SWbiobot_obj::worlditerate()
{
  // this function gets called automatically by the world timer

  if (worldp->isstepping())
    {
      // if we are stepping, then we are running
      worldp->running=true;
    }

  // if world is paused then do nothing
  if (!worldp->isrunning())
    return;

  // clear any old forces (before controllers!)
  worldp->clearforces();

  // notify any pre-iterate listeners (CONTROLLERS)
  pretrigger->set(worldp->gettime());

  // now update the world
  worldp->step();

  // notify any listeners that we have iterated and pass them the new simulation time
  clocktrigger->set(worldp->gettime());

  // if we are stepping through, then pause after each step.
  if (worldp->isstepping())
    {
      worldp->stopstepping();
    }
}





bool SWbiobot_obj::moviesave(FILE *fp)
{
  // save the wrapped biobot object to file.
  // this should only be called on a wrapped world.  return true on success.
  bool bretv;

  if (classname()!="World")
    return false;
  worldp->movie_save(fp);

  return true;
}


bool SWbiobot_obj::movieload(FILE *fp)
{
  // load the wrapped biobot object from file.
  // this should only be called on a wrapped world.  return true on success.
  bool bretv;

  if (classname()!="World")
    return false;
  if (worldp->movie_load(fp)!=1)
    return false;

  return true;
}



void SWbiobot_obj::setmovieplaybackflag(bool inflag)
{
  // let the wrapped object know that it is being played back from a file, so don't do any internal updateing
  movieplaybackflag=true;
  if (classname()!="World")
    return;

  worldp->setmovieloadingflag();
}






void SWbiobot_obj::loadmeshdata()
{
  // this is called on completion of a mesh object.
  // we want to load the height data from the user specified input stream,
  //  and send it to the mesh.
  // now load all heights into an array and add it to the mesh
  Mesh *meshp=(Mesh *)shapep;
  Datastream *streamp;
  float *heights;
  long xdim,ydim;
  bool bretv;



  //  float heights[5*5];
  //  heights[0]=20; heights[1]=20; heights[2]=20; heights[3]=20; heights[4]=20; 
  //  heights[5]=20; heights[6]=1; heights[7]=1; heights[8]=1;  heights[9]=20; 
  //  heights[10]=20; heights[11]=1; heights[12]=0; heights[13]=1; heights[14]=20; 
  //  heights[15]=20; heights[16]=1; heights[17]=1; heights[18]=1; heights[19]=20; 
  //  heights[20]=20; heights[21]=20; heights[22]=20; heights[23]=20; heights[24]=20; 
  //  meshp->loadheights(&heights[0],25);
  //  return;


  // first get the child input stream and open it
  streamp=(Datastream*)((configppp->findchild(this,"Datastream",true))->getlocalp());
  streamp->start();

  // get the size of the mesh from the shape
  xdim=meshp->getgridsx();
  ydim=meshp->getgridsy();

  // now ask the stream to fill an array of floats for us, expecting strict formatting of the table
  bretv=streamp->readfloatarray(&heights,ydim,xdim,true,true,true,true,true);
  if (!bretv)
    {
      // data failed to load
      return;
    }

  // now reset the mesh dimensions in case they were altered during reading
  meshp->setdims(xdim,ydim);

  // okay, just pass the array on to the mesh
  meshp->loadheights(heights,xdim*ydim);

  // and free up the array that readfloatarray() allocated for us
  delete [] heights;

  // then close the data stream
  streamp->finish();
}








