//  Jesse Reichler, January 1996
//  Fred Delcomyn
//
//  world.h
//  This file contains the class definition for the World class,
//   which contains robots and objects and the ground, and constants
//   such as gravity.
//

//class World;

#ifndef World_defined
#define World_defined

#include "thing.h"
#include "ground.h"
#include "grobcamera.h"
#include <stdlib.h>
#include <time.h>


class World : public GrobVideoCameraDrawer
{
  friend class Experiment;
  friend class Robot;
public:
  char name[NAMESTRLEN];
  bool running;
  bool stepping;
  bool movieloading;
protected:
  percision deltat;
  time_t endtime;
  time_t starttime;
  double runtime;
private:
  percision worldgravity;
  percision worldtime;
  class Ground *ground;
  int things;
  class Thing *thing[MAX_THINGS_PERWORLD];
  int contacts;
  class Contact *contact[MAX_CONTACTS_PERWORLD];
  int needsrecamera;
public:
  World(const char *worldname);
  void done_construct();
  ~World();
public:
  void addthing(class Thing *athing);
public:
  bool step();
  void updatefordraw();
  void clearforces();
public:
  void makecontacts();
  void makeobjcontacts(Physicalobject *obj);
  void addcontact(class Contact *icontact);
  void checkcontacts(double deltat);
public:
  percision gravity() {return worldgravity;};
  char *worldname() {return name;};
public:
  class Thing *lookupthing(char *thingname);
  class Physicalobject *lookupphysobject(char *objname);
public:
  percision rettime() {return worldtime;};
  void fixupruntime();
public:
  void setpause(percision dummyv);
  void setgravity(percision newval) {worldgravity=newval;};
  void setdeltat(percision newval) {deltat=newval;};
  void setground(class Ground *inground) {ground=inground;};
  void settime(double intime){worldtime=intime;};
  double gettime(){return (double)worldtime;};
  bool isrunning() {return running;};
  bool isstepping() {return stepping;};
  void gostepmode() {stepping=true;};
  void stopstepping() {running=false;stepping=false;};
  void setmovieloadingflag(bool inflag=true){movieloading=inflag;};
public:
  void movie_save(FILE *moviefp);
  int movie_load(FILE *moviefp);
public:
  void grobvidcam_draw(class GrobVideoCamera *cam);
};





#endif
