// infactory.C


#include "infactory.h"



// -------------------------------------------------------------------------------------
// -- DatInStore
// -------------------------------------------------------------------------------------


DatInStore::DatInStore(CfpVAccessor *invac, Datafactory *indfactp,SampleInType insmode,int inindex)
  :DatStore(invac,indfactp,inindex)
{
  // set the datstore
  int count;
  
  // samplemode
  sampmode=insmode;
  
  // initialize the stores
  for (count=0;count<3;++count)
    val[count].set_number(0.0);
}


void DatInStore::update(int windex)
{
  // write the current value to the accessor

  if (!skip)
    vac->set(&val[windex],index);
}




























// -------------------------------------------------------------------------------------
// -- DatafactoryIn
// -------------------------------------------------------------------------------------


DatafactoryIn::DatafactoryIn(Configpp *cfpp, int insamprate, bool inrecycle)
  :Datafactory(cfpp,insamprate)
{
  // Construct a new input file, where control info is stored
 
  // initialize
  indexp=NULL;
  readindex=writeindex=0;
  lowerindex=upperindex=-1;
  punctuate=false;
  recycle=inrecycle;
  modulus=0;delay=0;
}


DatafactoryIn::~DatafactoryIn()
{
  // Destruct an input file (file is closed in base deleter)
  
}





CfpVal *DatafactoryIn::getivalp(int index,int subindex)
{
  // return a pointer to the local val
  return (CfpVal*)(((DatInStore*)datlist[index])->val+subindex);
}


CfpVal *DatafactoryIn::getivalp(int subindex)
{
  // return a pointer to the local val
  return (CfpVal*)((indexp->val)+subindex);
}









// -------------------------------------------------------------------------------------
// -- DatafactoryIn
// -------------------------------------------------------------------------------------


void DatafactoryIn::start()
{
  // just tell stream to open the file
  Datafactory::start();
  streamp->openfile();
}


void DatafactoryIn::update()
{
  // figure out what data to use from the Datafactory and modify the accessors appropriately.
  // first check that we have a current file
  if (!streamp || done)
    return;

  // we only read every samplerate times
  if (++dtcounter<samprate)
    return;
  dtcounter=0;

  // locate, read in (and interpolate where nesc.) the values to send out to the accessors
  computevals();
  if (validvals)
    {
      // now send the accessor information out if appropriate
      //  (in punctauted mode, we only send it when we move to a new eventline (row of data))
      output();
      if (punctuate)
	punctuate=false;
    }
}



void DatafactoryIn::output()
{
  // assuming that the dats are loaded, send them out via there accessors
  int count;
  DatInStore *dp;
  int vacount=vacs();

  for (count=0;count<vacount;++count)
    {
      dp=((DatInStore *)datlist[count]);
      if (dp->sampmode==punctuated_sampimode && !punctuate)
	continue;
      else if (dp->sampmode==interpolate_sampimode)
	dp->update(0);
      else
	dp->update(writeindex);
    }
}





// -------------------------------------------------------------------------------------
// -- DatafactoryIn::computevals()
// -------------------------------------------------------------------------------------

void DatafactoryIn::computevals()
{
  // determine a row of data to send to the accessors. this is not trivial.  here is what we want:
  // sometimes the Datafactory has an "index" dat, like time, which provides us with a variable
  //  to use as an index into the file table.  so time might be used as an index into a table of
  //  time-locked forces to be applied.
  // if no index is being used, then we just sequentially apply rows from the Datafactory.
  // if we do use an index, there are other options concerning what values we use when the index
  //  is not matched in the table exactly (ie do we interpolate btwn. near matches, etc.)
  // If we have valid vals to output, set validvals, else set it to false
  FilterReturnType fretv;
  bool retb;

  if (!indexp)
    {
      // we have no index so we just read a row of data and apply it immediately.
      // this is the easy kind of input data file.
      readindex=writeindex=0;
      fretv=filterp->readrow(readindex);

      if (fretv==end_retval && recycle)
	{
	  // just move file pointer back to begining of file and start reading again
	  reset();
	  fretv=filterp->readrow(readindex);
	  // drop through
	}
      
      if (fretv==end_retval)
	{
	  // we are done with the file
	  finish();
	  validvals=false;
	  return;
	}
      else if (fretv==error_retval)
	{
	  // error reading
	  error("unexpected error while reading Datafactory.");
	}

      // okay we got some good values
      validvals=true;
      return;
    }

  // we are using an indexed data file, which requires some work on our part

  // first get the latest index value (which needs to be a number)
  curvalue=(indexp->getval()).get_number();

  // adjust it for modulus
  if (modulus!=0 && curvalue>=modulus)
    curvalue=fmod(curvalue,(modulus-delay))+delay;

  //  cout << "using curvalue=" << curvalue << "  mod=" << modulus << "   del=" << delay << ".\n";

  // now load in two surrounding rows below and above the curvalue
  if (retb=loadsurrounds())
    {
      // okay we got two surrounding rows, now make the write row
      makewriterow();
    }

  if (!retb)
    {
      // couldn't find surrounding rows
      validvals=false;
    }
  else
    validvals=true;

  return;
}




bool DatafactoryIn::loadsurrounds()
{
  // now load in two surrounding rows below and above the curvalue
  FilterReturnType fretv;

  if (lowerindex==-1)
    {
      // this is our first read so we load in a row and set lower and upper to it
      if (!initialread())
	return false;
    }

  if (curvalue<lowervalue && getrowcount()>1)
    {
      // current index is too low, and we are not at begining, so we need to reset
      // this can happen on a modulus before end of file
      reset();
      if (!initialread())
	return false;
    }

  while (uppervalue<curvalue)
    {
      // our surrounding rows are too low, so we bump up lower to upper and read in a new upper
      punctuate=true;
      readindex=lowerindex;
      lowerindex=upperindex;
      if (readindex==lowerindex)
	readindex=2;
      upperindex=readindex;
      fretv=filterp->readrow(readindex);
      if (fretv==end_retval)
	{
	  if (modulus==0 && recycle)
	    {
	      // if the file is set to recycle on index mode but no modulus is specified
	      // then we use an implicit modulus of the highest index in the file
	      modulus=uppervalue;
	      reset();
	      if (!initialread())
		return false;
	      delay=lowervalue;
	      curvalue=delay;
	    }
	  else if (modulus!=0)
	    {
	      reset();
	      initialread();
	      return false;
	    }
	  else
	    {
	      streamp->finish();
	      streamp=NULL;
	      return false;
	    }
	}
      else if (fretv==error_retval)
	{
	  streamp->finish();
	  streamp=NULL;
	  return false;
	}
      else
	{
	  lowervalue=uppervalue;
	  uppervalue=getivalp(upperindex)->get_number();
	}
    }

  if (curvalue<lowervalue || curvalue>uppervalue)
    return false;

  return true;
}



bool DatafactoryIn::initialread()
{
  // read the first index line
  FilterReturnType fretv;

  writeindex=0;
  lowerindex=upperindex=1;
  readindex=1;
  fretv=filterp->readrow(readindex);
  if (fretv!=success_retval)
    {
      streamp->finish();
      streamp=NULL;
      return false;
    }
  lowervalue=getivalp(lowerindex)->get_number();
  uppervalue=lowervalue;
  return true;
}





void DatafactoryIn::makewriterow()
{
  // okay we got two surrounding rows, now make the write row and set writeindex appropriatly.

  if (curvalue==lowervalue)
    {
      // an exact match, so just write out the value from that row next time
      writeindex=lowerindex;
    }
  else if (curvalue==uppervalue)
    {
      // an exact match, so just write out the value from that row next time
      writeindex=upperindex;
    }
  else
    writeindex=lowerindex;

  makerowinterp();
}





void DatafactoryIn::makerowinterp()
{
  // go through all dats and build an interpolated write row, between lowerindex and upperindex
  double valdif,valscale;
  double lval,uval,curv;
  int count;
  DatInStore *dp;
  int interprow=0;
  int valcount=vacs();

  valdif=uppervalue-lowervalue;
  valscale=(curvalue-lowervalue)/valdif;

  for (count=0;count<valcount;++count)
    {
      dp=((DatInStore *)datlist[count]);
      if (dp->sampmode==interpolate_sampimode)
	  {
	    lval=getivalp(count,lowerindex)->get_number();
	    uval=getivalp(count,upperindex)->get_number();
	    curv=lval+((uval-lval)*valscale);
	    (dp->val[0]).set_number(curv);
	  }
    }
}
      





void DatafactoryIn::reset()
{
  // we are recycling the input
  streamp->reset();
  filterp->resetrowcount();
}
