// matlabfilter.C
// Class methods for the matlab file filter

#include "matlabfilter.h"
#include <cstring>


FilterReturnType matlabStreamFilter::writeheader()
{
  // Write a nice header with variable labels
  time_t curtime;
  int count;

  // first the opening header
  curtime=time(NULL);
  streamp->write("%% Configpp version "+(string)SAF_VERSIONSTRING+" using Matlab filter.\n");
  streamp->write("%% Data output file \""+streamp->getname()+"\" created "+JRtostr(ctime(&curtime))+"\n");
  streamp->write("%% \n");

  // print optional user description
  if (streamp->getdesc()!="")
    {
      string desc=streamp->getdesc();
      string descline;

      streamp->write("%% User Description:\n");
      while ((descline=JRstripline(desc))!="")
	streamp->write("%%   "+descline+"\n");
      streamp->write("%% \n");
    }
  
  //now the variable list
  streamp->write("%% Variable List\n");
  streamp->write("%% --------------\n");
  for (count=0;count<vacs();++count)
    writeheadervar(count);
  streamp->write("%% --------------\n");
  streamp->write("%% \n\n\n");
      
  streamp->write("datamatrix=[\n");
  return success_retval;
}



FilterReturnType matlabStreamFilter::writeheadervar(int index)
{
  // write a nice label for the variable/datum
  string name,label,unitstr,classname,trailstr;

  name=vac(index)->getname();
  label=vac(index)->getlabel();
  unitstr=vac(index)->getunitstr();
  trailstr=vac(index)->getobjtrail();
  classname=vac(index)->getclassname();
  if (unitstr!="")
    unitstr=" ("+unitstr+")";

  streamp->write(JRstrprintf("%%%% %14s    %s%s\n",name.c_str(),label.c_str(),unitstr.c_str()));
  streamp->write(JRstrprintf("%%%% %14s     from object %s\n","",trailstr.c_str()));
  streamp->write(JRstrprintf("%%%% %14s     of class %s\n","",classname.c_str()));
  
  return success_retval;
}





FilterReturnType matlabStreamFilter::writetrailer()
{
  // add some extra stuff at the end of the matlab file
  string name1,name2;
  int count;

  // no trailer for matlab data files
  streamp->write("];\n\n\n");
  
  streamp->write("%% Extract variables from the big matrix and place into individual row vectors:\n");

  // now extract the variables and give title variables
  for (count=0;count<vacs();++count)
    writetrailervar(count);
  
  streamp->write("\n");
  streamp->write("%% And now delete the original big matrix:\n");
  streamp->write("clear datamatrix;\n");
  streamp->write("\n\n");
  if (vacs()>1)
    {
      // create a nice example to help them
      name1=vac(0)->getname();
      name2=vac(1)->getname();
      streamp->write("%% Here is an example of how to make a plot and use labels:\n");
      streamp->write("%%   plot ("+name1+","+name2+");\n");
      streamp->write("%%   xlabel("+name1+"label);\n");
      streamp->write("%%   ylabel("+name2+"label);\n");
      streamp->write("\n\n");
    }
  
  return success_retval;
}




FilterReturnType matlabStreamFilter::writetrailervar(int index)
{
  // create the matlab individual variable
  string name,label,unitstr;

  name=vac(index)->getname();
  label=vac(index)->getlabel();
  unitstr=vac(index)->getunitstr();
  if (unitstr!="")
    unitstr=" ("+unitstr+")";

  streamp->write(JRstrprintf("%s(1:%ld) = datamatrix(:,%d);\n",name.c_str(),(long)rowcount,index+1));
  streamp->write(name+"label = '"+label+unitstr+"';\n");
  streamp->write("\n");
  return success_retval;
}






FilterReturnType matlabStreamFilter::writerow()
{
  // now write a row of dats
  int count;
  CfpVal sval;

  for (count=0;count<vacs();++count)
    {
      // display the dat
      sval=val(count);
      streamp->write(sval.get_string());
      
      if (count<vacs()-1)
	streamp->write(" ");
      else
	{
	  // what to put at the end of a row
	  streamp->write(";\n");
	}
    }

  // increase the rowcount which keeps track of how many rows of data we wrote out.
  ++rowcount;
  
  return success_retval;
}


