// outfactory.C

#include "outfactory.h"




// -------------------------------------------------------------------------------------
// -- DataOutStore
// -------------------------------------------------------------------------------------


DatOutStore::DatOutStore(CfpVAccessor *invac,Datafactory *indfactp, SampleOutType insmode,int inindex)
  :DatStore(invac,indfactp,inindex)
{
  // set the datstore
  
  // samplemode
  sampmode=insmode;

  // initialize the store
  storeval.set_number(0.0);
}


void DatOutStore::update()
{
  // update the datstore, according the sampling mode. (ie do we average valus, sum them, etc.)
  CfpVal val;

  // now update the stores
  switch(sampmode)
    {
    case current_sampomode:
      // this can be used even for non-numeric values
      storeval=vac->get(index);
      break;
    case average_sampomode:
      // will fail for non-numeric
      val=vac->get(index);
      storeval.set_number((double)storeval+((double)val/(dfactp->getsamprate())) );
      break;
    case sum_sampomode:
      // will fail for non-numeric
      val=vac->get(index);
      storeval.set_number((double)storeval+(double)val);
      break;
    }
}





































// -------------------------------------------------------------------------------------
// -- DatafactoryOut
// -------------------------------------------------------------------------------------


DatafactoryOut::DatafactoryOut(Configpp *cfpp,int insamprate)
  :Datafactory(cfpp,insamprate)
{
  // create and open a new output file
}




DatafactoryOut::~DatafactoryOut()
{
  // write the end trailer for the file, if any.  file will be closed by base class
  finish();
}



void DatafactoryOut::start()
{
  // open the output file in the proper format, etc.

  // base start()
  Datafactory::start();

  // now open the file
  streamp->openfile();

  // now write the header
  filterp->writeheader();
}


void DatafactoryOut::finish()
{
  // write trailer and close the file

  filterp->writetrailer();
  Datafactory::finish();
}





void DatafactoryOut::update()
{
  // Update all dats in the record file
  int count;
  bool writepass;
  DatOutStore *dp;

  if (!streamp || done)
    return;
  
  ++dtcounter;
  if (dtcounter<samprate)
    writepass=false;
  else
    writepass=true;

  // update the dat values (perform averaging, etc.)
  for (count=0;count<datlist.size();++count)
    {
      dp=((DatOutStore *)datlist[count]);
      if (writepass || dp->sampmode!=current_sampomode)
	dp->update();
    }
  
  // even after updating, we only write every samplerate times
  if (!writepass)
    return;

  // write the new row
  filterp->writerow();

  // update local counters
  dtcounter=0;
}




