// swfltk.h
// This file describes fltk gui classes

// forward declarations
class SWfltk_modal;
class SWfltk_widget;
class SWfltk_glWindow;
class SWfltk_grobCamera;
class SWfltk_guipending;




#ifndef INC_swfltkh
#define INC_swfltkh

#include "cfpinc.h"
#include "grobcamera.h"

#include <FL/Fl.H>
#include <FL/gl.h>
#include <FL/glut.H>
#include <FL/math.h>
#include <FL/config.h>
#include <FL/Enumerations.H>
#include <FL/fl_ask.H>
#include <FL/fl_file_chooser.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Gl_Window.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Menu_.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Menu_Button.H>
#include <FL/Fl_Menu_Item.H>
#include <FL/Fl_Choice.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Toggle_Button.H>
#include <FL/Fl_Light_Button.H>
#include <FL/Fl_Valuator.H>
#include <FL/Fl_Slider.H>
#include <FL/Fl_Hor_Slider.H>
#include <FL/Fl_Value_Slider.H>



#define USER_DAMAGE 0x40



class SWfltk_modal: public CfpObject
{
private:
  string text;
public:
  SWfltk_modal(bool registerme=true):CfpObject() {if (registerme) GlobalFactoryreg(this,"Fltk.Modal");};
  SWfltk_modal(Configpp *incfpfp,CfpClass *inclassp,string oname);
public:
  virtual void registerwrappers(CfpFactory *sfactp);
  void postinterpret(CfpClass *inclassp);				// called once per class
  CfpObject *makenew(CfpClass *inclassp,string objname);		// generic makenew
  void construct();							// no arg constructor of wrappedobj
  CfpVal done();							// any return value?
  void destruct();							// deinstatiate wrapped obj
};







// GL window subclass
class SWfltk_glWindow : public Fl_Gl_Window
{
protected:
  Configpp *configppp;
protected:
  bool flgl_prepared;
  bool hiding;
public:
  SWfltk_glWindow(Configpp *incfpfp,int x,int y,int w,int h,const char *l=0):Fl_Gl_Window(x,y,w,h,l) {initialize(incfpfp);};
  SWfltk_glWindow(Configpp *incfpfp,int w,int h,const char *l=0):Fl_Gl_Window(w,h,l) {initialize(incfpfp);};
protected:
  void initialize(Configpp *incfpfp);
  void validify();
public:
  void draw();
  void draw_overlay();
  void singlebuffer() {if (mode()&FL_DOUBLE) mode( (mode()&~FL_DOUBLE) );};
  void doublebuffer() {if (!(mode()&FL_DOUBLE)) mode( mode()|FL_DOUBLE );};
public:
  virtual void flgl_prepare() {flgl_prepared=true;};
  virtual void customdraw() {;};
  virtual void customdraw_overlay() {;};
};


// Class used for initialization of gl stuff
class SWfltk_glinitWindow: public SWfltk_glWindow
{
public:
  SWfltk_glinitWindow():SWfltk_glWindow(NULL,60,1,"GL"){;};
public:
  virtual void customdraw();
};



// GL videocamera
class SWfltk_grobCamera : public SWfltk_glWindow, public GrobVideoCamera
{
public:
  SWfltk_grobCamera(Configpp *incfpfp,int x,int y,int w,int h,const char *l=0):SWfltk_glWindow(incfpfp,x,y,w,h,l){grobcam_initialize();};
  SWfltk_grobCamera(Configpp *incfpfp,int w,int h,const char *l=0):SWfltk_glWindow(incfpfp,w,h,l){grobcam_initialize();};
  void grobcam_initialize();
public:
  virtual void customdraw();
  virtual void customdraw_overlay() {;};
protected:
  void flgl_prepare();							// called first time after window shown
  void request_redraw(GrobVideoCameraDrawer *ingdrawerp);		// the drawer of our window has new info
};










class SWfltk_widget: public CfpObject
{
private:
  Fl_Window *windowp;
  Fl_Box *boxp;
  Fl_Button *buttonp;
  Fl_Value_Slider *vsliderp;
  Fl_Menu_Bar *menubarp;
  Fl_Menu_ *menup;
  SWfltk_widget *parentp;
  CfpVAccessor *accessorp;
  static SWfltk_glinitWindow *glinitwin;
private:
  int xpos,ypos;
  int width,height;
  string label;
  string text;
  string menuname;
  int menuindex;
  SWfltk_widget *lastitemp;
  int menuflags;
  int textsize;
  string textpos;
  int placex,placey;
  int leftspace,rightspace,topspace,bottomspace;
  int bigy,bigx;
  int spacerx,spacery;
  double startvalue;
public:
  SWfltk_widget(bool registerme=true):CfpObject() {if (registerme) GlobalFactoryreg(this,"Fltk.Widget");};
  SWfltk_widget(Configpp *cfpfp,CfpClass *inclassp,string oname);
public:
  virtual void registerwrappers(CfpFactory *sfactp);
  CfpObject *makenew(CfpClass *inclassp,string objname);		// generic makenew
  void construct();							// no arg constructor of wrappedobj
  CfpVal done();							// any return value?
  void destruct();							// deinstatiate wrapped obj
public:
  Fl_Window *getwinp(){return windowp;};				// accessor
  void *getlocalp(){return localp;};					// accessor
  int getwidth(){return width;};					// accessor
  int getheight(){return height;};					// accessor
  int getplacewidth(){return width-spacerx*2;};				// accessor (max width to be in window)
  int getplaceheight(){return height-spacery*2;};			// accessor (max height to be in window)
private:
  void construct_window();						// instantiation routine
  void construct_button();						// instantiation routine
  void construct_valuator();						// instantiation routine
  void construct_menu();						// instantiation routine
  void construct_glwindow();						// instantiation routine
  void construct_cameraview();						// instantiation routine
  void addaccessors();							// add accessors after instantiation
private:
  void addargs_widget(CfpClass *newclassp,string sizestr,string posstr);
  void addargs_label(CfpClass *newclassp);				// add default label args
  void addargs_callback(CfpClass *newclassp);				// add args for a callback
  void addargs_grob(CfpClass *newclassp);				// add videocam args
  void decodeargs_widget();						// handle default widget args
  void runargs_align(Fl_Widget *widgetp,string tpos);			// make fltk calls to setup alignment
  void runargs_grob();							// setup the grob drawer
private:
  void guessextent(string intext);					// guess loc and size of widget
  void guesssize(string intext);					// guess size of widget
  void guessposition();							// guess loc of widget
  void guesstextsize(string intext,int pointsize,int &textwidth,int &textheight);
									// guess text size
  int guesstextwidth(string intext,int pointsize);			// guess and return text width
  int guesstextheight(string intext,int pointsize);			// guess and return text height
  void guesslabelspace(string intext,int &gw,int &gh,int &lspc,int &rspc,int &tspc, int &bspc);	// adjust for label
private:
  void showwindow(Fl_Window *winp);					// show window and all its children
private:
  void place(CfpObject *childp,int &inxpos,int &inypos,int inwidth,int inheight,int lspc,int rspc,int tspc,int bspc,int pspacerx=-1,int pspacery=-1);
									// return a place on our wndow and update
public:
  void connectwires();
  static void ValCallback(Fl_Widget *widgep,void *accp);		// valuator callbacks route through here
  static void ButtonCallback(Fl_Widget *widgep,void *accp);		// button callbacks route through here
  static void MenuItemCallback(Fl_Widget *widgep,void *accp);		// menuitem callbacks route through here
  static void glstartcheck();						// make sure gl is ready
};




//: This is a derived timer which tells the gui (fltk) to process any pending gui events

class SWfltk_guipending : public CfpTimer
{
public:
  SWfltk_guipending(Configpp *inconfigppp): CfpTimer("Fltk process",inconfigppp,1){;};
public:
  int cycle() {return Fl::check();};						// just iterate the gui
};












#endif
