// SWhexcontrol.C

#include "swhexcontrol.h"



//--------------------------------------------------------------------------------
//---- SWhexcontrol_obj::SWhexcontrol_obj()
//--------------------------------------------------------------------------------

SWhexcontrol_obj::SWhexcontrol_obj(Configpp *cfpfp,CfpClass *inclassp,string oname)
  :CfpObject(cfpfp,inclassp,oname)
{
  // this is the constructor that gets called every time the user creates a new
  // instantiated object of the class. This function and makenew, the next, are partners.
}


//--------------------------------------------------------------------------------
//---- SWhexcontrol_obj::makenew()
//--------------------------------------------------------------------------------

CfpObject *SWhexcontrol_obj::makenew(CfpClass *inclassp,string objname)
{
  // create a new instance of ourselves, the wrapper.
  // Important: instantiating the wrapped class will come later (in function 'construct').

  return (new SWhexcontrol_obj(configppp,inclassp,objname));
}




//--------------------------------------------------------------------------------
//---- SWhexcontrol_obj()
//--------------------------------------------------------------------------------

void SWhexcontrol_obj::registerwrappers(CfpFactory *sfactp)
{
  // this is the constructor that gets called ONCE when the class is registered with the factory.;
  CfpClass *newclassp;
  
  // add the hexapod controller class
  newclassp=sfactp->registerclass("hexcontrol","Controller.Hexapod",this);

  // add the trigger for the controller
  newclassp->addvar("trigger","",false,"accessor for deciding when to iterate the controller)");
  newclassp->addvar("cycletime",".1",false,"how long it takes to complete one leg protraction-retraction cycle");
  newclassp->addvar("risetime","0",false,"how long before it starts to even drive its legs and stand up");
  newclassp->addvar("stabletime","0",false,"how long does it take to stand up");
  newclassp->addvar("walktime","0",false,"how long does it wait after standing before it starts to walk");
  // add the input accessors
  newclassp->addvar("leftfrontcoxacurrent","",false,"current angle of this joint");
  newclassp->addvar("leftfrontfemurcurrent","",false,"current angle of this joint");
  newclassp->addvar("leftfronttibiacurrent","",false,"current angle of this joint");
  newclassp->addvar("leftmiddlecoxacurrent","",false,"current angle of this joint");
  newclassp->addvar("leftmiddlefemurcurrent","",false,"current angle of this joint");
  newclassp->addvar("leftmiddletibiacurrent","",false,"current angle of this joint");
  newclassp->addvar("leftrearcoxacurrent","",false,"current angle of this joint");
  newclassp->addvar("leftrearfemurcurrent","",false,"current angle of this joint");
  newclassp->addvar("leftreartibiacurrent","",false,"current angle of this joint");
  newclassp->addvar("rightfrontcoxacurrent","",false,"current angle of this joint");
  newclassp->addvar("rightfrontfemurcurrent","",false,"current angle of this joint");
  newclassp->addvar("rightfronttibiacurrent","",false,"current angle of this joint");
  newclassp->addvar("rightmiddlecoxacurrent","",false,"current angle of this joint");
  newclassp->addvar("rightmiddlefemurcurrent","",false,"current angle of this joint");
  newclassp->addvar("rightmiddletibiacurrent","",false,"current angle of this joint");
  newclassp->addvar("rightrearcoxacurrent","",false,"current angle of this joint");
  newclassp->addvar("rightrearfemurcurrent","",false,"current angle of this joint");
  newclassp->addvar("rightreartibiacurrent","",false,"current angle of this joint");
  // add the output accessors
  newclassp->addvar("leftfrontcoxagoal","",false,"goal angle of this joint");
  newclassp->addvar("leftfrontfemurgoal","",false,"goal angle of this joint");
  newclassp->addvar("leftfronttibiagoal","",false,"goal angle of this joint");
  newclassp->addvar("leftmiddlecoxagoal","",false,"goal angle of this joint");
  newclassp->addvar("leftmiddlefemurgoal","",false,"goal angle of this joint");
  newclassp->addvar("leftmiddletibiagoal","",false,"goal angle of this joint");
  newclassp->addvar("leftrearcoxagoal","",false,"goal angle of this joint");
  newclassp->addvar("leftrearfemurgoal","",false,"goal angle of this joint");
  newclassp->addvar("leftreartibiagoal","",false,"goal angle of this joint");
  newclassp->addvar("rightfrontcoxagoal","",false,"goal angle of this joint");
  newclassp->addvar("rightfrontfemurgoal","",false,"goal angle of this joint");
  newclassp->addvar("rightfronttibiagoal","",false,"goal angle of this joint");
  newclassp->addvar("rightmiddlecoxagoal","",false,"goal angle of this joint");
  newclassp->addvar("rightmiddlefemurgoal","",false,"goal angle of this joint");
  newclassp->addvar("rightmiddletibiagoal","",false,"goal angle of this joint");
  newclassp->addvar("rightrearcoxagoal","",false,"goal angle of this joint");
  newclassp->addvar("rightrearfemurgoal","",false,"goal angle of this joint");
  newclassp->addvar("rightreartibiagoal","",false,"goal angle of this joint");
}






//--------------------------------------------------------------------------------
//---- SWhexcontrol_obj::destruct()
//--------------------------------------------------------------------------------

void SWhexcontrol_obj::destruct()
{
  // de-instantiate the wrapped object and free memory

  instantiated=false;

  // delete the proxy that was created
  delete proxyp;
}














//--------------------------------------------------------------------------------
//---- SWhexcontrol_obj::construct() ------------------------------------------------
//--------------------------------------------------------------------------------

void SWhexcontrol_obj::construct()
{
  // instantiate the wrapped object.  this is where we actually do late (lazy) instantiation.
  // if we are a wrapper for another class, we would allocate that target wrapped class here.
  // and save it to a local variable.  otherwise this is its own wrapper.
  // the args will have been filled by now.
  float cycletime,stabletime,risetime,walktime;

  // create the wrapped object
  controllerp=new Hexcon();
  controllerp->buildcontroller();

  // add accessors
  addaccessors();

  // set the instantiated flag
  instantiated=true;
}


//--------------------------------------------------------------------------------
//---- SWhexcontrol_obj::done()
//--------------------------------------------------------------------------------

CfpVal SWhexcontrol_obj::done()
{
  // the object is done (ie no more children) return any return value
  
  return retval;
}













//--------------------------------------------------------------------------------
//---- SWhexcontrol_obj::addaccessors()
//--------------------------------------------------------------------------------

void SWhexcontrol_obj::addaccessors()
{
  // add the accessors for the hexapod controller
  CfpVAccessor *varp;
  CfpVAccessor *triggerp;


  // add the parameter accessors
  addvar(new CfpVFAMsetval<Hexcon,float>("cycletime",this,controllerp,&(controllerp->setcycletime)));
  addvar(new CfpVFAMsetval<Hexcon,float>("risetime",this,controllerp,&(controllerp->setrisetime)));
  addvar(new CfpVFAMsetval<Hexcon,float>("stabletime",this,controllerp,&(controllerp->setstabletime)));
  addvar(new CfpVFAMsetval<Hexcon,float>("walktime",this,controllerp,&(controllerp->setwalktime)));

  // build a trigger proxy
  proxyp=new CfpVAccTriggerProxy("trigger");

  // now tell the proxy about the input variables that must be up-to-date when we are triggered
  addvar(varp=new Cfpdirectval<float>("leftfrontcoxacurrent",this,&(controllerp->linkstate[0][0][0])));
  proxyp->addaccessor(varp,inputtype);
  addvar(varp=new Cfpdirectval<float>("leftfrontfemurcurrent",this,&(controllerp->linkstate[0][0][1])));
  proxyp->addaccessor(varp,inputtype);
  addvar(varp=new Cfpdirectval<float>("leftfronttibiacurrent",this,&(controllerp->linkstate[0][0][2])));
  proxyp->addaccessor(varp,inputtype);
  addvar(varp=new Cfpdirectval<float>("leftmiddlecoxacurrent",this,&(controllerp->linkstate[0][1][0])));
  proxyp->addaccessor(varp,inputtype);
  addvar(varp=new Cfpdirectval<float>("leftmiddlefemurcurrent",this,&(controllerp->linkstate[0][1][1])));
  proxyp->addaccessor(varp,inputtype);
  addvar(varp=new Cfpdirectval<float>("leftmiddletibiacurrent",this,&(controllerp->linkstate[0][1][2])));
  proxyp->addaccessor(varp,inputtype);
  addvar(varp=new Cfpdirectval<float>("leftrearcoxacurrent",this,&(controllerp->linkstate[0][2][0])));
  proxyp->addaccessor(varp,inputtype);
  addvar(varp=new Cfpdirectval<float>("leftrearfemurcurrent",this,&(controllerp->linkstate[0][2][1])));
  proxyp->addaccessor(varp,inputtype);
  addvar(varp=new Cfpdirectval<float>("leftreartibiacurrent",this,&(controllerp->linkstate[0][2][2])));
  proxyp->addaccessor(varp,inputtype);
  addvar(varp=new Cfpdirectval<float>("rightfrontcoxacurrent",this,&(controllerp->linkstate[1][0][0])));
  proxyp->addaccessor(varp,inputtype);
  addvar(varp=new Cfpdirectval<float>("rightfrontfemurcurrent",this,&(controllerp->linkstate[1][0][1])));
  proxyp->addaccessor(varp,inputtype);
  addvar(varp=new Cfpdirectval<float>("rightfronttibiacurrent",this,&(controllerp->linkstate[1][0][2])));
  proxyp->addaccessor(varp,inputtype);
  addvar(varp=new Cfpdirectval<float>("rightmiddlecoxacurrent",this,&(controllerp->linkstate[1][1][0])));
  proxyp->addaccessor(varp,inputtype);
  addvar(varp=new Cfpdirectval<float>("rightmiddlefemurcurrent",this,&(controllerp->linkstate[1][1][1])));
  proxyp->addaccessor(varp,inputtype);
  addvar(varp=new Cfpdirectval<float>("rightmiddletibiacurrent",this,&(controllerp->linkstate[1][1][2])));
  proxyp->addaccessor(varp,inputtype);
  addvar(varp=new Cfpdirectval<float>("rightrearcoxacurrent",this,&(controllerp->linkstate[1][2][0])));
  proxyp->addaccessor(varp,inputtype);
  addvar(varp=new Cfpdirectval<float>("rightrearfemurcurrent",this,&(controllerp->linkstate[1][2][1])));
  proxyp->addaccessor(varp,inputtype);
  addvar(varp=new Cfpdirectval<float>("rightreartibiacurrent",this,&(controllerp->linkstate[1][2][2])));
  proxyp->addaccessor(varp,inputtype);

  // now create the trigger for the controller
  triggerp=new CfpVFAMsetval<Hexcon,float>("trigger",this,controllerp,&(Hexcon::process));
  // add it to the proxy
  proxyp->addaccessor(triggerp,triggertype);
  // and add the *proxies* trigger to the visible variables that the user has access to
  addvar(proxyp->getproxytriggerp());

  // and lastly add the output variables, so they deliver their updates after trigger finishes
  addvar(varp=new Cfpdirectval<float>("leftfrontcoxagoal",this,&(controllerp->linkgoal[0][0][0])));
  proxyp->addaccessor(varp,outputtype);
  addvar(varp=new Cfpdirectval<float>("leftfrontfemurgoal",this,&(controllerp->linkgoal[0][0][1])));
  proxyp->addaccessor(varp,outputtype);
  addvar(varp=new Cfpdirectval<float>("leftfronttibiagoal",this,&(controllerp->linkgoal[0][0][2])));
  proxyp->addaccessor(varp,outputtype);
  addvar(varp=new Cfpdirectval<float>("leftmiddlecoxagoal",this,&(controllerp->linkgoal[0][1][0])));
  proxyp->addaccessor(varp,outputtype);
  addvar(varp=new Cfpdirectval<float>("leftmiddlefemurgoal",this,&(controllerp->linkgoal[0][1][1])));
  proxyp->addaccessor(varp,outputtype);
  addvar(varp=new Cfpdirectval<float>("leftmiddletibiagoal",this,&(controllerp->linkgoal[0][1][2])));
  proxyp->addaccessor(varp,outputtype);
  addvar(varp=new Cfpdirectval<float>("leftrearcoxagoal",this,&(controllerp->linkgoal[0][2][0])));
  proxyp->addaccessor(varp,outputtype);
  addvar(varp=new Cfpdirectval<float>("leftrearfemurgoal",this,&(controllerp->linkgoal[0][2][1])));
  proxyp->addaccessor(varp,outputtype);
  addvar(varp=new Cfpdirectval<float>("leftreartibiagoal",this,&(controllerp->linkgoal[0][2][2])));
  proxyp->addaccessor(varp,outputtype);
  addvar(varp=new Cfpdirectval<float>("rightfrontcoxagoal",this,&(controllerp->linkgoal[1][0][0])));
  proxyp->addaccessor(varp,outputtype);
  addvar(varp=new Cfpdirectval<float>("rightfrontfemurgoal",this,&(controllerp->linkgoal[1][0][1])));
  proxyp->addaccessor(varp,outputtype);
  addvar(varp=new Cfpdirectval<float>("rightfronttibiagoal",this,&(controllerp->linkgoal[1][0][2])));
  proxyp->addaccessor(varp,outputtype);
  addvar(varp=new Cfpdirectval<float>("rightmiddlecoxagoal",this,&(controllerp->linkgoal[1][1][0])));
  proxyp->addaccessor(varp,outputtype);
  addvar(varp=new Cfpdirectval<float>("rightmiddlefemurgoal",this,&(controllerp->linkgoal[1][1][1])));
  proxyp->addaccessor(varp,outputtype);
  addvar(varp=new Cfpdirectval<float>("rightmiddletibiagoal",this,&(controllerp->linkgoal[1][1][2])));
  proxyp->addaccessor(varp,outputtype);
  addvar(varp=new Cfpdirectval<float>("rightrearcoxagoal",this,&(controllerp->linkgoal[1][2][0])));
  proxyp->addaccessor(varp,outputtype);
  addvar(varp=new Cfpdirectval<float>("rightrearfemurgoal",this,&(controllerp->linkgoal[1][2][1])));
  proxyp->addaccessor(varp,outputtype);
  addvar(varp=new Cfpdirectval<float>("rightreartibiagoal",this,&(controllerp->linkgoal[1][2][2])));
  proxyp->addaccessor(varp,outputtype);
}





