// cfpmacros.C

#include <stdlib.h>
#include <stdio.h>

#include "cfpmacros.h"
#include "JRutils.h"






CfpMacroClass::CfpMacroClass(Configpp *inconfigppp,string inname)
{
  // create the macro
  configppp=inconfigppp;
  name=inname;

  // create the CfpMacroObject which will be responsible for allocating others
  allocatorobj=new CfpMacroObject(inconfigppp,this);

  // register the allocator object which will receive instantiation requests
  CfpFactory *sfactp=configppp->getfactoryp();
  cfpclassp=sfactp->registerclass("usermacro",name,allocatorobj);
}


CfpMacroClass::~CfpMacroClass()
{
  // delete the variable list
  list<CfpMemberVar *>::iterator li=varlist.begin();
  list<CfpMemberVar *>::iterator lend=varlist.end();
  // Macro 'variables' are just simple CfpVal accessors which manage their own data.
  while (li!=lend)
    {
      delete (*li);
      li++;
    }

  // delete the allocator object we created
  delete allocatorobj;
  allocatorobj=NULL;
}





void CfpMacroClass::addarg(string vaname,CfpVal &vadefault,string valabel)
{
  // add the arg to our internal list.  we will use it when we register later.
  cfpclassp->addarg(vaname,vadefault,false,valabel);
}


void CfpMacroClass::addvar(string vaname,CfpVal &vadefault,string valabel)
{
  CfpMemberVar *varp;

  // add the arg to our internal list.  we will use it when we register later.
  varp=cfpclassp->addvar(vaname,vadefault,false,valabel);
  // we have to keep track of variables since we will need them later
  varlist.push_back(varp);
}



































//--------------------------------------------------------------------------------
//---- CfpMacroObject::CfpMacroObject()
//--------------------------------------------------------------------------------

CfpMacroObject::CfpMacroObject(Configpp *cfpfp,CfpMacroClass *inmacroclassp)
  :CfpObject()
{
  // this is the constructor that gets called once for each new macro class
  configppp=cfpfp;
  macroclassp=inmacroclassp;
}







//--------------------------------------------------------------------------------
//---- CfpMacroObject::CfpMacroObject()
//--------------------------------------------------------------------------------

CfpMacroObject::CfpMacroObject(Configpp *cfpfp,CfpClass *inclassp,string oname,CfpMacroClass *inmacroclassp)
  :CfpObject(cfpfp,inclassp,oname)
{
  // this is the constructor that gets called every time the user creates a new
  // instantiated instance of a macro. This function and makenew, the next, are partners.
  macroclassp=inmacroclassp;
}


//--------------------------------------------------------------------------------
//---- CfpMacroObject::makenew()
//--------------------------------------------------------------------------------

CfpObject *CfpMacroObject::makenew(CfpClass *inclassp,string objname)
{
  // create a new instance of ourselves, the wrapper.
  // Important: instantiating the wrapped class will come later (in function 'construct').
  CfpObject *newp=NULL;

  newp=new CfpMacroObject(configppp,inclassp,objname,macroclassp);
  return newp;
}






void CfpMacroObject::registerwrappers(CfpFactory *sfactp)
{
  // for macros, the spawer does not register the wrapper accessors and variables
  // instead this is done from the swmacro class while the macro is being described
}







//--------------------------------------------------------------------------------
//---- CfpMacroObject::destruct()
//--------------------------------------------------------------------------------

void CfpMacroObject::destruct()
{
  // de-instantiate the wrapped object and free memory
  instantiated=false;
}





//--------------------------------------------------------------------------------
//---- CfpMacroObject::construct() ------------------------------------------------
//--------------------------------------------------------------------------------

void CfpMacroObject::construct()
{
  // instantiate the wrapped object.  this is where we actually do late (lazy) instantiation.
  // if we are a wrapper for another class, we would allocate that target wrapped class here.
  // and save it to a local variable.  otherwise this is its own wrapper.
  // the args will have been filled by now.

  // create the wrapped object

  // SO, what we want to do now is tell the configpp parser to parse our stored codeblock,
  //  just as if it were code that the user specified within the Macro object.

  // add variable accessors
  addaccessors();

  // set the instantiated flag
  instantiated=true;
}


//--------------------------------------------------------------------------------
//---- CfpMacroObject::done()
//--------------------------------------------------------------------------------

CfpVal CfpMacroObject::done()
{
  // Macro instantiation is complete.  user has specified any arguments, etc.
  
  return retval;
}














//--------------------------------------------------------------------------------
//---- CfpMacroObject::addaccessors()
//--------------------------------------------------------------------------------

void CfpMacroObject::addaccessors()
{
  // here we want to create the variables that were defined in by macro class

  list<CfpMemberVar *>::iterator li=(macroclassp->varlist).begin();
  list<CfpMemberVar *>::iterator lend=(macroclassp->varlist).end();
  // Macro 'variables' are just simple CfpVal accessors which manage their own data.
  while (li!=lend)
    {
      addvar(new CfpValAccessor((*li)->getname(),this));
      li++;
    }
}





string CfpMacroObject::postparse()
{
  // this function is called by the parser right before the object is completed.
  // it allows us to specify some extra Configpp code to be executed as if it were within (and at the end of)
  //  the object scope.

  return macroclassp->codeblock;
}
