// SWpdcontrol.C

#include "swpdcontrol.h"
#include "pdcontroller.h"


//--------------------------------------------------------------------------------
//---- SWpdcontrol_obj::SWpdcontrol_obj()
//--------------------------------------------------------------------------------

SWpdcontrol_obj::SWpdcontrol_obj(Configpp *cfpfp,CfpClass *inclassp,string oname)
  :CfpObject(cfpfp,inclassp,oname)
{
  // this is the constructor that gets called every time the user creates a new
  // instantiated object of the class. This function and makenew, the next, are partners.
}



//--------------------------------------------------------------------------------
//---- SWpdcontrol_obj::makenew()
//--------------------------------------------------------------------------------

CfpObject *SWpdcontrol_obj::makenew(CfpClass *inclassp,string objname)
{
  // create a new instance of ourselves, the wrapper.
  // Important: instantiating the wrapped class will come later (in function 'construct').

  return (new SWpdcontrol_obj(configppp,inclassp,objname));
}





//--------------------------------------------------------------------------------
//---- SWpdcontrol_obj()
//--------------------------------------------------------------------------------

void SWpdcontrol_obj::registerwrappers(CfpFactory *sfactp)
{
  // this is the constructor that gets called ONCE when the class is registered with the factory.
  CfpClass *newclassp;

  newclassp=sfactp->registerclass("Pdcontrol","Controller.PD",this);
  newclassp->addvar("trigger","",false,"accessor for deciding when to iterate the controller)");
  newclassp->addvar("pconstant","0",true,"proportional constant of controller");
  newclassp->addvar("dconstant","0",true,"derivitive constant of controller");
  newclassp->addvar("multiplier","1",true,"multiplier of constants for fast scaling");
  newclassp->addvar("current","",false,"source for current state");
  newclassp->addvar("goal","",false,"source for goal state");
  newclassp->addvar("pcomponent","",false,"proportional component of output");
  newclassp->addvar("dcomponent","",false,"derivitive component of output");
  newclassp->addvar("output","",false,"output value");
  newclassp->addvar("error","",false,"tracking error (distance from goal) in radians");
  newclassp->addvar("maximumoutput","",false,"maximum allowed output of controller");
}




//--------------------------------------------------------------------------------
//---- SWpdcontrol_obj::destruct()
//--------------------------------------------------------------------------------

void SWpdcontrol_obj::destruct()
{
  // de-instantiate the wrapped object and free memory

  instantiated=false;
  if (classname()=="Controller.PD")
    {
      // delete the proxy that was created
      delete proxyp;
    }
}














//--------------------------------------------------------------------------------
//---- SWpdcontrol_obj::construct() ------------------------------------------------
//--------------------------------------------------------------------------------

void SWpdcontrol_obj::construct()
{
  // instantiate the wrapped object.  this is where we actually do late (lazy) instantiation.
  // if we are a wrapper for another class, we would allocate that target wrapped class here.
  // and save it to a local variable.  otherwise this is its own wrapper.
  // the args will have been filled by now.

  // create the wrapped object

  if (classname()=="Controller.PD")
    construct_pdcontroller();

  // add accessors
  addaccessors();

  // set the instantiated flag
  instantiated=true;
}


//--------------------------------------------------------------------------------
//---- SWpdcontrol_obj::done()
//--------------------------------------------------------------------------------

CfpVal SWpdcontrol_obj::done()
{
  // the object is done (ie no more children) return any return value
  
  return retval;
}





//--------------------------------------------------------------------------------
//---- SWpdcontrol_obj::construct_pdcontroller()
//--------------------------------------------------------------------------------

void SWpdcontrol_obj::construct_pdcontroller()
{
  // build the pdcontroller
  controllerp=new PDcontroller();

  // build a trigger to tell us when to run
  //  val=argval("trigger");
  //  updateacc=val.get_vaccesspointer();
  //  makelistener(updateacc,new CfpVFAMnotify<PDcontroller>("_update",this,controllerp,&(PDcontroller::process)));
}












//--------------------------------------------------------------------------------
//---- SWpdcontrol_obj::addaccessors()
//--------------------------------------------------------------------------------

void SWpdcontrol_obj::addaccessors()
{
  CfpVAccessor *v1,*v2,*v3,*v4,*v5,*v6,*v7,*v8,*v9;
  CfpVAccessor *triggerp;

  // add the normal accessors
  addvar(v1=new CfpVFAMsetval<PDcontroller,float,float>("pconstant",this,controllerp,&(PDcontroller::setpconstant),&(PDcontroller::getpconstant)));
  addvar(v2=new CfpVFAMsetval<PDcontroller,float,float>("dconstant",this,controllerp,&(PDcontroller::setdconstant),&(PDcontroller::getdconstant)));
  addvar(v3=new CfpVFAMsetval<PDcontroller,float,float>("current",this,controllerp,&(PDcontroller::setcurrent),&(PDcontroller::getcurrent)));
  addvar(v4=new CfpVFAMsetval<PDcontroller,float,float>("goal",this,controllerp,&(PDcontroller::setgoal),&(PDcontroller::getgoal)));
  addvar(v5=new CfpVFAMgetval<PDcontroller,float>("pcomponent",this,controllerp,&(PDcontroller::getpcomponent)));
  addvar(v6=new CfpVFAMgetval<PDcontroller,float>("dcomponent",this,controllerp,&(PDcontroller::getdcomponent)));
  addvar(v7=new CfpVFAMgetval<PDcontroller,float>("output",this,controllerp,&(PDcontroller::getoutput)));
  addvar(v9=new CfpVFAMgetval<PDcontroller,float>("error",this,controllerp,&(PDcontroller::getdist)));
  addvar(v8=new CfpVFAMsetval<PDcontroller,float,float>("maximumoutput",this,controllerp,&(PDcontroller::setmaxout),&(PDcontroller::getmaxout)));
  addvar(v1=new CfpVFAMsetval<PDcontroller,float,float>("multiplier",this,controllerp,&(PDcontroller::setmultiplier),&(PDcontroller::getmultiplier)));


  // build a trigger proxy
  proxyp=new CfpVAccTriggerProxy("trigger");

  // now tell the proxy about the input variables that must be up-to-date when we are triggered
  proxyp->addaccessor(v1,inputtype);
  proxyp->addaccessor(v2,inputtype);
  proxyp->addaccessor(v3,inputtype);
  proxyp->addaccessor(v4,inputtype);
  proxyp->addaccessor(v8,outputtype);

  // now create the trigger for the controller
  triggerp=new CfpVFAMnotify<PDcontroller>("trigger",this,controllerp,&(PDcontroller::process));
  // add it to the proxy
  proxyp->addaccessor(triggerp,triggertype);
  // and add the *proxies* trigger to the visible variables that the user has access to
  addvar(proxyp->getproxytriggerp());

  // and lastly add the output variables, so they deliver their updates after trigger finishes
  proxyp->addaccessor(v5,outputtype);
  proxyp->addaccessor(v6,outputtype);
  proxyp->addaccessor(v7,outputtype);
  proxyp->addaccessor(v9,outputtype);
}



