// Cfpmain.C
// The default main class for a generic configpp interpreter
// The Configpp library is NOT build with this file.  It is used for standalone
//  testing and debugging.  Normally a user will have a replacement version of main().


#include "cfpinc.h"
#include "cfpfactory.h"


//Not implemented in gcc yet (2.8.1)
//using namespace std;
//using namespace Configpp;



// Unfortunately, for now we use a global pointer to a configpp object in order to let the CfpVal objects
//  (our primitive data type), resolve references to accessors.  We should fix this so that it is not
//  performed within the CfpVal class, but for now.
Configpp *globalconfigppp;




void usage()
{
  // Tell the user how to invoke the program
  cerr << "\n";
  cerr << "Error.  Config++ interpreter usage:   cfp [commandline_options] filename\n";
  Configpp::show_commandline_options();
  cerr << "\n\n";
}





void main(int argc, char **argv)
{
  // Create the class factory and the interpreter
  bool bretv;
  string fname;
  string errstring;
  Configpp *configppp;
  CfpFactory *factoryp;

  // get a pointer to the global factory
  factoryp=GlobalFactoryp();
  
  // allocate the configpp class and tell it about the global factory, and any commandline args
  globalconfigppp=configppp=new Configpp("TestApp",factoryp,argc,argv);

  // now tell the global factory about the commandline
  factoryp->viewcommandline(argc,argv);

  // anounce that the interpreter has been invoked
  configppp->welcome();

  // compress any commandline arguments that may have been zeroed-out by configpp interp
  // and get the first *remaining* arg, which better be a file name.
  fname=JRstripcomarg(argc,argv);
  if (fname=="")
    {
      usage();
      delete configppp;
      exit(EXIT_FAILURE);
    }

  // check if there is invalid arguments
  if (JRinvalidcomargs(argc,argv,true))
    {
      Configpp::show_commandline_options();
      delete configppp;
      exit(EXIT_FAILURE);
    }

  // now tell the interpreter to run a file (a false return means it failed)
  bretv=configppp->parsefile(fname);
  if (bretv)
    bretv=configppp->run();

  // now clean up by deleteing the interpreter
  delete configppp;

  // lastly, exit with proper code
  if (!bretv)
    exit(EXIT_FAILURE);
  exit(EXIT_SUCCESS);
}
