// cfpobject.h
// This file describes the Configpp CfpObject class, the class which is subclasses
//  to yield user-derived objects, and so can be expected to be "included" from
//  user files.

#ifndef INC_cfpobjecth
#define INC_cfpobjecth

#include "cfpinc.h"

//Not implemented in gcc yet (2.8.1)
//namespace Configpp{






//--------------------------------------------------------------------------------
//---- CfpObject -----------------------------------------------------------------
//--------------------------------------------------------------------------------

//: The base class for Configpp buildable classes
// This is the virtual object which will be used to subclass user derived objects.

class CfpObject
{
  friend class CfpClass;
protected:
  Configpp *configppp;							// pointer to main configpp object
  CfpClass *classp;							// pointer to creating class
  string name;								// name of any object
  bool instantiated;							// is the wrapper/class object instantiated
  bool cfphandle_memory;						// should interpreter handle memory
  bool cfphandle_file;							// should interpreter handle file save/load
  void *localp;								// pointer to local wrapped object
  CfpVal retval;							// return value from object
  long idnum;								// unique idnum (ordered by creationtime)
  bool movieplaybackflag;
private:
  map<string, CfpVAccessor *> argmanager;				// map version of var container
  map<string, CfpVAccessor *> varmanager;				// map version of var container
  map<string, CfpFuncAccessor *> funcmanager;				// map version of func container
  list<CfpVAccessor *> localaccs;					// for storing accessors we create
protected:
  CfpObject *parentobjp;						// pointer to the objects parent
  list<CfpObject *> childmanager;					// collection of adopted children
public:
  CfpObject();      						        // constructor;
  CfpObject(Configpp *incfpfp,CfpClass *inclassp,string inname);		// constructor
public:
  virtual void registerwrappers(CfpFactory *factoryp);// =0;             // one-shot registration, must be subclassed
  virtual void viewcommandline(int &argc,char **inargv) {;};// one-shot peek at commandline args
public:
  virtual void preinterpret(CfpClass *inclassp) {;};			// before interpretation begins
  virtual void postinterpret(CfpClass *inclassp) {;};			// after interpretation ends
  virtual CfpObject *makenew(CfpClass *inclassp,string objname);	// instantiate a new object
  virtual CfpVal done() {return retval;};				// object scope is over, return retval
  virtual void destruct() {instantiated=false;};						// destruct
  virtual void construct(string argstr,string keyword,map<string,CfpVal *> *argmap);		// construct
  virtual void construct();									// construct;
public:
  virtual CfpAdoptType adoptionquery(CfpObject *objp);			// will we adopt the child?
  virtual CfpVal invokef(string fname,string argstr,string keyword,map<string,CfpVal *> *argmap); // invoke func
public:
  void setconfigppp(Configpp *inconfigppp){configppp=inconfigppp;};
  void setname(string inname) {name=inname;};				// accessor
  void setclassp(CfpClass *inclassp) {classp=inclassp;};		// accessor
  string getname() {return name;};					// accessor
  string classname();							// accessor
  string subclass();							// accessor
  string baseclass();							// accessor
  string prettyname();							// accessor
  string prettyiname();							// accessor
  string gettrail();							// accessor
  string getitrail();							// accessor
  bool isinstantiated() {return instantiated;};				// accessor
  long getidnum(){return idnum;};					// accessor
  void setidnum(long inidnum){idnum=inidnum;};				// accessor
  CfpVAccessor *lookupva(string inname,bool throwerr=true);		// accessor
  CfpVAccessor *lookuparg(string inname,bool throwerr=true);		// accessor
  CfpVAccessor *lookupvar(string inname,bool throwerr=true);		// accessor
  CfpFuncAccessor *lookupfunc(string inname,bool throwerr=true);	// accessor
  void sethandle_memory(bool inval=true){cfphandle_memory=true;};	// accessor
  void sethandle_file(bool inval=true){cfphandle_file=true;};		// accessor
  bool gethandle_memory(){return cfphandle_memory;};			// accessor
  bool gethandle_file(){return cfphandle_file;};			// accessor
  CfpVal argval(string inname);						// accessor
  CfpVal argval(string inname,string defaultstr);			// accessor
  CfpVal varval(string inname);						// accessor
  CfpVal varval(string inname,string defaultstr);			// accessor
  void setargval(string inname,string valu);				// accessor
  void setargval(string inname,float valu);				// accessor
  CfpVal argval(string inname,int vecindex);			        // accessor
  CfpVal argval(string inname,int vecindex,string defaultstr);          // accessor
  void setargval(string inname,int vecindex,float valu);		// accessor
  void setargval(string inname,int vecindex,string valu);		// accessor
  bool argdefined(string inname);					// accessor
  bool vardefined(string inname);					// accessor
  CfpClass *getclassp(){return classp;};				// accessor
  CfpObject *getparentobjp() {return parentobjp;};			// pointer to the objects parent
  virtual bool scopesearchable() { return true;};                       // when searching for ids should we lookin here (seemacros)
public:
  bool argeq(string inname,string valstring);				// is arg eq string
  bool argeq(string inname,double valnum);				// is arg eq string
public:
  void *getlocalp() {return localp;};				// return a voidcast pointer to wrappedobj
  void setlocalp(void *vp) {localp=vp;};			// set the local pointer
  bool isa(string typestr);					// is the object a derivitive of typestr?
  bool hasa(string typenam);					// does the object have a child queried?
  CfpObject *hasobj(string typenam);				// does the object have a child queried?
protected:
  CfpVAccessor *addarg(CfpVAccessor *inaccp);					// add new object arg
  CfpVAccessor *addvar(CfpVAccessor *inaccp);					// add new object var
  CfpVAccessor *addvarcopy(string cpname,CfpVAccessor *inaccp);		// add object var from copy
  CfpFuncAccessor *addfunc(CfpFuncAccessor *infaccp);				// add new object func
  CfpVAccessor *makelistener(CfpVAccessor *sourcep,CfpVAccessor *targetp,int toindex=-1,int fromindex=-1);	// add listener accessor
private:
  void makeargmembers();						// create local args from class members
public:
  virtual bool moviesave(FILE *fp) {return false;};
  virtual bool movieload(FILE *fp) {return false;};
  virtual void setmovieplaybackflag(bool inflag=true) {movieplaybackflag=inflag;};
public:
  virtual string preparse() {return "";};
  virtual string postparse() {return "";};
};








//}
// Namespace close-brace
#endif
