// cfpstdlib.C

#include "cfpinc.h"
#include "datastream.h"

//Not implemented in gcc yet (2.8.1)
//namespace Configpp{






//--------------------------------------------------------------------------------
//---- CfpstdInclude -------------------------------------------------------------
//--------------------------------------------------------------------------------

CfpstdInclude::CfpstdInclude(Configpp *cfpfp,CfpClass *inclassp,string oname)
  :CfpObject(cfpfp,inclassp,oname)
{
  // this is the constructor that gets called every time the user creates a new
  // instantiated object of the class.
  // This function and makenew, the next, are partners.
}


//--------------------------------------------------------------------------------
//---- CfpstdInclude::makenew() --------------------------------------------------
//--------------------------------------------------------------------------------

CfpObject *CfpstdInclude::makenew(CfpClass *inclassp,string objname)
{
  // create a new instance of ourselves, the wrapper.
  // Important: instantiating the wrapped class will come later (in function 'construct').

  return (new CfpstdInclude(configppp,inclassp,objname));
}



//--------------------------------------------------------------------------------
//---- CfpstdInclude -------------------------------------------------------------
//--------------------------------------------------------------------------------

void CfpstdInclude::registerwrappers(CfpFactory *sfactp)
{
  // this is the constructor that gets called ONCE when the class is registered with the factory.
  CfpClass *newclassp;

  newclassp=sfactp->registerclass("CfpStd","Includefile",this);
  sfactp->addalias("include","Includefile");

  // Add the members to the CLASS
  newclassp->addarg("filename","",false,"the file name to include (as a var)");
}




//--------------------------------------------------------------------------------
//---- CfpstdInclude::construct() ------------------------------------------------
//--------------------------------------------------------------------------------

void CfpstdInclude::construct()
{
  // instantiate the wrapped object.  this is where we actually do late (lazy) instantiation.
  // if we are a wrapper for another class, we would allocate that target wrapped class here.
  // and save it to a local variable.  otherwise this is its own wrapper.

  // set the instantiated flag
  instantiated=true;

  // now just tell the parser to load in the new file and treat it like inline code
  string filename=(string)argval("filename");
  configppp->pushfile(filename);
}




//--------------------------------------------------------------------------------
//---- CfpstdInclude::destruct() -------------------------------------------------
//--------------------------------------------------------------------------------

void CfpstdInclude::destruct()
{
  // de-instantiate the wrapped object.
  // we wrap ourselves so there is no need to delete any 'wrapped class', just set the instantiated flag to false
  instantiated=false;
}



















//--------------------------------------------------------------------------------
//---- CfpstdInline -------------------------------------------------------------
//--------------------------------------------------------------------------------

CfpstdInline::CfpstdInline(Configpp *cfpfp,CfpClass *inclassp,string oname)
  :CfpObject(cfpfp,inclassp,oname)
{
  // this is the constructor that gets called every time the user creates a new instance.
  CfpToken toke;
  CfpTokenType toketype;
  int braces;

  // we are going to swallow up everything within the next block before we retun to the parser
  // and open a copy of the parsing file as a new file with start and end pointing to
  //  the inline data inside the block.

  filename=configppp->getcurfilename();
  if (filename=="")
    throw CfpError("Blocks only allowed inside files.","",file,false,configppp);
  sisp=configppp->getcursource();

  // Read to end of block to get endpos - note that we don't require what's in the block to be valid
  //  toke=(configppp->getlexerp())->readtoken();
  toke=(configppp->getlexerp())->readpuretoken();
  toketype=toke.gettype();
  if (toketype != openbraceSym)
    throw CfpError("Block contents need to be enclosed in a set of braces { }.","the improper token was '"+toke.getstr()+"'.",file,false,configppp);
  braces=1;

  startpos=configppp->getcurpos();
  for (;;)
    {
      endpos=configppp->getcurpos();
      toke=(configppp->getlexerp())->readtoken();
      toketype=toke.gettype();
      if (toketype == eofSym)
	break;
      if (toketype == openbraceSym)
	++braces;
      if (toketype == closebraceSym)
	{
	  if (--braces==0)
	    break;
	}
    }
  if (toketype!=closebraceSym)
    throw CfpError("Block contents need to be enclosed in a set of braces { }, and can only consist of numbers and strings.","the improper token was '"+toke.getstr()+"'.",file,false,configppp);

  // now importantly we have to push on a semicolon to replace the block that we stole from parser
  (configppp->getlexerp())->pushtoken(CfpToken(semicolonSym,";"));
}


//--------------------------------------------------------------------------------
//---- CfpstdInline::makenew() --------------------------------------------------
//--------------------------------------------------------------------------------

CfpObject *CfpstdInline::makenew(CfpClass *inclassp,string objname)
{
  // create a new instance of ourselves, the wrapper.
  // Important: instantiating the wrapped class will come later (in function 'construct').

  return (new CfpstdInline(configppp,inclassp,objname));
}




//--------------------------------------------------------------------------------
//---- CfpstdInline -------------------------------------------------------------
//--------------------------------------------------------------------------------

void CfpstdInline::registerwrappers(CfpFactory *sfactp)
{
  // this is the constructor that gets called ONCE when the class is registered with the factory.
  CfpClass *newclassp;

  // register the class
  newclassp=sfactp->registerclass("CfpStd","Block.Data",this);
  newclassp=sfactp->registerclass("CfpStd","Block.Code",this);
}




//--------------------------------------------------------------------------------
//---- CfpstdInline::construct() ------------------------------------------------
//--------------------------------------------------------------------------------

void CfpstdInline::construct()
{
  // instantiate the wrapped object.  this is where we actually do late (lazy) instantiation.
  // if we are a wrapper for another class, we would allocate that target wrapped class here.
  // and save it to a local variable.  otherwise this is its own wrapper.

  // in this case, the object wraps itself, so we don't have to create the wrapped object

  // set the instantiated flag
  instantiated=true;
}


//--------------------------------------------------------------------------------
//---- CfpstdInline::destruct() -------------------------------------------------
//--------------------------------------------------------------------------------

void CfpstdInline::destruct()
{
  // de-instantiate the wrapped object.
  // we wrap ourselves so there is no need to delete any 'wrapped class', just set the instantiated flag to false
  instantiated=false;
}




//--------------------------------------------------------------------------------
//---- CfpstdInline::gulpstring() ------------------------------------------------
//--------------------------------------------------------------------------------

string CfpstdInline::gulpstring()
{
  // read in the entire stream into a string and return the string
  // first position file to start
  // then read in the exact length of data into string
  string gulpedstr;
  char *tempstr;
  int len;
  istream *isp=sisp->getstreamp();
  long origpos=isp->tellg();

  // length of inline block embedded in stream
  len=endpos-startpos;
  // allocate space for temp char buffer
  tempstr=new char[len+1];
  // read the buffer
  isp->seekg(startpos,ios::beg);
  isp->read(tempstr,len);
  tempstr[len]='\0';
  // convert to string
  gulpedstr=(string)tempstr;
  // release temp buffer
  delete [] tempstr;
  // now return stream to where we found it
  isp->seekg(origpos,ios::beg);
  
  return gulpedstr;
}
























//--------------------------------------------------------------------------------
//---- CfpstdWire -------------------------------------------------------------
//--------------------------------------------------------------------------------

CfpstdWire::CfpstdWire(Configpp *cfpfp,CfpClass *inclassp,string oname)
  :CfpObject(cfpfp,inclassp,oname)
{
  // this is the constructor that gets called every time the user creates a new
  // instantiated object of the class.
  // This function and makenew, the next, are partners.

  // initialize
  fromvarp=NULL;
  tovarp=NULL;
  initialize=true; // if true, a signal will be sent along wire as soon as it is created.
}


//--------------------------------------------------------------------------------
//---- CfpstdWire::makenew() --------------------------------------------------
//--------------------------------------------------------------------------------

CfpObject *CfpstdWire::makenew(CfpClass *inclassp,string objname)
{
  // create a new instance of ourselves, the wrapper.
  // Important: instantiating the wrapped class will come later (in function 'construct').

  return (new CfpstdWire(configppp,inclassp,objname));
}




//--------------------------------------------------------------------------------
//---- CfpstdWire -------------------------------------------------------------
//--------------------------------------------------------------------------------

void CfpstdWire::registerwrappers(CfpFactory *sfactp)
{
  // this is the constructor that gets called ONCE when the class is registered with the factory.
  CfpClass *newclassp;

  newclassp=sfactp->registerclass("CfpStd","Wire",this);
  newclassp->addarg("from","",true,"the source to read from");
  newclassp->addarg("to","",true,"the source to write to");
  newclassp->addarg("value","",true,"optional starting value");
}




//--------------------------------------------------------------------------------
//---- CfpstdWire::construct() ------------------------------------------------
//--------------------------------------------------------------------------------

void CfpstdWire::construct()
{
  // instantiate the wrapped object.  this is where we actually do late (lazy) instantiation.
  // if we are a wrapper for another class, we would allocate that target wrapped class here.
  // and save it to a local variable.  otherwise this is its own wrapper.

  // set the instantiated flag
  instantiated=true;

  // get the from and to, and be clever about it.
  fromval=argval("from");
  toval=argval("to");

  // eventually we would like to be clever about guessing the from and to if we are inside
  //  another object, but for now we insist that they both be specified explicitly.

  int fromindex,toindex;
  fromvarp=fromval.get_vaccesspointer();
  fromindex=fromval.get_vaccessindex();
  tovarp=toval.get_vaccesspointer();
  toindex=toval.get_vaccessindex();

  // make sure these exist
  if (fromvarp==NULL)
    {
      // error no source
      throw CfpError("the source ('From' field) for the wire is NULL","",syntactical,false,configppp);
    }
  else if (tovarp==NULL)
    {
      // error no target
      throw CfpError("the target ('Source' field) for the wire is NULL","",syntactical,false,configppp);
    }

  configppp->debug("creating a wire from " + fromvarp->getname() + " to " + tovarp->getname() +".\n");
  laststubp=(fromvarp->addlistener(tovarp,fromindex,toindex));
  //  cout << "creating a wire from " + fromvarp->getname() + " to " + tovarp->getname() +".\n";

  // should we send an initial signal from source to target?
  if (initialize)
    {
      if (argdefined("value"))
	{
	  // if the user has defined an initial value, then use that
	  //	  cout << "initializing with value.\n";
	  CfpVal initval=argval("value");
	  tovarp->set(&initval,toindex);
	}
      if (fromvarp->getable())
	{
	  // otherwise if the source is getable, so we just get it and set it
	  //	  cout << "initializing with getable.\n";
	  laststubp->retrieveandset();
	}
    }

}




//--------------------------------------------------------------------------------
//---- CfpstdWire::done() --------------------------------------------------------
//--------------------------------------------------------------------------------

CfpVal CfpstdWire::done()
{
  // children have been added, so now we can finalize the wire
  
  // We will eventually need to handle forward referencing; to do this we might just put off instantiating
  // all wires until all objects have been instantiated, then create all wires in their original order.

  // some times a single variable needs dif. accessors for input and output, but we want this
  //  to be transparent, so that if user is refering to a var as input, we first look for an input version
  //  and if it's output, we look for an output version


  return retval;
}






//--------------------------------------------------------------------------------
//---- CfpstdWire::addobjmembers() --------------------------------------------
//--------------------------------------------------------------------------------

void CfpstdWire::addobjmembers()
{
  // add the member variables to the OBJECT
}


//--------------------------------------------------------------------------------
//---- CfpstdWire::destruct() -------------------------------------------------
//--------------------------------------------------------------------------------

void CfpstdWire::destruct()
{
  // de-instantiate the wrapped object.
  // we wrap ourselves so there is no need to delete any 'wrapped class', just set the instantiated flag to false
  instantiated=false;
}






































//--------------------------------------------------------------------------------
//---- CfpstdSplice -------------------------------------------------------------
//--------------------------------------------------------------------------------

CfpstdSplice::CfpstdSplice(Configpp *cfpfp,CfpClass *inclassp,string oname)
  :CfpObject(cfpfp,inclassp,oname)
{
  // this is the constructor that gets called every time the user creates a new
  // instantiated object of the class.
  // This function and makenew, the next, are partners.
}


//--------------------------------------------------------------------------------
//---- CfpstdSplice::makenew() --------------------------------------------------
//--------------------------------------------------------------------------------

CfpObject *CfpstdSplice::makenew(CfpClass *inclassp,string objname)
{
  // create a new instance of ourselves, the wrapper.
  // Important: instantiating the wrapped class will come later (in function 'construct').

  return (new CfpstdSplice(configppp,inclassp,objname));
}




//--------------------------------------------------------------------------------
//---- CfpstdSplice -------------------------------------------------------------
//--------------------------------------------------------------------------------

void CfpstdSplice::registerwrappers(CfpFactory *sfactp)
{
  // this is the constructor that gets called ONCE when the class is registered with the factory.
  CfpClass *newclassp;

  newclassp=sfactp->registerclass("CfpStd","Splice",this);

  // register an optional trigger (if not specified, info will process and propagate when source changes)
  newclassp->addarg("trigger","",false,"the trigger source to cause info to propagate through splice");
}




//--------------------------------------------------------------------------------
//---- CfpstdSplice::constructsetup() --------------------------------------------
//--------------------------------------------------------------------------------

void CfpstdSplice::constructsetup()
{
  // instantiate the wrapped object.  this is where we actually do late (lazy) instantiation.
  // if we are a wrapper for another class, we would allocate that target wrapped class here.
  // and save it to a local variable.  otherwise this is its own wrapper.

  // set the instantiated flag
  instantiated=true;

  // create the accessors for the inputval and outputval
  inputp=new CfpValAccessor("_from",this,&inputval);
  outputp=new CfpValAccessor("_to",this,&outputval);
}




//--------------------------------------------------------------------------------
//---- CfpstdSplice::done() --------------------------------------------------------
//--------------------------------------------------------------------------------

CfpVal CfpstdSplice::donesetup()
{
  // children have been added, so now we can finalize the splice
  CfpVAccstub *newstubp;
  CfpstdWire *wirep;
  CfpVal triggerval;
  int triggerindex;
  CfpVAccessor *processp;

  // get a pointer to the parent wire
  wirep=(CfpstdWire *)(configppp->parentinstantiate(this,"Wire",true));

  // okay, first get the last listener stub on the wire
  laststubp=wirep->getlaststubp();

  // splice in the splices inputs and outputs
  newstubp=(laststubp->listenersplice(inputp,-1,outputp,-1));

  //  // now indicate the the filter has the prior input as a spaker
  //  inputp->addspeaker(laststubp);

  // now set the trigger for the splice
  if (argdefined("trigger"))
    {
      // user has specified a dedicated trigger which will tell us when to produce an output
      triggerval=argval("trigger");
      triggerp=triggerval.get_vaccesspointer();
      triggerindex=triggerval.get_vaccessindex();
      if (!((laststubp->getsource())->willnotify()))
	{
	  // our input source has told us that it will NOT notify us when its value changes, ie we must manually request it
	  //  when we want it.
	  if (!((laststubp->getsource())->getable()))
	    {
	      //  The input source is marked as one which is NOT getable, so we can't very well get values manually
	      configppp->warning("Input source '"+(laststubp->getsource())->getname()+"' indicates that it is not getable, but Splice was planning to retrieve values from it.  Splice may never be triggered.");
	      manualretrieve=false;
	    }
	  else
	    manualretrieve=true;
	}
      else
	manualretrieve=false;
    }
  else
    {
      // no explicit trigger was specified, so we assume that we should trigger whenever our input changes
      if (!((laststubp->getsource())->willnotify()))
	{
	  // the user has told us that the input source will be our trigger,
	  //  but the input source is marked as one which doesn not notify people of changes
	  configppp->warning("Input source '"+(laststubp->getsource())->getname()+"' is being used as a trigger for a splice, but it indicates that it may change without notifying the splice, so splice may never be triggered.");
	}
      manualretrieve=false;
      // test
      // Add a notification from input variable to filter input
      // add a speaker pointer from input of filter to output of filter,
      // so that if someone explicitly requests output of filter, we know to get it from
      //  the input
      // NOTE no one will delete this, so you should fix deletion later
      // the idea here is that if someone tells us that our output must be refreshed,
      //  then we want the function refreshinput to be called, which will set the updating chain in motion
      refreshinputp=new CfpVFAMgetval<CfpstdSplice,CfpVal>("_splicerefresh",this,this,&CfpstdSplice::refreshinput);
      outputp->addspeaker(new CfpVAccstub(refreshinputp,outputp,-1,-1));
      triggerp=inputp;
      triggerindex=-1;
      // and then from filter input to filter processing
    }

  // we want the process() function to be called when the trigger goes off
  // so first we create an accessor that will trigger the process function
  //  processnotify=new CfpVFAMnotify<CfpstdSplice>("_spliceproccess",this,this,&CfpstdSplice::process);
  processp=new CfpVFAMcfpsetval<CfpstdSplice>("_spliceproccess",this,this,&CfpstdSplice::process);

  // and now we link it to the trigger signal
  triggerp->addlistener(processp,triggerindex,-1);

  // now modify the wire so that it reflects the new scenario about where subsequent splices should be spliced
  wirep->setlaststubp(newstubp);

  return retval;
}






//--------------------------------------------------------------------------------
//---- CfpstdSplice::addobjmembers() --------------------------------------------
//--------------------------------------------------------------------------------

void CfpstdSplice::addobjmembers()
{
  // add the member variables to the OBJECT
}


//--------------------------------------------------------------------------------
//---- CfpstdSplice::destruct() -------------------------------------------------
//--------------------------------------------------------------------------------

void CfpstdSplice::destruct()
{
  // de-instantiate the wrapped object.
  // we wrap ourselves so there is no need to delete any 'wrapped class', just set the instantiated flag to false
  instantiated=false;
}



void CfpstdSplice::process(CfpVal *valp)
{
  // the splice is 'triggered' by some trigger signal
  // (which may or may not be the same as the input variable).
  //  (ie a trigger could be a clock signal - which is passed via *valp)

  // this is the supervisory function call that should initiate a chain of others.
  // we break them up so that
  //  1) derived classes only have to reimplement the functions they want to
  //  2) so that we can selectively do computation less frequently than every time we are trigger if we want to.
  //     ie a splice that only executes every 10 seconds shouldn't bother doing all this computation on every input.

  // set the trigger value pointer for the other functions so we don't have to pass it to very one
  triggervalp=valp;

  //  cout << "... In a splice process.\n";

  if (iscomputetime())
    {
      if (manualretrieve)
	{
	  // normally, we would always have the latest version of the input, since we still get notified
	  //  every time the input changes.
	  // BUT it is possible that our input is from a source that doesn't inform us about changes, in which
	  //  case we want to explicitly retrieve the latest input source value.
	  //	  cout << "the splice is manually requesting the input source value.\n";
	  laststubp->retrieveandset();
	}

      // first preprocess the input (apply any noise, etc.)
      preprocessinput();
      // now perform the computation
      compute();
      // now postprocess the output (apply any noise, etc.)
      postprocessoutput();
    }
      
  if (isoutputtime())
    {
      // now send out the output
      //      cout << "... It's output time.  seding changes.\n";
      outputp->changes();
    }
}











CfpVal CfpstdSplice::refreshinput()
{
  // This can get called when the splice is a manually activated one, and we get a bottom-up
  //  request for the latest version.

  // manually tell our inputs to get new their latest values
  inputp->latestupdate();

  // now for compliance we return an undefined CfpVal
  CfpVal temp;
  return temp;
}



















//--------------------------------------------------------------------------------
//---- CfpstdDatastream -------------------------------------------------------------
//--------------------------------------------------------------------------------

CfpstdDatastream::CfpstdDatastream(Configpp *cfpfp,CfpClass *inclassp,string oname)
  :CfpObject(cfpfp,inclassp,oname)
{
  // this is the constructor that gets called every time the user creates a new
  // instantiated object of the class.
  // This function and makenew, the next, are partners.
  streamp=NULL;
}


//--------------------------------------------------------------------------------
//---- CfpstdDatastream::makenew() --------------------------------------------------
//--------------------------------------------------------------------------------

CfpObject *CfpstdDatastream::makenew(CfpClass *inclassp,string objname)
{
  // create a new instance of ourselves, the wrapper.
  // Important: instantiating the wrapped class will come later (in function 'construct').
  return (new CfpstdDatastream(configppp,inclassp,objname));
}





//--------------------------------------------------------------------------------
//---- CfpstdDatastream -------------------------------------------------------------
//--------------------------------------------------------------------------------

void CfpstdDatastream::registerwrappers(CfpFactory *sfactp)
{
  // this is the constructor that gets called ONCE when the class is registered with the factory.
  CfpClass *newclassp;

  // Register Datastream.In class
  newclassp=sfactp->registerclass("CfpStd","Datastream.Infile",this);
  newclassp->addarg("description"," ",true,"optional description of data stream");
  newclassp->addarg("filename","",true,"name of the file to read from");

  // Register Datastream.Out class
  newclassp=sfactp->registerclass("CfpStd","Datastream.Outfile",this);
  newclassp->addarg("description"," ",true,"optional description of data stream");
  newclassp->addarg("filename","",true,"name of the file to write from");
  newclassp->addarg("append","",false,"");

  // Register Datastream.Inline class
  newclassp=sfactp->registerclass("Cfpstd","Datastream.Inline",this);
  newclassp->addarg("description"," ",true,"optional description of inline data");
}



//--------------------------------------------------------------------------------
//---- CfpstdDatastream::construct() ------------------------------------------------
//--------------------------------------------------------------------------------

void CfpstdDatastream::construct()
{
  // instantiate the wrapped object.  this is where we actually do late (lazy) instantiation.
  // if we are a wrapper for another class, we would allocate that target wrapped class here.
  // and save it to a local variable.  otherwise this is its own wrapper.

  // in this case, the object wraps itself, so we don't have to create the wrapped object

  if (subclass()=="Infile")
    {
      // input data file
      localp=streamp=new DatastreamIn(configppp);
      streamp->setname((string)argval("filename"));
    }
  else if (subclass()=="Inline")
    {
      // input data file
      localp=streamp=new DatastreamInline(configppp);
    }
  else if (subclass()=="Outfile")
    {
      // output data file
      localp=streamp=new DatastreamOut(configppp);
      streamp->setname((string)argval("filename"));
      if (argeq("append","true"))
	((DatastreamOut*)streamp)->setfilemode(append_writemode);
      else
	((DatastreamOut*)streamp)->setfilemode(overwrite_writemode);
    }

  streamp->setdesc((string)argval("description"));
  // set the instantiated flag
  instantiated=true;
}




//--------------------------------------------------------------------------------
//---- CfpstdDatastream::destruct() -------------------------------------------------
//--------------------------------------------------------------------------------

void CfpstdDatastream::destruct()
{
  // de-instantiate the wrapped object.
  // we wrap ourselves so there is no need to delete any 'wrapped class', just set the instantiated flag to false
  instantiated=false;
}



//--------------------------------------------------------------------------------
//---- CfpstdDatastream::done() --------------------------------------------------
//--------------------------------------------------------------------------------

CfpVal CfpstdDatastream::done()
{
  if (classname()=="Datastream.Inline")
    {
      // inline Datastream requires some extra work
      // we need to get a sister Inline object which holds the file data
      CfpstdInline *sip;
      sip=(CfpstdInline *)configppp->findchild(this,"Block",true);
      if (!sip)
	throw CfpError("Datastream.Inline requires a Block.Data {} object which specifies the raw data.","",io,false,configppp);
      streamp->setname(sip->getfilename());
      streamp->setbounds(sip->getstart(),sip->getend());
    }
  return retval;
}









//}
// Namespace close-brace
