// cfptimer.C

#include "cfpinc.h"

//Not implemented in gcc yet (2.8.1)
//namespace Configpp{











//--------------------------------------------------------------------------------
//-- CfpTimer
//--------------------------------------------------------------------------------

CfpTimer::CfpTimer(string intimername, Configpp *inconfigppp,int innice,int indeathcount)
{
  // construct a timer
  name=intimername;
  activations=0;
  sim_time=0;
  cycle_time=0;
  cycle_deltat_activate=1;
  timertype=cycleclockType;
  configppp=inconfigppp;
  managerp=configppp->gettimermanagerp();
  deathcount=indeathcount;
  niceness=innice;

  if (niceness<1)
    niceness=1;

  // allocate the accessor
  acctimer=new CfpValAccessor("timer accessor",NULL,&acval);
}


CfpTimer::~CfpTimer()
{
  // destruct the timer
}


CfpVAccstub *CfpTimer::addlistener(CfpVAccessor *listenerp,int toindex=-1)
{
  // just a shortcut to acctimer->addlistener()
  return acctimer->addlistener(listenerp,-1,toindex);
}




void CfpTimer::simclock(double insimstep,double insimact)
{
  // set that this is a simclockType
  timertype=simclockType;
  sim_timestep=insimstep;
  sim_deltat_activate=insimact;
}

void CfpTimer::wallclock(double intimeact)
{
  // set that this is a wallclockType
  timertype=wallclockType;
  wall_deltat_activate=intimeact;
}

void CfpTimer::cycleclock(long intimeact)
{
  // set that this is a cycleclockType
  timertype=cycleclockType;
  cycle_deltat_activate=intimeact;
}




int CfpTimer::cycle()
{
  // the manager has given us a slice of time, so let's see if we want to activate
  bool dotrigger=false;
  
  ++activations;
  // update our deltat's (calculate simulation time based on cycles*sim_timestep)
  wall_elapsedt=managerp->wall_time-wall_time;
  cycle_elapsedt=managerp->cycle_time-cycle_time;
  sim_elapsedt=cycle_elapsedt*sim_timestep;

  switch(timertype)
    {
    case wallclockType:
      wall_delay+=wall_elapsedt-wall_deltat_activate;
      if (wall_elapsedt>wall_deltat_activate)
	{
	  dotrigger=true;
	  acval.set_number(wall_elapsedt);
	}
      break;
    case cycleclockType:
      cycle_delay+=cycle_elapsedt-cycle_deltat_activate;
      if (cycle_elapsedt>cycle_deltat_activate)
	{
	  dotrigger=true;
	  acval.set_number(cycle_elapsedt);
	}
      break;
    case simclockType:
      sim_delay+=sim_elapsedt-sim_deltat_activate;
      if (sim_elapsedt>sim_deltat_activate)
	{
	  dotrigger=true;
	  acval.set_number(sim_elapsedt);
	}
      break;
    }

  if (dotrigger)
    {
      // enough time has passed, so update internal clocks and trigger it.
      wall_time+=wall_elapsedt;
      cycle_time+=cycle_elapsedt;
      sim_time+=sim_elapsedt;
      return trigger();
    }

  // return 0 on suicide request
  return 1;
}



int CfpTimer::trigger()
{
  // we have decided to trigger our timer!
  //  cout << "Timer " << name << " is activating.\n";

  // now tell all listeners that the timer has fired (accessor has its value=elapsed 'time')
  acctimer->changes();

  // return 0 on suicide request
  return 1;
}








//--------------------------------------------------------------------------------
//---- CfpTimerManager --------------------------------------------------------
//--------------------------------------------------------------------------------

CfpTimerManager::CfpTimerManager(Configpp *inconfigppp)
{
  // initialize the timer manager
  timercount=0;
  tbegin=timers.begin();
  tend=timers.end();
  timers_modified=false;
  configppp=inconfigppp;
  wantstodie=false;

  // initialize times
  cycle_time=0;
  wall_time=0; // fix this
  min_nice=0;
  max_nice=0;
}


CfpTimerManager::~CfpTimerManager()
{
  // delete the timer manager
  // we should delete each timer?
  timercount=0;
}



void CfpTimerManager::addtimer(CfpTimer *timerp)
{
  // add a timer to the manager's timer list
  configppp->debug("adding timer '"+timerp->getname()+"'.");

  timers.push_back(timerp);
  ++timercount;

  tbegin=timers.begin();
  tend=timers.end();
  timers_modified=true;

  // adjust nice range
  if (timerp->niceness<min_nice)
    min_nice=timerp->niceness;
  if (timerp->niceness>max_nice)
    max_nice=timerp->niceness;
}


void CfpTimerManager::takeover()
{
  // control is passed to the timer manager which just cycles through all of its timers
  //  until there are no more to execute.  Note, anyone can cfpely cause the exit of
  //  the program by calling the cfptimermanager kill() function.

  configppp->debug("Timer manager is taking over.");

  while (!wantstodie && timercount)
    cycle();
}



void CfpTimerManager::cycle()
{
  // one cycle of the program has passed, now invoke timers as appropriate
  // this is the MAIN function of the interpreter that gets called repeatedly.
  // Note: because his is the main loop we are forever in, we should replace stl
  //  lists with static pointer array.
  int retv;
  int count;
  CfpTimer *timerp;

  // do nicecycles
  for (count=min_nice;count<=max_nice;++count)
    {
      //update the clock
      update_clock();
      
      // invoke timers
      ti=tbegin;
      ti=timers.begin();
      while (ti!=tend)
	{
	  timerp=(*ti);
	  if (count%timerp->niceness==0)
	    {
	      retv=(*ti)->cycle();
	      if (!retv || wantstodie)
		{
		  // this timer is commiting suicide, so remove it
		  // for now we exit
		  wantstodie=true;
		  return;
		}
	      else if (timers_modified)
		{
		  timers_modified=false;
		  return;
		}
	    }
	  ++ti;
	}
    }
}


void CfpTimerManager::update_clock()
{
  // get new new clock values
  ++cycle_time;
  wall_time=0;
}


void CfpTimerManager::update_niceties()
{
  // ideally, we want to decrease or increase the invocation of timers based on their delays and
  // and their nice values.
}






























//}
// Namespace close-brace
