// cfpval.h
// This file describes the CfpVal class, which is used to pass values back and forth between objects,
//  function calls, etc.

#ifndef INC_cfpvalh
#define INC_cfpvalh

#include "cfpinc.h"

//Not implemented in gcc yet (2.8.1)
//using namespace std;
//namespace Configpp{








//--------------------------------------------------------------------------------
//---- CfpVal --------------------------------------------------------------------
//--------------------------------------------------------------------------------
//: Holds rvalues (values of expressions on the right hand side of assignments, etc.)

class CfpVal
{
private:
  CfpValType valtype;
  CfpUnit sunit;
  CfpLoc sloc;									// optional location of error;
  int veclength;
  int accessindex;
private:
  string stringval;
  long double numberval;
  bool boolval;
  void *voidpointer;
  CfpClass *classpointer;
  CfpObject *objectpointer;
  CfpVAccessor *vaccesspointer;
public:
  void maketype(CfpValType expectedtype);	// convert to type or throw error if you can't
  CfpValType getvaltype() {return valtype;}								// accessor
  void settype(CfpValType invaltype) {valtype=invaltype;}						// accessor
  void setloc(CfpLoc inloc) {sloc=inloc;};								// accessor
  bool isdefined();											// accessor
  void alocatespace();
  void deallocatespace();
public:
  // constructors
  CfpVal();
  CfpVal(string instring,bool condef=false);			// create (condef says whether blank strings count)
  CfpVal(int inint) {set_number(inint);};
  CfpVal(long inlong) {set_number(inlong);};
  CfpVal(long double inldouble) {set_number(inldouble);};
  CfpVal(double indouble){set_number(indouble);};
//  CfpVal(CfpVal insv) {*this=insv;};
public:
  long double get_number() {maketype(numberVType);return numberval;};					// accessor
  float get_float() {maketype(numberVType);return (float)numberval;};					// accessor
  string get_string() {maketype(stringVType);return stringval;};					// accessor
  void *get_voidpointer() {maketype(voidpointerVType);return voidpointer;};				// accessor
  CfpClass *get_classpointer() {maketype(classpointerVType);return classpointer;};			// accessor
  CfpObject *get_objectpointer() {maketype(objectpointerVType);return objectpointer;};			// accessor
  CfpVAccessor *get_vaccesspointer() {maketype(vaccesspointerVType);return vaccesspointer;};		// accessor
  int get_vaccessindex() {return accessindex;};
  const char *get_charp() {maketype(stringVType);return stringval.c_str();};				// accessor
  void set_number(long double val) {settype(numberVType);numberval=val;};				// accessor
  void set_number(string stringnum);									// accessor
  void set_string(string val) {settype(stringVType);stringval=val;};					// accessor
  void set_voidpointer(void *val) {settype(voidpointerVType);voidpointer=val;};				// accessor
  void set_classpointer(CfpClass *val) {settype(classpointerVType);classpointer=val;};			// accessor
  void set_objectpointer(CfpObject *val) {settype(objectpointerVType);objectpointer=val;};		// accessor
  void set_vaccesspointer(CfpVAccessor *val){settype(vaccesspointerVType);vaccesspointer=val;};		// accessor
  void set_vaccessindex(int inindex){accessindex=inindex;};                             		// accessor
  void convertin(char *val);										// acccssor
  void stringset(string instring);									// accessor
  void print();
public:
  string typestr() {return typestr(valtype,&sunit);};		// return string describing the current value type
  string typestr(CfpValType vtype,CfpUnit *unitp=NULL);	// return string describing value passed
public:
  operator const bool() {maketype(boolVType);return boolval;};			// automatic casting
  operator const int() {maketype(numberVType);return (int)numberval;};		// automatic casting
  operator const long() {maketype(numberVType);return (long)numberval;};	// automatic casting
  operator const double() {maketype(numberVType);return (double)numberval;};	// automatic casting
  operator const float() {maketype(numberVType);return (float)numberval;};	// automatic casting
  operator const char*() {maketype(stringVType);return stringval.c_str();};	// automatic casting
  bool operator!=(CfpVal v2);
};












//}
// Namespace close-brace
#endif
