//  datafile.h
//  This file contains the class definition for the classes which read and write statistics/data
//  periodically to and from files
//

#ifndef datastreamh_defined
#define datastreamh_defined

#include "cfpinc.h"
#include "JRutils.h"


#include <time.h>
#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <string>
#include <vector>
#include <iomanip>
#include <iostream>
#include <fstream>



// ---------------------------------------------------------------------------------------
// -- Typedefs
// ---------------------------------------------------------------------------------------


//: WriteModeType for methods of writing data files
typedef enum FileWritemodeType
{
  append_writemode,
  overwrite_writemode
};


//: StreamtokenType for reading tokens from streams
typedef enum StreamtokenType
{
  StreamTokeType_CR,
  StreamTokeType_EOF,
  StreamTokeType_ERROR,
  StreamTokeType_number,
  StreamTokeType_alpha
};



//: Maximum length of an input file line of data
#define MAX_DF_STRLEN 600







// ---------------------------------------------------------------------------------------
// -- Datastream
// ---------------------------------------------------------------------------------------
//:The base class for Datasource.Inline, Datasource.In, Datasource.Out
// these are helper classes for Datastream below

class Datastream
{
protected:
  string name;
  fstream *filep;
  Configpp *configppp;
  string user_description;
  bool done;
protected:
  long realstart,realend;
  long curline,numlines;
public:
  Datastream(Configpp *cfpp);
  ~Datastream();
public:
  virtual void start() {openfile();};
  virtual void finish();
public:
  virtual void openfile();
  void closefile();
public:
  virtual void error(string errstring,string hinstr="");
public:
  void setbounds(long spos,long epos) {realstart=spos;realend=epos;};
  void setstartpos(long pos){realstart=pos;};
  void setendpos(long pos){realend=pos;};
  void setname(string inname) {name=inname;};
  string getname(){return name;};
  string getdesc(){return user_description;};
  void setdesc(string indesc){user_description=indesc;};
  long getendpos() {return realend;};
  fstream *getfilep() {return filep;};
public:
  virtual void reset();
  virtual void write(string str) {error("Attempt to write from a Datastream.In");};
  virtual void write(char *str)  {error("Attempt to write from a Datastream.In");};
  virtual string readline();
  virtual string readword();
  virtual StreamtokenType readtoken(string &word,bool skipcomments=true);
  virtual bool readline(char *str,int maxlen);
  virtual bool ateof();
public:
  virtual bool readfloatarray(float **arraypp,long &rows, long &cols,bool weallocate=true,bool fullrows=true,bool formatted=true,bool skipcomments=true,bool complain=true);
};


class DatastreamOut : public Datastream
{
  FileWritemodeType fmode;
public:
  DatastreamOut(Configpp *cfpp) : Datastream(cfpp){fmode=overwrite_writemode;};
public:
  virtual void openfile();
  virtual void error(string errstring,string hinstr="");
public:
  virtual void write(string str);
  virtual void write(char *str);
  virtual bool ateof() {return false;};
  virtual void reset() {error("Attempt to reset from a Datastream.Out");};
  virtual string readline() {error("Attempt to read from a Datastream.Out"); return "";};
  virtual string readword() {error("Attempt to read from a Datastream.Out"); return "";};
  virtual StreamtokenType readtoken(string &word,bool skipcomments=true) {error("Attempt to read a token from a Datastream.Out"); return StreamTokeType_ERROR;};
  virtual bool readline(char *str,int maxlen) {error("Attempt to read from a Datastream.Out"); return false;};
  virtual bool readfloatarray(float **arraypp,long &rows,long &cols,bool weallocate=true,bool fullrows=true,bool formatted=true,bool complain=true) {error("Attempt to read an array from a Datastream.Out"); return false;};
  void setfilemode(FileWritemodeType infmode) {fmode=infmode;};
};

class DatastreamInline : public Datastream
{
public:
  DatastreamInline(Configpp *cfpp) : Datastream(cfpp){;};
public:
  virtual void openfile();
};


class DatastreamIn : public Datastream
{
public:
  DatastreamIn(Configpp *cfpp);
};







#endif
