/***********************************************************************

 ProjectionMatrix.h
 Projection Matrix - a Camera type viewing matrix
 Copyright (C) 1993
        Eric C. Welton 1993/94
        Beckman Institute - Urbana, Illinois
        For Neuronal Pattern Analysis Group and Hexapod Project

        These libraries are free and may be redistributed and/or modified,
        provided the above notice and this paragraph are included.
        These libraries may not be sold and are distributed without
        any warranty or implication of fitness for a purpose.


***********************************************************************/

#ifndef PROJECTION_MATRIX_HDR
#define PROJECTION_MATRIX_HDR

#include <math.h>
#include <DMatrix.h>
#include <FMatrix.h>
#include <CoordVectors.h>

class CameraDHP32:public DMatrix {
 DVectorHR3  eye;
 DVectorHR3  ref;
 DVectorHR3  up;
 void       Set();
public:
 const DVectorHR3& Eye;
 const DVectorHR3& Ref;
 const DVectorHR3& Up;

 CameraDHP32(const DVectorR3& _eye,const DVectorR3& _ref,const DVectorR3& _up):
   DMatrix(4,4),Eye(eye),Ref(ref),Up(up)
   { eye=_eye; ref=_ref; up=_up; Set(); }
 CameraDHP32(const DVectorHR3& _eye,const DVectorHR3& _ref,const DVectorHR3& _up):
   DMatrix(4,4),Eye(eye),Ref(ref),Up(up)
   { eye=_eye; ref=_ref; up=_up; Set(); }

 void ChgEye(const DVectorR3& _eye) { eye=_eye; Set(); }
 void ChgEye(const DVectorHR3& _eye) { eye=_eye; Set(); }

 void ChgRef(const DVectorR3& _ref) { ref=_ref; Set(); }
 void ChgRef(const DVectorHR3& _ref) { ref=_ref; Set(); }

 void ChgUp( const DVectorR3& _up)  { up =_up;  Set(); }
 void ChgUp( const DVectorHR3& _up)  { up =_up;  Set(); }

 inline DMatrix& operator =(const DMatrix& m) { return DMatrix::operator=(m); }
};

class CameraFHP32:public FMatrix {
 FVectorHR3  eye;
 FVectorHR3  ref;
 FVectorHR3  up;
 void       Set();
public:
 const FVectorHR3& Eye;
 const FVectorHR3& Ref;
 const FVectorHR3& Up;

 CameraFHP32(const FVectorR3& _eye,const FVectorR3& _ref,const FVectorR3& _up):
   FMatrix(4,4),Eye(eye),Ref(ref),Up(up)
   { eye=_eye; ref=_ref; up=_up; Set(); }
 CameraFHP32(const FVectorHR3& _eye,const FVectorHR3& _ref,const FVectorHR3& _up):
   FMatrix(4,4),Eye(eye),Ref(ref),Up(up)
   { eye=_eye; ref=_ref; up=_up; Set(); }

 void ChgEye(const FVectorR3& _eye) { eye=_eye; Set(); }
 void ChgEye(const FVectorHR3& _eye) { eye=_eye; Set(); }

 void ChgRef(const FVectorR3& _ref) { ref=_ref; Set(); }
 void ChgRef(const FVectorHR3& _ref) { ref=_ref; Set(); }

 void ChgUp( const FVectorR3& _up)  { up =_up;  Set(); }
 void ChgUp( const FVectorHR3& _up)  { up =_up;  Set(); }

 inline FMatrix& operator =(const FMatrix& m) { return FMatrix::operator=(m); }
};


//======================================================================
//
// Default type establishment
//
#ifdef __Concat3
#undef __Concat3
#endif
#ifdef _Concat3
#undef _Concat3
#endif
#define __Concat3(a,b,c) a##b##c
#define _Concat3(a,b,c) __Concat3(a,b,c)

typedef _Concat3(Camera,DEF_PREC_MATRIX_PREFIX,HP32) CameraHP32;

#undef __Concat3
#undef _Concat3

#endif

