/***********************************************************************

 TransformationMatrix.c
 Transformation Matrix classes including Scaling, Rotation, and Translation
 Copyright (C) 1993
        Eric C. Welton 1993/94
        Beckman Institute - Urbana, Illinois
        For Neuronal Pattern Analysis Group and Hexapod Project

        These libraries are free and may be redistributed and/or modified,
        provided the above notice and this paragraph are included.
        These libraries may not be sold and are distributed without
        any warranty or implication of fitness for a purpose.

 General Information

        The DFMatrix library provides fundamental matrix operations.

***********************************************************************/

#include "TransformationMatrix.h"

static TFMatrixR2    FmatR2;
static TFMatrixR3    FmatR3;
static TFMatrixHR2   FmatHR2;
static TFMatrixHR3   FmatHR3;

TFMatrixR2 ScalingFMatrixR2(float x,float y)
   {  FmatR2.clear(); FmatR2(0,0)=x; FmatR2(1,1)=y; return TFMatrixR2(FmatR2); }
TFMatrixR2 ScalingFMatrixR21(float x)
   {  FmatR2.clear(); FmatR2(0,0)=x; FmatR2(1,1)=1.0; return TFMatrixR2(FmatR2); }
TFMatrixR2 ScalingFMatrixR22(float y)
   {  FmatR2.clear(); FmatR2(0,0)=1.0; FmatR2(1,1)=y; return TFMatrixR2(FmatR2); }

TFMatrixR3 ScalingFMatrixR3(float x,float y,float z)
   {  FmatR3.clear(); FmatR3(0,0)=x; FmatR3(1,1)=y; FmatR3(2,2)=z;
      return TFMatrixR3(FmatR3); }
TFMatrixR3 ScalingFMatrixR31(float x)
   {  FmatR3.clear(); FmatR3(0,0)=x; FmatR3(1,1)=1.0; FmatR3(2,2)=1.0;
      return TFMatrixR3(FmatR3); }
TFMatrixR3 ScalingFMatrixR32(float y)
   {  FmatR3.clear(); FmatR3(0,0)=1.0; FmatR3(1,1)=y; FmatR3(2,2)=1.0;
      return TFMatrixR3(FmatR3); }
TFMatrixR3 ScalingFMatrixR33(float z)
   {  FmatR3.clear(); FmatR3(0,0)=1.0; FmatR3(1,1)=1.0; FmatR3(2,2)=z;
      return TFMatrixR3(FmatR3); }

TFMatrixHR2 ScalingFMatrixHR2(float x,float y)
   {  FmatHR2.clear(); FmatHR2(0,0)=x; FmatHR2(1,1)=y; return TFMatrixHR2(FmatHR2); }
TFMatrixHR2 ScalingFMatrixHR21(float x)
   {  FmatHR2.clear(); FmatHR2(0,0)=x; FmatHR2(1,1)=1.0; return TFMatrixHR2(FmatHR2); }
TFMatrixHR2 ScalingFMatrixHR22(float y)
   {  FmatHR2.clear(); FmatHR2(0,0)=1.0; FmatHR2(1,1)=y; return TFMatrixHR2(FmatHR2); }

TFMatrixHR3 ScalingFMatrixHR3(float x,float y,float z)
   {  FmatHR3.clear(); FmatHR3(0,0)=x; FmatHR3(1,1)=y; FmatHR3(2,2)=z;
      return TFMatrixHR3(FmatHR3); }
TFMatrixHR3 ScalingFMatrixHR31(float x)
   {  FmatHR3.clear(); FmatHR3(0,0)=x; FmatHR3(1,1)=1.0; FmatHR3(2,2)=1.0;
      return TFMatrixHR3(FmatHR3); }
TFMatrixHR3 ScalingFMatrixHR32(float y)
   {  FmatHR3.clear(); FmatHR3(0,0)=1.0; FmatHR3(1,1)=y; FmatHR3(2,2)=1.0;
      return TFMatrixHR3(FmatHR3); }
TFMatrixHR3 ScalingFMatrixHR33(float z)
   {  FmatHR3.clear(); FmatHR3(0,0)=1.0; FmatHR3(1,1)=1.0; FmatHR3(2,2)=z;
      return TFMatrixHR3(FmatHR3); }

TFMatrixHR2 TranslationFMatrixHR2(float x,float y)
   {  FmatHR2.ident(); FmatHR2(0,2)=x; FmatHR2(1,2)=y; return TFMatrixHR2(FmatHR2); }
TFMatrixHR2 TranslationFMatrixHR21(float x)
   {  FmatHR2.ident(); FmatHR2(0,2)=x; return TFMatrixHR2(FmatHR2); }
TFMatrixHR2 TranslationFMatrixHR22(float y)
   {  FmatHR2.ident(); FmatHR2(1,2)=y; return TFMatrixHR2(FmatHR2); }

TFMatrixHR3 TranslationFMatrixHR3(float x,float y,float z)
   {  FmatHR3.ident(); FmatHR3(0,3)=x; FmatHR3(1,3)=y; FmatHR3(2,3)=z;
      return TFMatrixHR3(FmatHR3); }
TFMatrixHR3 TranslationFMatrixHR31(float x)
   {  FmatHR3.ident(); FmatHR3(0,3)=x;
      return TFMatrixHR3(FmatHR3); }
TFMatrixHR3 TranslationFMatrixHR32(float y)
   {  FmatHR3.ident(); FmatHR3(0,3)=y;
      return TFMatrixHR3(FmatHR3); }
TFMatrixHR3 TranslationFMatrixHR33(float z)
   {  FmatHR3.ident(); FmatHR3(0,3)=z;
      return TFMatrixHR3(FmatHR3); }

TFMatrixR2 RotationFMatrixR2(float theta)
   {  FmatR2.clear();
      FmatR2(0,0)=     (FmatR2(1,1)=(float)(cos(theta)));
      FmatR2(0,1)=-1.0*(FmatR2(1,0)=(float)(sin(theta)));
      return TFMatrixR2(FmatR2); }
TFMatrixR3 RotationFMatrixR31(float theta)
   {  FmatR3.clear();
      FmatR3(1,1)=     (FmatR3(2,2)=(float)(cos(theta)));
      FmatR3(1,2)=-1.0*(FmatR3(2,1)=(float)(sin(theta)));
      FmatR3(0,0)=1.0; return TFMatrixR3(FmatR3); }
TFMatrixR3 RotationFMatrixR32(float theta)
   {  FmatR3.clear();
      FmatR3(0,0)=     (FmatR3(2,2)=(float)(cos(theta)));
      FmatR3(0,2)=-1.0*(FmatR3(2,0)=(float)(sin(theta)));
      FmatR3(1,1)=1.0; return TFMatrixR3(FmatR3); }
TFMatrixR3 RotationFMatrixR33(float theta)
   {  FmatR3.clear();
      FmatR3(0,0)=     (FmatR3(1,1)=(float)(cos(theta)));
      FmatR3(0,1)=-1.0*(FmatR3(1,0)=(float)(sin(theta)));
      FmatR3(2,2)=1.0; return TFMatrixR3(FmatR3); }

TFMatrixHR2 RotationFMatrixHR2(float theta)
   {  FmatHR2.clear();
      FmatHR2(0,0)=     (FmatHR2(1,1)=(float)(cos(theta)));
      FmatHR2(0,1)=-1.0*(FmatHR2(1,0)=(float)(sin(theta)));
      return TFMatrixHR2(FmatHR2); }
TFMatrixHR3 RotationFMatrixHR31(float theta)
   {  FmatHR3.clear();
      FmatHR3(1,1)=     (FmatHR3(2,2)=(float)(cos(theta)));
      FmatHR3(1,2)=-1.0*(FmatHR3(2,1)=(float)(sin(theta)));
      FmatHR3(0,0)=1.0; return TFMatrixHR3(FmatHR3); }
TFMatrixHR3 RotationFMatrixHR32(float theta)
   {  FmatHR3.clear();
      FmatHR3(0,0)=     (FmatHR3(2,2)=(float)(cos(theta)));
      FmatHR3(0,2)=-1.0*(FmatHR3(2,0)=(float)(sin(theta)));
      FmatHR3(1,1)=1.0; return TFMatrixHR3(FmatHR3); }
TFMatrixHR3 RotationFMatrixHR33(float theta)
   {  FmatHR3.clear();
      FmatHR3(0,0)=     (FmatHR3(1,1)=(float)(cos(theta)));
      FmatHR3(0,1)=-1.0*(FmatHR3(1,0)=(float)(sin(theta)));
      FmatHR3(2,2)=1.0; return TFMatrixHR3(FmatHR3); }





static TDMatrixR2    DmatR2;
static TDMatrixR3    DmatR3;
static TDMatrixHR2   DmatHR2;
static TDMatrixHR3   DmatHR3;

TDMatrixR2 ScalingDMatrixR2(double x,double y)
   {  DmatR2.clear(); DmatR2(0,0)=x; DmatR2(1,1)=y; return TDMatrixR2(DmatR2); }
TDMatrixR2 ScalingDMatrixR21(double x)
   {  DmatR2.clear(); DmatR2(0,0)=x; DmatR2(1,1)=1.0; return TDMatrixR2(DmatR2); }
TDMatrixR2 ScalingDMatrixR22(double y)
   {  DmatR2.clear(); DmatR2(0,0)=1.0; DmatR2(1,1)=y; return TDMatrixR2(DmatR2); }

TDMatrixR3 ScalingDMatrixR3(double x,double y,double z)
   {  DmatR3.clear(); DmatR3(0,0)=x; DmatR3(1,1)=y; DmatR3(2,2)=z;
      return TDMatrixR3(DmatR3); }
TDMatrixR3 ScalingDMatrixR31(double x)
   {  DmatR3.clear(); DmatR3(0,0)=x; DmatR3(1,1)=1.0; DmatR3(2,2)=1.0;
      return TDMatrixR3(DmatR3); }
TDMatrixR3 ScalingDMatrixR32(double y)
   {  DmatR3.clear(); DmatR3(0,0)=1.0; DmatR3(1,1)=y; DmatR3(2,2)=1.0;
      return TDMatrixR3(DmatR3); }
TDMatrixR3 ScalingDMatrixR33(double z)
   {  DmatR3.clear(); DmatR3(0,0)=1.0; DmatR3(1,1)=1.0; DmatR3(2,2)=z;
      return TDMatrixR3(DmatR3); }

TDMatrixHR2 ScalingDMatrixHR2(double x,double y)
   {  DmatHR2.clear(); DmatHR2(0,0)=x; DmatHR2(1,1)=y; return TDMatrixHR2(DmatHR2); }
TDMatrixHR2 ScalingDMatrixHR21(double x)
   {  DmatHR2.clear(); DmatHR2(0,0)=x; DmatHR2(1,1)=1.0; return TDMatrixHR2(DmatHR2); }
TDMatrixHR2 ScalingDMatrixHR22(double y)
   {  DmatHR2.clear(); DmatHR2(0,0)=1.0; DmatHR2(1,1)=y; return TDMatrixHR2(DmatHR2); }

TDMatrixHR3 ScalingDMatrixHR3(double x,double y,double z)
   {  DmatHR3.clear(); DmatHR3(0,0)=x; DmatHR3(1,1)=y; DmatHR3(2,2)=z;
      return TDMatrixHR3(DmatHR3); }
TDMatrixHR3 ScalingDMatrixHR31(double x)
   {  DmatHR3.clear(); DmatHR3(0,0)=x; DmatHR3(1,1)=1.0; DmatHR3(2,2)=1.0;
      return TDMatrixHR3(DmatHR3); }
TDMatrixHR3 ScalingDMatrixHR32(double y)
   {  DmatHR3.clear(); DmatHR3(0,0)=1.0; DmatHR3(1,1)=y; DmatHR3(2,2)=1.0;
      return TDMatrixHR3(DmatHR3); }
TDMatrixHR3 ScalingDMatrixHR33(double z)
   {  DmatHR3.clear(); DmatHR3(0,0)=1.0; DmatHR3(1,1)=1.0; DmatHR3(2,2)=z;
      return TDMatrixHR3(DmatHR3); }

TDMatrixHR2 TranslationDMatrixHR2(double x,double y)
   {  DmatHR2.ident(); DmatHR2(0,2)=x; DmatHR2(1,2)=y; return TDMatrixHR2(DmatHR2); }
TDMatrixHR2 TranslationDMatrixHR21(double x)
   {  DmatHR2.ident(); DmatHR2(0,2)=x; return TDMatrixHR2(DmatHR2); }
TDMatrixHR2 TranslationDMatrixHR22(double y)
   {  DmatHR2.ident(); DmatHR2(1,2)=y; return TDMatrixHR2(DmatHR2); }

TDMatrixHR3 TranslationDMatrixHR3(double x,double y,double z)
   {  DmatHR3.ident(); DmatHR3(0,3)=x; DmatHR3(1,3)=y; DmatHR3(2,3)=z;
      return TDMatrixHR3(DmatHR3); }
TDMatrixHR3 TranslationDMatrixHR31(double x)
   {  DmatHR3.ident(); DmatHR3(0,3)=x;
      return TDMatrixHR3(DmatHR3); }
TDMatrixHR3 TranslationDMatrixHR32(double y)
   {  DmatHR3.ident(); DmatHR3(1,3)=y;
      return TDMatrixHR3(DmatHR3); }
TDMatrixHR3 TranslationDMatrixHR33(double z)
   {  DmatHR3.ident(); DmatHR3(1,3)=z;
      return TDMatrixHR3(DmatHR3); }

TDMatrixR2 RotationDMatrixR2(double theta)
   {  DmatR2.clear();
      DmatR2(0,0)=     (DmatR2(1,1)=(double)(cos(theta)));
      DmatR2(0,1)=-1.0*(DmatR2(1,0)=(double)(sin(theta)));
      return TDMatrixR2(DmatR2); }
TDMatrixR3 RotationDMatrixR31(double theta)
   {  DmatR3.clear();
      DmatR3(1,1)=     (DmatR3(2,2)=(double)(cos(theta)));
      DmatR3(1,2)=-1.0*(DmatR3(2,1)=(double)(sin(theta)));
      DmatR3(0,0)=1.0; return TDMatrixR3(DmatR3); }
TDMatrixR3 RotationDMatrixR32(double theta)
   {  DmatR3.clear();
      DmatR3(0,0)=     (DmatR3(2,2)=(double)(cos(theta)));
      DmatR3(0,2)=-1.0*(DmatR3(2,0)=(double)(sin(theta)));
      DmatR3(1,1)=1.0; return TDMatrixR3(DmatR3); }
TDMatrixR3 RotationDMatrixR33(double theta)
   {  DmatR3.clear();
      DmatR3(0,0)=     (DmatR3(1,1)=(double)(cos(theta)));
      DmatR3(0,1)=-1.0*(DmatR3(1,0)=(double)(sin(theta)));
      DmatR3(2,2)=1.0; return TDMatrixR3(DmatR3); }

TDMatrixHR2 RotationDMatrixHR2(double theta)
   {  DmatHR2.clear();
      DmatHR2(0,0)=     (DmatHR2(1,1)=(double)(cos(theta)));
      DmatHR2(0,1)=-1.0*(DmatHR2(1,0)=(double)(sin(theta)));
      return TDMatrixHR2(DmatHR2); }
TDMatrixHR3 RotationDMatrixHR31(double theta)
   {  DmatHR3.clear();
      DmatHR3(1,1)=     (DmatHR3(2,2)=(double)(cos(theta)));
      DmatHR3(1,2)=-1.0*(DmatHR3(2,1)=(double)(sin(theta)));
      DmatHR3(0,0)=1.0; return TDMatrixHR3(DmatHR3); }
TDMatrixHR3 RotationDMatrixHR32(double theta)
   {  DmatHR3.clear();
      DmatHR3(0,0)=     (DmatHR3(2,2)=(double)(cos(theta)));
      DmatHR3(0,2)=-1.0*(DmatHR3(2,0)=(double)(sin(theta)));
      DmatHR3(1,1)=1.0; return TDMatrixHR3(DmatHR3); }
TDMatrixHR3 RotationDMatrixHR33(double theta)
   {  DmatHR3.clear();
      DmatHR3(0,0)=     (DmatHR3(1,1)=(double)(cos(theta)));
      DmatHR3(0,1)=-1.0*(DmatHR3(1,0)=(double)(sin(theta)));
      DmatHR3(2,2)=1.0; return TDMatrixHR3(DmatHR3); }

