// grobaxis.C
// This files contains the methods for axis-related Grob drawing and creating

#include "grob.h"
#include "JRutils.h"

#include <cmath>

// test
float testtextscale=.0007;




//Not implemented in gcc yet (2.8.1)
//using namespace std;
//namespace Grob{








int GrobGlHelper::makeAxes(VisibleAxis *xaxisp,VisibleAxis *yaxisp,VisibleAxis *zaxisp,GrobDetail detail,float cxscale,float cyscale,float czscale,int linewidth,bool clearbox,GrobSpacerType usespacer)
{
  // build a GL list for x,y,z axes, and draw some nice label axes for the user to get their bearings
  bool bitfont=false;
  float sx,ix,ex,sy,iy,ey,sz,iz,ez;
  float bsx,bsy,bsz,bix,biy,biz,bex,bey,bez;
  float xthickness,ythickness,zthickness;
  float xspace,yspace,zspace;
  int axiscount=0;
  float widthx,widthy,widthz,heightx,heighty,heightz,depthx,depthy,depthz;
  float zspacer;
  float bsz2;
  int olinewidth;

  // fine detail stuff is not quite good, so we don't use if for now
  if (detail==finedetail)
    detail=highdetail;

  // also, force into linewidth(1)
  olinewidth=linewidth;
  linewidth=1;
  glLineWidth(1);

  // prepare gl context
  int displaylistnum=startlist();
  if (!xaxisp && !yaxisp && !zaxisp)
    {
      // there are no axes to draw
      endlist(); return displaylistnum;
    }

  // defaults
  sx=ix=ex=sy=iy=ey=sz=iz=ez=0.0;
  bsx=bsy=bsz=bix=biy=biz=bex=bey=bez=0.0;
  xspace=yspace=zspace=0.0;
  depthx=heightx=depthy=heighty=depthz=heightz=0.0;

  // get axis boundaries
  if (xaxisp)
    {
      xaxisp->getaxispoints(sx,ix,ex);
      bix=ix;
      if (sx>=ex)
	xaxisp=NULL;
      else
	{
	  if (usespacer==box_spacer || usespacer==xy_spacer)
	    (xspace=(xaxisp->symbolsize())/cxscale)*4;
	  else
	    xspace=((xaxisp->symbolsize())/cxscale)*2;

	  if (xaxisp->minint())
	    bix=ix-xspace;
	}
    }
  if (yaxisp)
    {
      yaxisp->getaxispoints(sy,iy,ey);
      biy=iy;
      if (sy>=ey)
	yaxisp=NULL;
      else
	{
	  if (usespacer==box_spacer || usespacer==xy_spacer)
	    yspace=((yaxisp->symbolsize())/cyscale)*4;
	  else
	    yspace=((yaxisp->symbolsize())/cyscale)*2;

	  if (yaxisp->minint())
	    biy=iy-yspace;
	}
    }
  if (zaxisp)
    {
      zaxisp->getaxispoints(sz,iz,ez);
      biz=iz;
      if (sz>=ez)
	zaxisp=NULL;
      else
	{
	  if (usespacer==box_spacer || usespacer==z_spacer)
	    zspace=((zaxisp->symbolsize())/czscale)*4;
	  else
	    {
	      // leave a tiny space so that floor does not interfere
	      zspace=(ez-sz)/1000;
	      //	      zspace=((zaxisp->symbolsize())/czscale)*5;
	    }

	  if (zaxisp->minint())
	    biz=iz-zspace;
	}
    }
  //  cout<< sx<<" "<<ix<<" "<<ex << "     " << sy<<" "<<iy<<" "<<ey << "     " << sz<<" "<<iz<<" "<<ez<<"\n";

  // boundaries for displaying axes
  bsx=sx-xspace;
  bsy=sy-yspace;
  bsz=sz-zspace;
  bex=ex+xspace;
  bey=ey+yspace;
  bez=ez+zspace;
  xthickness=xspace/2;
  ythickness=yspace/2;
  zthickness=zspace/2;
  // modified to lower axes surface
  bsz2=bsz-(bez-bsz)/1000;

  // a box surrounding contents?
  if (clearbox)
    {
      // draw a box enclosing the axes
      drawBox(bsx,bsy,bsz,bex,bey,bez);
    }

  // Now draw the axes and axes walls
  if (xaxisp)
    {
      // draw any border walls
      if (xaxisp->getminwall()!=no_wall)
	drawWall(bsx,bsy,bsz,bsx,bey,bez,xthickness,xaxisp->getminwall(),detail);
      if (xaxisp->getmaxwall()!=no_wall && ex!=sx)
	drawWall(bex,bsy,bsz,bex,bey,bez,xthickness,xaxisp->getmaxwall(),detail);
      if (xaxisp->getintwall()!=no_wall && ix!=sx)
	drawWall(ix-(xthickness/2),bsy,bsz,ix-(xthickness/2),bey,bez,xthickness,xaxisp->getintwall(),detail);
      // draw the axis
      if (xaxisp->getshow())
	drawAxis(&widthx,&heightx,&depthx,xaxisp,bsx,bsy,bsz2,bex,bsy,bsz2,bitfont,detail,cxscale,cyscale,czscale,xspace,linewidth);
    }
  if (yaxisp)
    {
      // draw any border walls
      if (yaxisp->getminwall()!=no_wall)
	drawWall(bsx,bsy,bsz,bex,bsy,bez,ythickness,yaxisp->getminwall(),detail);
      if (yaxisp->getmaxwall()!=no_wall && ey!=sy)
	drawWall(bsx,bey,bsz,bex,bey,bez,ythickness,yaxisp->getmaxwall(),detail);
      if (yaxisp->getintwall()!=no_wall && iy!=sy)
	drawWall(bsx,iy-(ythickness/2),bsz,bex,iy-(ythickness/2),bez,ythickness,yaxisp->getintwall(),detail);
      // draw the axis
      if (yaxisp->getshow())
	drawAxis(&widthy,&heighty,&depthy,yaxisp,bsx,bsy,bsz2,bsx,bey,bsz2,bitfont,detail,cxscale,cyscale,czscale,yspace,linewidth);
    }
  if (zaxisp)
    {
      // draw any border walls
      if (zaxisp->getminwall()!=no_wall)
	drawWall(bsx,bsy,bsz,bex,bey,bsz,zthickness,zaxisp->getminwall(),detail);
      if (zaxisp->getmaxwall()!=no_wall && ez!=sz)
	drawWall(bsx,bsy,bez,bex,bey,bez,zthickness,zaxisp->getmaxwall(),detail);
      if (zaxisp->getintwall()!=no_wall && iz!=sz)
	drawWall(bsx,bsy,iz-(zthickness/2),bex,bey,iz-(zthickness/2),zthickness,zaxisp->getintwall(),detail);
      // draw the axis
      if (zaxisp->getshow())
	drawAxis(&widthz,&heightz,&depthz,zaxisp,bsx,bey,bsz2,bsx,bey,bez,bitfont,detail,cxscale,cyscale,czscale,zspace,linewidth);
    }

  if (detail==finedetail)
    {
      // for fine details, we draw a 3d platform for the axis labels we just drew, this is a little tricky
      drawFloor(xaxisp,yaxisp,zaxisp,heightx,heighty,heightz,depthx,depthy,depthz,bsx,bex,bsy,bey,bsz2,bez);
    }

  // cleanup
  endlist();
  glLineWidth(olinewidth);
  return displaylistnum;
}






void GrobGlHelper::drawFloor(VisibleAxis *xaxisp,VisibleAxis *yaxisp,VisibleAxis *zaxisp,float heightx,float heighty,float heightz,float depthx,float depthy,float depthz,float bsx,float bex,float bsy,float bey,float bsz2,float bez)
{
      GLfloat specular[] = {.7, .7, .7, .8};
      GLfloat shininess[] = {75};
      GLfloat triColor2[] = {1, 0.5, .05, .2};
      glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, triColor2);
      glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
      glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);
      float xbuffer,ybuffer,zbuffer;
      float nsx,nsy,nsz,nex,ney,nez;

      xbuffer=heighty;
      ybuffer=heightx;
      if (depthx==0)
	zbuffer=depthy;
      else
	zbuffer=depthx;

      // main surface (xy)
      nsx=bsx-xbuffer;
      if (ybuffer!=0)
	nex=bex+xbuffer;
      else
	nex=bex;
      nsy=bsy-ybuffer;
      
      if (xbuffer!=0)
	ney=bey+ybuffer;
      else
	ney=bey;
      nsz=bsz2;
      nez=(bsz2-zbuffer);
      glPushMatrix();
      glTranslatef((nex+nsx)/2,(ney+nsy)/2,(nez+nsz)/2);
      drawMechBox(nex-nsx,ney-nsy,nez-nsz,1);
      glPopMatrix();

      if (zaxisp)
	{
	  if (zaxisp->getshow())
	    {
	      // z pole
	      nsx=bsx-heightz;
	      nex=bsx;
	      nsy=bey;
	      ney=bey+depthz;
	      if (xbuffer==0 || ybuffer==0)
		nez=bsz2-zbuffer;
	      else
		nez=bsz2;
	      nsz=bez;
	      glTranslatef((nex+nsx)/2,(ney+nsy)/2,(nez+nsz)/2);
	      drawMechBox(nex-nsx,ney-nsy,nez-nsz,1);
	    }
	}
}









void GrobGlHelper::drawAxis(float *widthp,float *heightp,float *depthp,VisibleAxis *axisp,float sx,float sy,float sz,float ex,float ey,float ez,bool bitfont,GrobDetail detail,float cxscale,float cyscale,float czscale,float space,int linewidth)
{
  // draw an axis with labels and ticks and numbers
  string label=axisp->getlabel();
  int st,et;

  // turn off lighting
  pushlighting();
  lightsoff();
  glColor3f(1,1,1);

  // draw the label
  if (label!="")
    {
      if (sx!=ex)
	{
	 drawAxisLabel(axisp,widthp,heightp,depthp,sx,sy,sz,ex-sx,cxscale,cyscale,czscale,space,detail,0,linewidth);
	}
      else if (sy!=ey)
	{
	 drawAxisLabel(axisp,widthp,heightp,depthp,sx,sy,sz,ey-sy,cxscale,cyscale,czscale,space,detail,1,linewidth);
	}
      else if (sz!=ez)
	{
	 drawAxisLabel(axisp,widthp,heightp,depthp,sx,sy,sz,ez-sz,cxscale,cyscale,czscale,space,detail,2,linewidth);
	}
    }

  // restore lighting
  poplighting();
}
















void GrobGlHelper::drawAxisLabel(VisibleAxis *axisp,float *rwidthp,float *rheightp,float *rdepthp,float sx, float sy, float sz, float length,float xscale, float yscale, float zscale,float space,GrobDetail detail,int axis,int linewidth)
{
  // draw ticks, labels, numbers, etc. for the axis.
  // when we are done, set the width,height,depth that we used
  float width;
  float wscale,hscale,dscale;
  float totalheight=0.0;
  float d=0;
  float vertdir;

  // prepare for drawing
  pushlighting();
  glPushMatrix();

  // go to depth lessequal so that labels are visible at equal depth of box, and prepare for labels
  depthoverwrite();
  lightsoff();

  // find allignment of text axes with rotation axis, so that all drawing can be done uniformally across the GL X axis
  switch(axis)
    {
    case 0:
      // x axis
      wscale=xscale;
      hscale=yscale;
      dscale=zscale;
      vertdir=1;
      break;
    case 1:
      // y axis
      wscale=yscale;
      hscale=xscale;
      dscale=zscale;
      vertdir=-1;
      break;
    case 2:
      // z axis
      wscale=zscale;
      hscale=xscale;
      dscale=yscale;
      vertdir=-1;
      break;
    }

  // length of the axes space
  width=(1/wscale)+space*2;

  // now translate to start position of text
  // note: y&z axes are oriented reversly along width (ie text goes from max to min), so we need to 
  switch(axis)
    {
    case 0:
      // x axis
      glTranslatef(sx,sy,sz);
      break;
    case 1:
      // y axis
      glTranslatef(sx,sy,sz);
      break;
    case 2:
      // z axis
      glTranslatef(sx,sy,sz);
      break;
    }

  // rotate to align x axis for text
  rotatetoaxis(axis);

  // now we are positioned and ready for writing text
  // push matrix for later popping
  glPushMatrix();


  // NOW CALL TEXT WRITERs

  // write ticks
  glPopMatrix();  glPushMatrix();  glTranslatef(0,-totalheight*vertdir,0);
  totalheight+=AxisWriter_Ticks(axisp,sx,wscale,hscale,dscale,width,length,space,linewidth,vertdir);

  // write label
  glPopMatrix();  glPushMatrix();  glTranslatef(0,-totalheight*vertdir,0);
  totalheight+=AxisWriter_Label(axisp,axisp->getlabel(),wscale,hscale,dscale,width,space,linewidth,vertdir);

  // write enums
  glPopMatrix();  glPushMatrix();  glTranslatef(0,-totalheight*vertdir,0);
  totalheight+=AxisWriter_Enums(axisp,wscale,hscale,dscale,width,space,linewidth,vertdir);


  // restore state
  glPopMatrix();
  depthnormal();
  glPopMatrix();
  poplighting();

  // set the return width and depth if approp.
  if (rwidthp!=NULL)
    *rwidthp=width;
  if (rheightp!=NULL)
    *rheightp=totalheight;
  if (rdepthp!=NULL)
    *rdepthp=d;
}




float GrobGlHelper::AxisWriter_Label(VisibleAxis *axisp,string label,float wscale,float hscale,float dscale,float width,float space,int linewidth,float virtdir)
{
  // Simply write the text label, centered on axis, and return the height used by the label
  float height;
  const char *message;
  float textscale=testtextscale;
  float minscreensize=0;

  // text to display
  message=label.c_str();
  height=draw_labeltext(message,textscale,wscale,hscale,dscale,width/2,width,minscreensize,virtdir);

  return height;
}



float GrobGlHelper::draw_labeltext(const char *message,float textscale,float wscale,float hscale,float dscale,float xpos,float width,float minscreensize,float virtdir,bool outsideskip,bool spacetest)
{
  // write some text centered along the x axis.  if text is too small (<minscreensize), don't draw it.
  // return the height used up.
  // if spacetest==true, then don't display text, just test for fit and return would-be height

  float height;
  float heightfull=1.5;
  float widthfull=1.2;
  float textwidth,textheight;
  float charwidth,charheight;
  float widthoff,heightoff;
  float stextwidth,stextheight;
  float txscale,tyscale,tzscale;
  void *font=(void *)GLUT_STROKE_ROMAN;
  
  // guess normalized text size
  txscale=textscale/wscale;
  tyscale=textscale/hscale;
  tzscale=textscale/dscale;

  // real text heights
  charheight=110*textscale;
  textheight=charheight;
  stextheight=textheight/hscale;

  // real text widths (should be replaced by a GL query, but i couldnt get it to work)
  charwidth=75*textscale;
  textwidth=strlen(message)*charwidth;
  stextwidth=textwidth/wscale;
  widthoff=stextwidth/2;

  // full height we are going to use (includes extra space)
  height=stextheight*heightfull;
  heightoff=(height-stextheight)/2;

  if (stextwidth*widthfull>width)
    {
      // text is too big
      //      cout << "text is too big.\n";
      if (outsideskip || spacetest)
	return 0.0;
    }
  else if (spacetest)
    {
      // this call was just a test, and it fits
      return height;
    }

  // save initial state
  glPushMatrix();

  // Now draw the label
  if (virtdir==1)
    glTranslatef(xpos-widthoff,-height+heightoff,0);
  else
    glTranslatef(xpos-widthoff,heightoff,0);

  glScalef(txscale, tyscale, tzscale);
  drawStrokeText(font,message);

  // restore state
  glPopMatrix();

  return height;
}





float GrobGlHelper::AxisWriter_Ticks(VisibleAxis *axisp,float xpos,float wscale,float hscale,float dscale,float width,float length,float space,int linewidth,float virtdir)
{
  // draw ticks and number them if appropriate, and then return the height used by the ticks & numbers
  // we want to move across the axis, skipping border spacers, drawing minor and major ticks
  // and number the major ticks
  float height;
  char ticktext[100];
  int count;
  GLfloat orig_linewidth;
  float x1,x2;
  float minorinterval,majorinterval;
  float intercept;
  float sx,ex;
  float minorheight,majorheight;
  float minspace;
  float textscale=testtextscale/1.5;
  float minscreensize=0;
  float maxheight=0;
  float tickwidth=.8;
  bool doublexpand=true;

  // save initial state
  glPushMatrix();

  // get intervals for ticks
  minorinterval=axisp->getminorticks();
  majorinterval=axisp->getmajorticks();
  intercept=axisp->getintercept();
  sx=axisp->getmin();
  ex=axisp->getmax();

  // heuristic size of ticks, scaled to axis normalizations (ie percentage of axis size)
  //  should be replaced by caller specified majorheight
  minorheight=.025/hscale;
  majorheight=.05/hscale;

  if (minorinterval<=0)
    {
      // no ticks
      return 0.0;
    }

  // make a buffer so that rounding errors don't prevent the border ticks from showing
  minspace=minorinterval/10.0;

  // First we have to see what majorinterval for numbering will really fit
  if (majorinterval>0)
    {
      x1=intercept;
      axisnumbertext(ticktext,x1,sx,ex);
      height=draw_labeltext(ticktext,textscale,wscale,hscale,dscale,(x1-sx)+space,majorinterval*minorinterval*tickwidth,minscreensize,virtdir,false,true);
      for (count=1;;++count)
	{
	  x1=intercept+count*majorinterval*minorinterval;
	  x2=intercept-count*majorinterval*minorinterval;
	  if (x1>ex+minspace && x2<sx-minspace)
	    break;
	  if (x1<=ex+minspace)
	    {
	      axisnumbertext(ticktext,x1,sx,ex);
	      height=draw_labeltext(ticktext,textscale,wscale,hscale,dscale,(x1-sx)+space,majorinterval*minorinterval*tickwidth,minscreensize,virtdir,true,true);
	      if (height==0)
		{
		  // label didn't fit, so adjust interval and try again
		  if (doublexpand)
		    majorinterval*=2;
		  else
		    majorinterval+=1;
		  --count;
		  continue;
		}
	      }
	  if (x2>=sx-minspace && count!=0)
	    {
	      axisnumbertext(ticktext,x2,sx,ex);
	      height=draw_labeltext(ticktext,textscale,wscale,hscale,dscale,(x2-sx)+space,majorinterval*minorinterval*tickwidth,minscreensize,virtdir,true,true);
	      if (height==0)
		{
		  // label didn't fit, so adjust interval and try again
		  if (doublexpand)
		    majorinterval*=2;
		  else
		    majorinterval+=1;
		  --count;
		  continue;
		}
	    }
	}
    }



  // draw ticks, centered at intercept (which may be sx)
  glBegin(GL_LINES);
  for (count=0;;++count)
    {
      x1=intercept+count*minorinterval;
      x2=intercept-count*minorinterval;
      if (x1>ex+minspace && x2<sx-minspace)
	break;
      if (x1<=ex+minspace)
	{
	  if ((count%(int)majorinterval)==0)
	    {
	      // it's a major tick
	      glVertex3f((x1-sx)+space, 0, 0); glVertex3f((x1-sx)+space, majorheight*virtdir, 0);
	    }
	  else
	    {
	      // minor tick
	      glVertex3f((x1-sx)+space, 0, 0); glVertex3f((x1-sx)+space, minorheight*virtdir, 0);
	    }
	}
      if (x2>=sx-minspace && count!=0)
	{
	  if ((count%(int)majorinterval)==0)
	    {
	      // it's a major tick
	      glVertex3f((x2-sx)+space, 0, 0); glVertex3f((x2-sx)+space, majorheight*virtdir, 0);
	    }
	  else
	    {
	      // minor tick
	      glVertex3f((x2-sx)+space, 0, 0); glVertex3f((x2-sx)+space, minorheight*virtdir, 0);
	    }
	}
    }
  glEnd();


  // always use thin lines for numeric labels.
  // with real big graphs, thick is better, but it's rare.
  glLineWidth(1);


  // write major tick numbers (note this cannot be done inside the glBeing loop above)
  if (majorinterval>0)
    {
      x1=intercept;
      axisnumbertext(ticktext,x1,sx,ex);
      height=draw_labeltext(ticktext,textscale,wscale,hscale,dscale,(x1-sx)+space,majorinterval*minorinterval*tickwidth,minscreensize,virtdir,false);
      if (height>maxheight)
	maxheight=height;
      for (count=1;;++count)
	{
	  x1=intercept+count*majorinterval*minorinterval;
	  x2=intercept-count*majorinterval*minorinterval;
	  if (x1>ex+minspace && x2<sx-minspace)
	    break;
	  if (x1<=ex+minspace)
	    {
	      axisnumbertext(ticktext,x1,sx,ex);
	      height=draw_labeltext(ticktext,textscale,wscale,hscale,dscale,(x1-sx)+space,majorinterval*minorinterval*tickwidth,minscreensize,virtdir,false);
	      if (height>maxheight)
		maxheight=height;
	      }
	  if (x2>=sx-minspace && count!=0)
	    {
	      axisnumbertext(ticktext,x2,sx,ex);
	      height=draw_labeltext(ticktext,textscale,wscale,hscale,dscale,(x2-sx)+space,majorinterval*minorinterval*tickwidth,minscreensize,virtdir,false);
	      if (height>maxheight)
		maxheight=height;
	    }
	}
    }



  // restore state
  glPopMatrix();
  glLineWidth(linewidth);

  return maxheight;
}




float GrobGlHelper::AxisWriter_Enums(VisibleAxis *axisp,float wscale,float hscale,float dscale,float width,float space,int linewidth,float virtdir)
{
  // write any enumerated labels for axis values, and return the height used up
  return 0.0;
}


void GrobGlHelper::axisnumbertext(char *charnum,float val,float min,float max)
{
  // set the string to the sprintf version of the val, nicely formatted for axis display
  // this is kind of a kludge to get reasonable percision numbers for the axis
  float range=max-min;
  int decimals;
  double dval;
  long lval;
  double dpower;

  if (range==0)
    {
      sprintf(charnum,"%g",val);
      return;
    }

  // decide how many decimal places it's reasonable to display, given range of value
  //  ie if value has wide range (ex. 0 to 100). then the decimals don't help
  //     wheras if val goes from 0.01 to 0.02, then the decimals are important.
  //  we use 3-log10(range) as a heuristic:
  //     range > 1000 gets 0 decimals
  //     range < .001 gets 6 decimals
  decimals=(int)(3-log10(range));
  if (decimals<0)
    decimals=0;
  if (decimals>6)
    {
      // let %g round it
      sprintf(charnum,"%g",val);
      return;
    }

  // round the value to the decimals chosen
  dpower=pow(10,decimals);
  dval=val*dpower;
  lval=(long)dval;
  dval=(double)(lval/dpower);

  sprintf(charnum,"%g",dval);
  return;
}






// }
// Namespace close-brace
