// grobcamera.C


#include "grobcamera.h"
#include "JRmat.h"

//Not implemented in gcc yet (2.8.1)
//using namespace std;
//namespace Grob{


#include "grobinline.h"
#include "grobcamerainline.h"









// ----------------------------------------------------------------------------------
// CameraView
// ----------------------------------------------------------------------------------

CameraView::CameraView(float ine1,float ine2,float ine3)
{
  // make a view which holds euler angles
  initialize();
  seteulers(ine1,ine2,ine3);
}

CameraView::CameraView(float ine1,float ine2,float ine3,float inzoom,GrobProjectionMode inpmode)
{
  // make a view which holds all info
  initialize();
  seteulers(ine1,ine2,ine3);
  setzoom(inzoom);
  setpmode(inpmode);
}

CameraView::CameraView(float ine1,float ine2,float ine3,float inzoom)
{
  // make a view which holds all info
  initialize();
  seteulers(ine1,ine2,ine3);
  setzoom(inzoom);
}

CameraView::CameraView(float ine1,float ine2,float ine3,float inzoom,float inydolly)
{
  // make a view which holds all info
  initialize();
  seteulers(ine1,ine2,ine3);
  setzoom(inzoom);
  setydolly(inydolly);
}

void CameraView::initialize()
{
  // reset use flags to say view is empty
  use_eulers=use_zoomfactor=use_projectionmode=use_dolly=false;
}


void CameraView::restore(float *oe1,float *oe2,float *oe3,float *ozoom,float *oydolly,GrobProjectionMode *opmode)
{
  // restore the appropriate & requested parameters from a saved view
  if (use_eulers)
    {
      // eulers were saved in the view, set them if requested
      if (oe1)
	*oe1=e1;
      if (oe2)
	*oe2=e2;
      if (oe2)
	*oe3=e3;
    }
  if (use_zoomfactor && ozoom)
    *ozoom=zoomfactor;
  if (use_projectionmode && opmode)
    *opmode=pmode;
  if (use_dolly && oydolly)
    *oydolly=ydolly;
}


























// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::construct()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::construct()
{
  // initialize the videocamera
  detail=lowdetail;
  focus_object=(Grob *)NULL;
  gvc_prepared=false;
  gdrawerp=NULL;
  gaxes=gtitle=glegend=gwalls=NULL;
  showaxes=false;
  showtitle=true;
  showlegend=false;
  showwalls=false;
  autocenter=true;
  autoscale=false;
  refreshcounter=0;
  redundantcounter=0;
  redrawrequested=false;
  needsrecalc=true;
  needsredraw=true;
  drawing_engaged=false;
  win_xpos=win_ypos=-1;
  win_width=100;win_height=100;
  viewsx=viewsy=0;
  vieww=win_width;viewh=win_height;
  clearcolor[0]=clearcolor[1]=clearcolor[2]=clearcolor[3]=0.0;
  linewidth=1;
  clearbox=false;
  usespacer=box_spacer;

  gaxes=gtitle=glegend=gwalls=NULL;
  xaxisp=yaxisp=zaxisp=NULL;

  title="";
  subtitle="";
  
  distance_lock=true;
  euler1=M_PI/2.0; euler2=0.0;  euler3=0.0;
  location=perVector(3);
  location.fill(0.0);
  location(2)=200;
  orientation=perMatrix(3,3);
  orientation.fill(0.0); orientation(0,0)=1.0;orientation(1,1)=1.0;orientation(2,2)=0.0;
  projectionmode=perspective;
  xskew=yskew=zskew=1.0;
  cxscale=.1*xskew;cyscale=.1*yskew;czscale=.1*zskew;

  // defaults
  distance=100;
  zoomfactor=1.0;
  aperture=1;
  aspect=1.0;
  znear=1.0;
  zfar=500.0;

  // test
  distance=60;
  znear=1.0;
  zfar=500.0;

  viewradius=zfar/5;
  refreshrate=1;

  focus_lock=true;
  focus_location.fill(0.0);
}



  
// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::gvc_prepare()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::gvc_prepare()
{
  // this function gets called before the very first drawing operation on the camera,
  // and not before (this is good for doing this that require the window to be onscreen,etc.)
  gvc_gl_prepare();

  // set the flag so we don't do this twice
  gvc_prepared=true;
}



// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::gvc_gl_prepare()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::gvc_gl_prepare()
{
  // initialize gl preparations

  // set up the lights in preparation for drawing
  GrobGlHelper::setuplights();
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glClearColor(clearcolor[0],clearcolor[1],clearcolor[2],clearcolor[3]);

  // set detail-specific rendering options
  detailchange();

  // make the initial stuff we need
  makeaxes();
  makelegend();
  makewalls();
  //  if (title!="" || subtitle!="")
  //    maketitle();

  //  update_eulers();
  //  recalc();


  if (views.size()==0)
    {
      // make the default views, and initialize
      add_defaultviews();
    }
  else
    setview(curview);
}



// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::destruct()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::destruct()
{
  // destruct
  gl_destruct();

  // should free any saved views
}



// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::gl_destruct()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::gl_destruct()
{
  // delete any gl preparations
  if (gaxes)
    delete gaxes;
  if (gtitle)
    delete gtitle;
  if (glegend)
    delete glegend;
  if (gwalls)
    delete gwalls;
}




// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::makeaxes()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::makeaxes()
{
  // make axes

  // does drawer want us to use a spacer
  usespacer=gdrawerp->wantspacer();

  if (gaxes)
    delete gaxes;
  gaxes=new Grob();
  if (xaxisp || yaxisp || zaxisp)
    gaxes->makeAxesList(xaxisp,yaxisp,zaxisp,cxscale,cyscale,czscale,usespacer,linewidth,clearbox);
  else
    gaxes->makeAxesList(viewradius,linewidth);
  needsredraw=true;
}



// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::maketitle()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::maketitle()
{
  // make title grob
  if (gtitle)
    delete gtitle;

  gtitle=new Grob();
  gtitle->makeTextBoxList(title,subtitle,win_width,win_height);
}


// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::makelegend()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::makelegend()
{
  // make legend grob
  return;
  if (glegend)
    delete glegend;
  glegend=new Grob();
}


// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::makewalls()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::makewalls()
{
  // make grob axis walls
  return;
  if (gwalls)
    delete gwalls;
  gwalls=new Grob();
}




















// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::setpos()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::setpos(perVectorR3 inlocation,perMatrix inorientation)
{
  // set location
  location=inlocation;
  
  // set orientation
  orientation=inorientation;
}


// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::setlocation()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::setlocation(perVectorR3 inlocation)
{
  // set location
  location=inlocation;
}


// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::setorientation()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::setorientation(perMatrix inorientation)
{
  // set orientation
  orientation=inorientation;
}

void GrobVideoCamera::setfocus(float x,float y, float z)
{
  focus_location(0)=x;
  focus_location(1)=y;
  focus_location(2)=z;
  needsredraw=true;
  needsrecalc=true;

  //  cout <<" focus =["<< x <<" " <<y<<" " <<z<<"].\n";
}





// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::begin_draw()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::begin_draw()
{
  // this is called before each pass of drawing objects

  //  cout << "in camera begindraw, zoom=" << zoomfactor << ".\n";

  // clear wantsredraw flag to signify that we are now drawing the latest view
  redrawrequested=false;
  needsredraw=false;

  // if this is our first draw ever, prepare anything we might need
  if (!gvc_prepared)
    gvc_prepare();
  
  // clear the display buffer and the display buffer
  glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
  glClearColor(clearcolor[0],clearcolor[1],clearcolor[2],clearcolor[3]);

  // show stuff that fails elsewhere
  //  if (showtitle && gtitle)
  //    gtitle->draw(this,detail);
  show_title();

  if (needsrecalc)
    {
      update_eulers();
      update_quats();
    }

  // has camera changed position, orientation, etc.?
  recalc();

  drawing_engaged=true;
}



// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::end_draw()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::end_draw()
{
  // all objects have been drawn, now show ancillaries
  if (showaxes && gaxes)
    gaxes->draw(this,detail);
  if (showlegend && glegend)
    glegend->draw(this,detail);
  if (showwalls && gwalls)
    gwalls->draw(this,detail);

  drawing_engaged=false;
}





// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::begin_incdraw()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::begin_incdraw()
{
  // this is called before each pass of INCREMENTAL drawing objects
  // clear wantsredraw flag to signify that we are now drawing the latest view
  redrawrequested=false;

  if (showtitle && gtitle)
    gtitle->draw(this,detail);
}

// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::end_incdraw()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::end_incdraw()
{
  // all objects have been INCREMENTALLY drawn
  if (showtitle && gtitle)
    gtitle->draw(this,detail);
}
















// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::update_eulers()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::update_eulers()
{
  // update orientation based on current eulers
  // User has changed orientation (or distance) of camera
  // if focus_lock==YES then we must change the location so that camera looks
  //  at focus object/location
  
  // compute the new transform matrix based on euler angles
  JRmathelper::ConvertEuler(euler1,euler2,euler3,orientation);
}



// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::update_quats()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::update_quats()
{
  // update orientation based on current quats
}






// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::location_change()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::location_change()
{
  // User has changed location of camera
  // if focus_lock==YES then we must change the orientation so that camera
  //   looks at focus object/location
}


void GrobVideoCamera::change()
{
  // request a redraw
  needsrecalc=true;
  needsredraw=true;
  request_redraw(gdrawerp);
}








// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::recalc()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::recalc()
{
  // a relevant camera parameter has cnaged, and so we need to
  //  readjust other parameters, and recalc. some other derived values
  perVectorR3 sizevec;
  perVectorR3 distvec;
  float mdistance;

  // clear flag
  needsrecalc=false;

  // First update the focus location
  if (focus_lock && focus_object!=(Grob *)NULL)
    {
      // this camera is focused on an object, so we need to update the focus_location for further processing.
      focus_location=focus_object->getlocation();
    }

  sizevec(0)=distance; sizevec(1)=0.0; sizevec(2)=0.0;
  if (focus_lock)
    {
      // our current location changes to that relative to target
      if (true || distance_lock)
	{
	  // we move the camera to keep our distance constant
	  location=focus_location-(orientation*(perVector)sizevec);
	}
      else
	{
	  // instead of moving the camera spatially
	  // we re-orient the camera to keep focus on object, from current location
	  distvec=focus_location-location;
	  mdistance=distvec(0)*distvec(0)+distvec(1)*distvec(1)+distvec(2)*distvec(2);
	  mdistance=sqrt(mdistance);
	  focus_location=location+(distvec*(distance/mdistance));
	  // now we need to set the orientation based on location and focus_location
	  // BUT i don't know how to do this...
	}
    }
  else
    {
      // see we don't a a specific location, we just compute what we are looking at
      focus_location=location+(orientation*(perVector)sizevec);
    }

  // compute the upector
  sizevec(0)=0.0; sizevec(1)=0.0; sizevec(2)=-10.0;
  upvector=(orientation*(perVector)sizevec);
  
  // okay, now update the internals
  //  update_internals();
  
  // and set the eye
  seteye();
}


// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::update_internals()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::update_internals()
{
  // Update the internal camera stuff that we need, based on routines
  //  from [Paul Bourke 94] (via internet).  THIS IS IF WE DON'T HAVE GLlookat?
  
  basisb=focus_location-location;
  JRmathelper::Normalize(basisb);
  basisa=JRmathelper::Cross(upvector,basisb);
  JRmathelper::Normalize(basisa);
  basisc=JRmathelper::Cross(basisb,basisa);

  tanthetah=tan(aperture/2.0);
  tanthetav=tan(aperture/2.0);
}



// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::seteye()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::seteye()
{
  // update our new camera view using gluPerspective and gluLookAt
  float mx,my,mz;
  float cx,cy,cz;
  float zf,izf;
  float scaler;

  // so we can push the new matrix later
  glPopMatrix();

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  if (projectionmode==perspective)
    {
      gluPerspective(aperture*RAD2DEG, aspect, znear, zfar);
      scaler=10.0;
    }
  else
    {
      // orthoggraphic projection assumes that view cscaled axis range is -1 to 1
      //  which should be enforced by regulating cxscale,cyscale,czscale, etc.
      // ortho projection is esp. useful for 2d drawing of 2d/3d plots, as the
      //  borders are cleanly rendered in 2d.
      glOrtho(-2,2,-2,2,znear,zfar);
      scaler=2.0;
    }

  cx=cxscale*scaler;
  cy=cyscale*scaler;
  cz=czscale*scaler;

  // scaling for zoom and axis normalization
  mx=zoomfactor*cx;
  my=zoomfactor*cy;
  mz=zoomfactor*cz;

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  // multiple gl windows requires us to reset global gl params each time.
  detailchange();

  // The following sequence took me 2 days to figure out; it's because things get confusing when you
  //  combine translation with scaling, etc. and we want autonormalization of axes.
  // Basically, we are at some location, looking at a focus point which may not be the orgin.
  //  the key thing that finally worked for me (is there a more efficient way? probably) was to
  //  add a translation at the end to center everything that gets drawn to the focus point.
  // Then the scaling and normalization seems to take okay.
  // Naively i thought i could do away with the firs and last translations...
  //  but note how tricky the order is:  1) camera location (translated to look at 0,0,0)
  //  followed by 2) the normalization/zoom
  //  followed by 3) translation of every object to center at 0,0,0

  // Show the headmounted lights
  show_lights_headmounted();

  // look at our focus
  gluLookAt(location(0)-focus_location(0), location(1)-focus_location(1), location(2)-focus_location(2), 0, 0, 0, upvector(0),upvector(1),upvector(2));

  // Show the scene lights
  show_lights_scene();

  // scale for zoom and axis normalization factors
  glScalef(mx,my,mz);

  // now move everything in world centered at focus_location to 0,0,0 where we are looking
  glTranslatef(-focus_location(0),-focus_location(1),-focus_location(2));

  // push matrix for later popping (see top)
  glPushMatrix();

  // set the viewport
  // doesn't work? why?
  glViewport(viewsx,viewsy,vieww,viewh);
  //  cout << "viewsx=" << viewsx<< "   viewsy="<<viewsy<< ".\n";
}







// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::show_lights()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::show_lights_scene()
{
  // show lights which are fixed in the world

  if (detail==lowdetail || detail==meddetail)
    {
    }
  else if (detail==highdetail)
    {
    }
  else if (detail==finedetail)
    {
      GLfloat lightTwoPosition[] = {1, -1, -1, 0};
      //      GLfloat lightThreePosition[] = {1, 0, -1, 0};
      GLfloat lightThreePosition[] = {-.1, -.1, -.1, 0};
      GLfloat lightFourPosition[] = {-1, -1, 1, 0};
      glLightfv(GL_LIGHT2, GL_POSITION, lightTwoPosition);
      glLightfv(GL_LIGHT3, GL_POSITION, lightThreePosition);
      glLightfv(GL_LIGHT4, GL_POSITION, lightFourPosition);
    }
}


void GrobVideoCamera::show_lights_headmounted()
{
  // show lights which move with viewer

  GLfloat lightZeroPosition[] = {-.3, 1, 1, 0};
  //  GLfloat lightOnePosition[] = {-.1, -.1, -.1, 0};
  GLfloat lightOnePosition[] = {1, 0, -1, 0};

  if (detail==lowdetail || detail==meddetail)
    {
    }
  else if (detail==highdetail)
    {
      glLightfv(GL_LIGHT0, GL_POSITION, lightZeroPosition);
    }
  else if (detail==finedetail)
    {
      glLightfv(GL_LIGHT0, GL_POSITION, lightZeroPosition);
      glLightfv(GL_LIGHT1, GL_POSITION, lightOnePosition);
    }
}









// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::detailchange()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::detailchange()
{
  // someone has changed the detail level of the camera
  GLfloat lmodel_ambient1[]={1,1,1,1.0};
  GLfloat lmodel_ambient2[]={.6,.6,.6,.6};
  //  GLfloat lmodel_ambient3[]={.3,.3,.3,.3};
  GLfloat lmodel_ambient3[]={.4,.4,.4,.4};

  switch (detail)
    {
    case lowdetail:
      GrobGlHelper::lightsoff();
    case meddetail:
      // no lights, fastest rendering
      GrobGlHelper::lightson();
      glLightModelfv(GL_LIGHT_MODEL_AMBIENT,lmodel_ambient1);
      glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_FALSE);
      glShadeModel(GL_FLAT);
      glDisable(GL_LIGHT0);
      glDisable(GL_LIGHT1);
      glDisable(GL_LIGHT2);
      glDisable(GL_LIGHT3);
      glDisable(GL_LIGHT4);
      //      glHint(GL_LINE_SMOOTH_HINT,GL_FASTEST);
      //      glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_FASTEST);
      //      glEnable(GL_CULL_FACE);
      glDisable(GL_LINE_SMOOTH);
      setlinewidth(1);
      break;
    case highdetail:
      GrobGlHelper::lightson();
      glLightModelfv(GL_LIGHT_MODEL_AMBIENT,lmodel_ambient2);
      glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_FALSE);
      //      glShadeModel(GL_SMOOTH);
      glShadeModel(GL_FLAT);
      glEnable(GL_LIGHT0);
      glDisable(GL_LIGHT1);
      glDisable(GL_LIGHT2);
      glDisable(GL_LIGHT3);
      glDisable(GL_LIGHT4);
      //      glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_FASTEST);
      //      glDisable(GL_BLEND);
      //      glHint(GL_LINE_SMOOTH_HINT,GL_FASTEST);
      // I spent about 17 hours trying to figure out why the lighting was so screwed up during zoom.  next line magically fixes it.
      setlinewidth(2);
      glDisable(GL_LINE_SMOOTH);
      glEnable(GL_NORMALIZE);
      break;
    case finedetail:
      GrobGlHelper::lightson();
      glLightModelfv(GL_LIGHT_MODEL_AMBIENT,lmodel_ambient3);
      glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
      glShadeModel(GL_SMOOTH);
      glEnable(GL_LIGHT0);
      glEnable(GL_LIGHT1);
      glEnable(GL_LIGHT2);
      glEnable(GL_LIGHT3);
      glEnable(GL_LIGHT4);
      //      glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
      //      glDisable(GL_CULL_FACE);
      //      glEnable(GL_BLEND);
      //      glEnable(GL_LINE_SMOOTH);
      //      setlinewidth(2);
      setlinewidth(2);
      // why does smoothing look so bad? (alpha values?)
      //      glEnable(GL_LINE_SMOOTH);
      //      glHint(GL_LINE_SMOOTH_HINT,GL_NICEST);
      // I spent about 17 hours trying to figure out why the lighting was so screwed up during zoom.  next line magically fixes it.
      glEnable(GL_NORMALIZE);
      break;
    }

  // test
  //  glDisable(GL_LIGHT2);
  glDisable(GL_LIGHT4);
}

















































// ----------------------------------------------------------------------------------
// -- Replacement routines for non GL simulation
// ----------------------------------------------------------------------------------




// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::world2eye()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::world2eye(float ix,float iy,float iz,float &ox,float &oy,float &oz)
{
  // go from world coords to eye coords
  //  from [Paul Bourke 94] (via internet)

  // translate world so camera is at origin
  ix-=location(0);
  iy-=location(1);
  iz-=location(2);

  // now convert to eye coords using basis vectors
  ox=ix*basisa(0) + iy*basisa(1) + iz*basisa(2);
  oy=ix*basisb(0) + iy*basisb(1) + iz*basisb(2);
  oz=ix*basisc(0) + iy*basisc(1) + iz*basisc(2);
}


// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::eye2norm()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::eye2norm(float ix,float iy,float iz,float &ox,float &oy,float &oz)
{
  // go from eye coords to normalized perspective coords.
  //  from [Paul Bourke 94] (via internet)
  float d;

  if (projectionmode==perspective)
    {
      d=zoomfactor/iy;
      ox=d*ix/tanthetah;
      oy=iy;
      oz=d*iz/tanthetav;
    }
  else
    {
      ox=zoomfactor*ix/tanthetah;
      oy=iy;
      oz=zoomfactor*iz/tanthetav;
    }
}


// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::norm2screen()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::norm2screen(float ix,float iy,float iz,float &ox,float &oy)
{
  // given the normalized x and z, decide where on screen it should go
  ox=-1.0 * focus_location(0)*ix;
  oy=-1.0 * focus_location(0)*iz;
  //  oy=-1.0 * focus_location(1)*iz;
}



// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::update_notify()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::update_notify(GrobVideoCameraDrawer *drawerp, bool need)
{
  // the 'world' we draw wants us to redraw it
  // note: we may receive this call multiple times between actual draws

  //  cout << "camera " << zoomfactor << " has been noitify updated; refreshcounter=" << refreshcounter << "  refreshrate=" << refreshrate<< ".\n";

  if (!redrawrequested)
    {
      // don't request redraw every moment, only every certain # of update requests
      if (++refreshcounter>=refreshrate || need)
	{
	  //	  cout << "requesting redraw.\n";
	  refreshcounter=0;
	  redundantcounter=0;
	  request_redraw(drawerp);
	  redrawrequested=true;
	}
    }
  else
    {
      // this is a redundant request for a redraw, since our redrawrequested flag is already set.
      // it indicates that we have already requested a redraw from our gui.
      // However, sometimes signals can get lost, so we reset the flag is we get some threshold number of
      //  redundant requests, assuming that the signal has somehow not gotten through.
      //      cout << "redundant request for redraw.\n";
      if (++redundantcounter>10)
	redrawrequested=false;
    }
}


// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::axes_notify()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::axes_notify(GrobVideoCameraDrawer *drawerp, bool need)
{
  // the 'world' we draw has modified our axes so we need to remake them

  // autoadjust focus and scale of camera
  if (autoscale)
    doautoscale(drawerp);
  if (autocenter)
    doautocenter(drawerp);

  // remake new axes and walls
  makeaxes();
  makewalls();

  // NOW we might have a tricky situation here.  it's possible that are axes were changed/rescaled while we were drawing.
  // in this case, what we'd like to do is clear screen, redraw axes, then resume
  if (drawing_engaged)
    {
      // we are in the process of drawing (ie begin_draw() has been called but not end_draw).
      // SO we simply recall begin_draw() which will reinitialize the viewpoint and scaling stuff.
      // we return afterwards, assuming that we don't need to post a new redraw request (ie if drawer partially drew scene
      //  before invoking axes_notify, IT should post an additional redraw request
      begin_draw();
      return;
    }

  // inform camera that it should redraw
  update_notify(drawerp,need);
}




void GrobVideoCamera::request_redraw(GrobVideoCameraDrawer *drawerp)
{
  // the virtual function does nothing. it must be subclassed by the gui manager, etc.
}





// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::setdrawer()
// ----------------------------------------------------------------------------------

GrobVideoCamera::setdrawer(GrobVideoCameraDrawer *ingdrawerp)
{
  // set the variable which tells us who draws our images (who to ask for updates)
  gdrawerp=ingdrawerp;
  gdrawerp->add_vidcam_listener(this);
  // now get the axes from the drawer
  gdrawerp->getpaxes(xaxisp,yaxisp,zaxisp);

  // some context-senstive settings
  if (gdrawerp->drawertype()=="Plot")
    {
      autoscale=true;
      projectionmode=orthogonal;
      // dark gray nice
      clearcolor[0]=clearcolor[1]=clearcolor[2]=.12;
      // white
      //      clearcolor[0]=clearcolor[1]=clearcolor[2]=.5;
    }

  // test
  axes_notify(ingdrawerp,false);
}



// ----------------------------------------------------------------------------------
// -- GrobVideoCamera::setaxes()
// ----------------------------------------------------------------------------------

void GrobVideoCamera::setaxes(VisibleAxis *inxaxisp,VisibleAxis *inyaxisp,VisibleAxis *inzaxisp)
{
  // simple accessor
  xaxisp=inxaxisp;
  yaxisp=inyaxisp;
  zaxisp=inzaxisp;
}





void GrobVideoCamera::settitle(string intit,string insub)
{
  // set the title and mark us as needing redraw
  title=intit;
  subtitle=insub;
  needsredraw=true;
  //  if (gvc_prepared)
  //    maketitle();
}


void GrobVideoCamera::setwinsize(int w,int h,int x,int y)
{
  // the gui should call this to tell us our window pos and size
  win_width=w;
  win_height=h;
  win_xpos=x;
  win_ypos=y;
  viewsx=0;vieww=w;
  viewsy=0;viewh=h;
}





void GrobVideoCamera::show_title()
{
  int len1,len2,sy,y,y2,y3;
  int titx,tity,titw,tith;
  float z=-100;
  float zf=.5;

  if (title=="" && subtitle=="")
    return;
  
  glPushMatrix();
  GrobGlHelper::pushlighting();
  GrobGlHelper::lightsoff();
  
  // set up orthographic coord<->pixel projection
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0,win_width,0,win_height,-1*(fabs(z)+1),fabs(z)+1);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  //  glViewport(0,0,win_width,win_height);
  
  // compute title sizes and locations
  sy=win_height-10;
  y=y2=y3=win_height-24;
  if (title!="")
    {
      len1=GrobGlHelper::glutbitmapwidth(GLUT_BITMAP_HELVETICA_18,title.c_str());
      y2-=15;
      y3-=15;
    }
  else
    len1=0;
  
  if (subtitle!="")
    {
      len2=GrobGlHelper::glutbitmapwidth(GLUT_BITMAP_HELVETICA_12,subtitle.c_str());
      y3-=15;
    }
  else
    len2=0;
  
  // compute size of title border
  if (len1>len2)
    titw=len1;
  else
    titw=len2;
  tith=y-y3;
  titx=(win_width-titw)/2;
  tity=sy;
  
  // draw a title border box?
  titw+=40;titx-=20;
  tith+=10;tity+=5;
  titx=5-1;titw=win_width-10;
  GrobGlHelper::drawBackbox(titx,tity,z,titw,tith);
  
  // adjust drawing viewport
  viewsx=titx;
  viewsy=tity+tith+4;
  vieww=titw;
  viewh=win_height-viewsy;
  
  // draw titles
  if (true)
    {
      // bitmap version
      if (title!="")
	GrobGlHelper::drawBitmapText(GLUT_BITMAP_HELVETICA_18,(win_width-len1)/2,y,-z+zf,1,title.c_str());
      if (subtitle!="")
      	GrobGlHelper::drawBitmapText(GLUT_BITMAP_HELVETICA_12,(win_width-len2)/2,y2,-z+zf,1,subtitle.c_str());
    }
  else
    {
      // stroke version
      if (title!="")
	GrobGlHelper::drawStrokeText(GLUT_STROKE_ROMAN,(win_width-len1)/2,y,-z+zf,.12,title.c_str());
      if (subtitle!="")
	GrobGlHelper::drawStrokeText(GLUT_STROKE_ROMAN,(win_width-len2)/2,y2,-z+zf,.08,subtitle.c_str());
    }

  GrobGlHelper::poplighting();
  glPopMatrix();
}














void GrobVideoCamera::doautocenter(GrobVideoCameraDrawer *drawerp)
{
  // adjust focus of camera based on center of axes
  float ox,oy,oz;
  float ofx,ofy,ofz;
  float cx,cy,cz;
  float sloc,iloc,eloc;

  ox=location(0);  oy=location(1);  oz=location(2);
  cx=ofx=focus_location(0);  cy=ofy=focus_location(1);  cz=ofz=focus_location(2);
  cx=0;cy=0;cz=0;

  if (xaxisp)
    {
      xaxisp->getaxispoints(sloc,iloc,eloc);
      cx=(eloc+sloc)/2.0;
    }
  if (yaxisp)
    {
      yaxisp->getaxispoints(sloc,iloc,eloc);
      cy=(eloc+sloc)/2.0;
    }
  if (zaxisp)
    {
      zaxisp->getaxispoints(sloc,iloc,eloc);
      cz=(eloc+sloc)/2.0;
    }

  // set new focus
  setfocus(cx,cy,cz);

  // move camera parallel (will be ignored if we are in distance_lock mode)
  //  location(0)=ox+(cx-ofx);
  //  location(1)=oy+(cy-ofy);
  //  location(2)=oz+(cz-ofz);
}



void GrobVideoCamera::doautoscale(GrobVideoCameraDrawer *drawerp)
{
  // adjust scale of axes so the volume appears unit length

  if (xaxisp)
    cxscale=xaxisp->getscale()*xskew;
  if (yaxisp)
    cyscale=yaxisp->getscale()*yskew;
  if (zaxisp)
    czscale=zaxisp->getscale()*zskew;

  //  cout << "cxscale="<<cxscale<< "  cyscale="<<cyscale<< "  czscale="<<czscale<<".\n";
}



void GrobVideoCamera::setlinewidth(int inlinewidth)
{
  if (inlinewidth>0)
    linewidth=inlinewidth;
  glLineWidth(linewidth);
}

void GrobVideoCamera::biglinewidth(int inc)
{
  glLineWidth(linewidth+inc);
}











































// ----------------------------------------------------------------------------------
// -- GrobVideoCameraDrawer
// ----------------------------------------------------------------------------------


GrobVideoCameraDrawer::GrobVideoCameraDrawer()
{
  // constructor
  listeners=0;
  gbegin=gvc_listeners.begin();
  gend=gvc_listeners.end();
  xaxisp=yaxisp=zaxisp=NULL;
}


void GrobVideoCameraDrawer::add_vidcam_listener(GrobVideoCamera *newcam)
{
  // add a videocamera listener to our list of people to inform when we have updated info
  gvc_listeners.push_back(newcam);
  gbegin=gvc_listeners.begin();
  gend=gvc_listeners.end();
  ++listeners;
}


void GrobVideoCameraDrawer::cameranotify()
{
  // notify all of our listeners that we have new stuff to draw, so that they may invoke
  // us, at their leisure, to display this stuff.

  if (!listeners)
    return;

  gi=gbegin;
  while (gi!=gend)
    {
      (*gi)->update_notify(this);
      gi++;
    }
}


void GrobVideoCameraDrawer::cameranotify_axes()
{
  // notify all of our listeners that we have changed the axes

  if (!listeners)
    return;

  gi=gbegin;
  while (gi!=gend)
    {
      (*gi)->axes_notify(this);
      gi++;
    }
}




void GrobVideoCameraDrawer::getpaxes(VisibleAxis* &cxaxisp,VisibleAxis* &cyaxisp,VisibleAxis* &czaxisp)
{
  // set the axis pointers for a camera, using drawer's axes
  cxaxisp=xaxisp;
  cyaxisp=yaxisp;
  czaxisp=zaxisp;
}












void GrobVideoCamera::setview(int viewnum)
{
  // restore a preset view
  float ydolly;
  if (viewnum>views.size())
    return;
  
  views[viewnum]->restore(&euler1,&euler2,&euler3,&zoomfactor,&ydolly,&projectionmode);
  setydolly(ydolly);
  needsrecalc=needsredraw=true;
}

int GrobVideoCamera::saveview()
{
  // save the current view
  CameraView *cvp;

  cvp=new CameraView(euler1,euler2,euler2,zoomfactor,projectionmode);
  return (addview(cvp));
}

int GrobVideoCamera::addview(CameraView *cviewp)
{
  views.push_back(cviewp);
  return (views.size()-1);
}

void GrobVideoCamera::add_defaultviews()
{
  CameraView *cvp;

  // (standard-steep, xyz)
  addview(new CameraView(-2.320,1.712,0.0));
  // (standard, nice, xyz)
  addview(new CameraView(-2.320,2.25,0.0));

  //  // (standard-shallow, xyz)
  //  addview(new CameraView(-2.320,2.9,0.0));

  // good 3d z axis viewing (shallow)
  addview(new CameraView(-1.944,2.480,0.0));

  // (xy orthographic)
  addview(new CameraView(-1*M_PI2,M_PI2,0.0));

  // (xz orthographic)
  //  addview(new CameraView(-1*M_PI2,2.4,0.0));
  //  addview(new CameraView(-1*M_PI2,M_PI,0.0));

  // (yz orthographic)
  //  addview(new CameraView(M_PI,2.4,0.0));
  //  addview(new CameraView(M_PI,M_PI,0.0));

  curview=1;
}







// }
// Namespace close-brace
