// inline grob functions for fast access


#ifndef INC_grobinlineh
#define INC_grobinlineh



#include "../../cfplibs/biobot/misc.h"

// ----------------------------------------------------------------------------------
// -- Grob::draw()
// ----------------------------------------------------------------------------------

inline void Grob::draw(GrobVideoCamera *cam)
{
  // figure out how to draw to the particular camera we are passed
  draw_gl();
}




// ----------------------------------------------------------------------------------
// -- Grob::makematrix()
// ----------------------------------------------------------------------------------

inline void Grob::makematrix()
{
  // set orientation
  olmat[0]=orientation(0,0);
  olmat[1]=orientation(1,0);
  olmat[2]=orientation(2,0);
  olmat[4]=orientation(0,1);
  olmat[5]=orientation(1,1);
  olmat[6]=orientation(2,1);
  olmat[8]=orientation(0,2);
  olmat[9]=orientation(1,2);
  olmat[10]=orientation(2,2);

  // set location
  olmat[12]=location(0);
  olmat[13]=location(1);
  olmat[14]=location(2);

  // scale?
  olmat[15]=1;
}


// ----------------------------------------------------------------------------------
// -- Grob::setlocation()
// ----------------------------------------------------------------------------------

inline void Grob::setlocation(perVectorR3 inlocation)
{
  // set location
  location=inlocation;

  // set update flags
  gl_needsupdate=true;
}


// ----------------------------------------------------------------------------------
// -- Grob::setorientation()
// ----------------------------------------------------------------------------------

inline void Grob::setorientation(perMatrix inorientation)
{
  // set orientation
  orientation=inorientation;

  // set update flags
  gl_needsupdate=true;
}


inline void Grob::readlocation(FILE *fp)
{
  Jvecload(location,fp,3);
  gl_needsupdate=true;
}
inline void Grob::writelocation(FILE *fp)
{
  Jvecsave(location,fp,3);
}
inline void Grob::readorientation(FILE *fp)
{
  Jmatload(orientation,fp,3,3);
}
inline void Grob::writeorientation(FILE *fp)
{
  Jmatsave(orientation,fp,3,3);
}





// ----------------------------------------------------------------------------------
// -- Grob::setlocation()
// ----------------------------------------------------------------------------------

inline void Grob::setlocation(float inloc[3])
{
  // set location
  location(0)=inloc[0];
  location(1)=inloc[1];
  location(2)=inloc[2];

  // set update flags
  gl_needsupdate=true;
}



// ----------------------------------------------------------------------------------
// -- Grob::gl_update()
// ----------------------------------------------------------------------------------

inline void Grob::gl_update()
{
  // orientation, location, and/or scale has changed
  makematrix();

  // clear the update flag
  gl_needsupdate=false;
}





// ----------------------------------------------------------------------------------
// -- Grob::draw_gl()
// ----------------------------------------------------------------------------------

inline void Grob::draw_gl()
{
  // this is simple, we assume that gl is setup and ready for us
  #ifdef GrobGL
  if (gl_currentnum==-1)
    return;
  else if (gl_needsupdate)
    gl_update();
  glPushMatrix();
  glMultMatrixf(&olmat[0]);
  glScalef(oscale[0],oscale[1],oscale[2]);
  glCallList(gl_currentnum);
  glPopMatrix();
  #endif
}




inline void GrobGlHelper::drawLine(double sx,double sy,double sz,double ex,double ey,double ez)
{
  // immediately draw a line

  pushlighting();
  lightsoff();

  glBegin(GL_LINES);
  glColor3f(7.0,.3,.4);
  glVertex3f(sx, sy, sz);
  glVertex3f(ex, ey, ez);
  glEnd();

  poplighting();
}






#endif
